/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XMLChBuilder.hpp"
#include <xercesc/util/Janitor.hpp>

using namespace std;

XMLChBuilder::XMLChBuilder()
  : fragments(),
    ownedBlocks()
{
  fragments.reserve(16);
  ownedBlocks.reserve(16);
}

XMLChBuilder::~XMLChBuilder()
{
  for(StringVector::const_iterator blockI = ownedBlocks.begin();
      blockI != ownedBlocks.end();
      ++blockI)
  {
    delete[] (XMLCh *)*blockI;
  }
}

void XMLChBuilder::append(const XMLCh* string)
{
  if(string && *string)
  {
    fragments.push_back(string);
  }
}

void XMLChBuilder::appendAndDelete(const XMLCh* string)
{
  if(string)
  {
    ownedBlocks.push_back(string);
    if(*string)
    {
      fragments.push_back(string);
    }
  }
}

void XMLChBuilder::appendLiteral(const char* string)
{
  appendAndDelete(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(string));
}

XMLCh* XMLChBuilder::fetch()
{
  unsigned int numBlocks = fragments.size();
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<unsigned int> lengths(new unsigned int[numBlocks]);

    // Get the lengths of the strings
    unsigned int totalLength = 0;
    unsigned int i;
    for(i = 0; i<numBlocks; ++i)
    {
      totalLength+=lengths[i]=XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(fragments[i]);
    }

    // Build the resultant string
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> result(new XMLCh[totalLength+1]);
  XMLCh* nextPosition = result.get();
    for(i = 0; i<numBlocks; ++i)
    {
      XERCES_CPP_NAMESPACE_QUALIFIER XMLString::moveChars(nextPosition,fragments[i],lengths[i]);
      nextPosition+=lengths[i];
    }
    *nextPosition=0;

  // Ownership transfers to caller
  return result.release();
}
