/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XMLChStringHash.hpp"

XMLChStringHash::XMLChStringHash()
{
  for(int i = 0; i < XMLChStringHash_hashSize; i++) {
    _data[i] = vector< pair<const XMLCh *, VarHashEntry *> >();
  }
}

XMLChStringHash::~XMLChStringHash()
{
  for(int j = 0; j < XMLChStringHash_hashSize; ++j) {

    vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator i = _data[j].begin();

    for(; i != _data[j].end(); ++i) {
      delete (XMLCh*)(i->first);
      delete i->second;
    }
  }
  /* nothing */
}

void XMLChStringHash::set(XMLCh *key, VarHashEntry * value)
{
  vector< pair<const XMLCh *, VarHashEntry *> >::iterator index = fnd(key);

  if(index != _data[hsh(key)].end()) {
    delete (XMLCh*)(index->first);
    delete index->second;
    index->first = key;
    index->second = value;
  }
  else {
    _data[hsh(key)].push_back(pair<const XMLCh *, VarHashEntry *>(key, value));
  }
}

pair<bool, VarHashEntry *> XMLChStringHash::find(const XMLCh *key) const
{
  vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator index = fnd(key);

  if(index == _data[hsh(key)].end()) {

    return pair<bool, VarHashEntry *>(false, 0);
  }

  return pair<bool, VarHashEntry *>(true, index->second);
}

void XMLChStringHash::erase( const XMLCh *key )
{

  vector< pair<const XMLCh *, VarHashEntry *> >::iterator index = fnd(key);

  if(index != _data[hsh(key)].end()) {
    delete (XMLCh*)(index->first);
    delete index->second;
    _data[hsh(key)].erase(index);
  }

}

unsigned int XMLChStringHash::size() const
{
  unsigned int size = 0;
  for(int i = 0; i < XMLChStringHash_hashSize; i++) {
    size += _data[i].size();
  }
  return size;
}

const int XMLChStringHash::hsh( const XMLCh *key ) const
{
  unsigned long checksum = 0;
  while(*key)
  {
    checksum += *key++;
  }

  return checksum % XMLChStringHash_hashSize;
}

vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator
XMLChStringHash::fnd(const XMLCh *key) const
{
  int h = hsh(key);
  vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator i = _data[h].begin();
  for (; i != _data[h].end(); ++i) {
    if( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(i->first,key)==0 ) {
      return i;
    }
  }

  return i;
}

vector< pair<const XMLCh *, VarHashEntry *> >::iterator
XMLChStringHash::fnd(const XMLCh *key)
{
  int h = hsh(key);
  vector< pair<const XMLCh *, VarHashEntry *> >::iterator i = _data[h].begin();
  for (; i != _data[h].end(); i++) {
    if( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(i->first,key)==0 ) {
      return i;
    }
  }

  return i;
}

XMLCh *XMLChStringHash::print() const
{
  XMLChBuilder result;
  Context context;

  for(int j = 0; j < XMLChStringHash_hashSize; ++j) {

    vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator i =
      _data[j].begin();

    for(; i != _data[j].end(); ++i) {
      result.appendLiteral("<Variable name=\"");
      result.append(i->first);
      result.appendLiteral("\">");
      result.append(i->second->getValue()->asString(&context));
      result.appendLiteral("</Variable>\n");
    }
  }

  return result.fetch();
}
