/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  XMLChStringHash - A hash table class for XMLChs and VarHashEntry

*/

#ifndef _XMLCHSTRINGHASH_HPP
#define _XMLCHSTRINGHASH_HPP

#include <config.h>


#ifdef WIN32
#pragma warning( disable: 4786 )
#endif

#include <vector>

#include "../DOMutils/XMLChBuilder.hpp"
#include "../context/Context.hpp"
#include "../simpleVariables/VarHashEntry.hpp"

#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOM.hpp>

#define XMLChStringHash_hashSize 23

using namespace std;

/** A templated hash table class for XMLChs. */
class /*PATHAN_EXPORT*/ XMLChStringHash
#if 0
  : public TestCase
#endif
{
public:
  /// constructor
  XMLChStringHash();
  /// destructor
  ~XMLChStringHash();

  /** Add or change an item in the hash table */
  void set(XMLCh *key, VarHashEntry * value);
  /** Find and return the item hashed with key */
  pair<bool, VarHashEntry *> find(const XMLCh *key) const;
  /** Erase the item hashed with key */
  void erase(const XMLCh *key);

  /// Returns the size of the hash table.
  unsigned int size() const;

  // caller owns
  XMLCh *print() const;

private:
  const int hsh(const XMLCh *key) const;
  vector< pair<const XMLCh *, VarHashEntry *> >::iterator fnd(const XMLCh *);
  vector< pair<const XMLCh *, VarHashEntry *> >::const_iterator fnd(const XMLCh *) const;
  vector< pair<const XMLCh *, VarHashEntry *> > _data[ XMLChStringHash_hashSize ];
};
#endif // _XMLCHSTRINGHASH_HPP
