/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _V2NSUTILS_HPP
#define _V2NSUTILS_HPP

#include <config.h>

#ifdef WIN32
#pragma warning(disable: 4786)
#endif

#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/util/XMLString.hpp>
#include <map>

#ifdef TEST
#include "../testharness/Test.h"
#include "../testharness/TestSuite.h"
#include "../testharness/TestCase.h"
#include "../testharness/TestCaller.h"
#endif

class Context;

namespace PathanInternalUse {
	//Used as a comparator in the namespace maps
	struct compareDOMStrings
	{
	  bool operator()(const XMLCh *s1, const XMLCh *s2) const
	  {
	    return XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(s1,s2) < 0;
	  }
	};//compareDOMStrings
}//PathanInternalUse

typedef std::map<const XMLCh *, const XMLCh *, PathanInternalUse::compareDOMStrings> namespaceMapType;


class PATHAN_EXPORT v2NSUtils
{
public:
  /// Get the proper local name (not null if the node is DOM level 1...)
  static const XMLCh *getLocalName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);
  static const XMLCh *getLocalName(const XMLCh *name);

  ///returns the prefix part of a xs:foo string or a zero length string if no prefix
  static const XMLCh *getPrefix(const XMLCh *name);
  ///does it's best to work out a prefix for the given node. Returns empty string if no prefix.
  static const XMLCh *getPrefix(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node,
                                const Context *context);

  ///returns the element node that is the parent or owner of node
  static XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *returnOwnerElement(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  ///form a qualified name from a namespace and a name
  static XMLCh *qualifyName(const XMLCh *prefix, const XMLCh *name);
  /**Gives you a QName for the node given. Namespace resolving is done via
     the context */
  static XMLCh *qualifyName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node,
                                  const Context *context);

  static XMLCh *getExpandedName(const XMLCh *namespaceURI,
                                      const XMLCh *name);

  ///gets the owner element of an attribute, and the parent of any other node
  static XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *getParent(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  //TODO: Comment
  static namespaceMapType *createNamespaceMap(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode);
};

#endif

