/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "v2Utils.hpp"
#include "v2NSUtils.hpp"
#include "../defs/NumUtils.hpp"
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/Janitor.hpp>
#include <iostream>
#include <assert.h>
#include <sstream>
#include "../exceptions/DataItemException.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "../defs/XMLScriptTypeDefs.hpp"
#include "../dom3-xpath/XPathNamespace.hpp"
#include "../DOMutils/XMLChBuilder.hpp"
#ifdef WIN32
#define snprintf _snprintf
#endif

// "element"
static const XMLCh textElement[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "attribute"
static const XMLCh textAttribute[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_b, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e,
  XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "text"
static const XMLCh textText[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_x, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "cdata_section"
static const XMLCh textCdataSection[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chUnderscore,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "entity_reference"
static const XMLCh textEntityReference[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_y, XERCES_CPP_NAMESPACE_QUALIFIER chUnderscore,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_f, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e,
  XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "entity"
static const XMLCh textEntity[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_y, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "processing_instruction"
static const XMLCh textProcessingInstruction[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chUnderscore,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "comment"
static const XMLCh textComment[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "document"
static const XMLCh textDocument[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "document_type"
static const XMLCh textDocumentType[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t,
  XERCES_CPP_NAMESPACE_QUALIFIER chUnderscore, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_y, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "document_fragment"
static const XMLCh textDocumentFragment[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t,
  XERCES_CPP_NAMESPACE_QUALIFIER chUnderscore,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_f, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "notation"
static const XMLCh textNotation[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "unknown"
static const XMLCh textUnknown[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_k, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "null node"
static const XMLCh textNullNode[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chSpace,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

bool v2Utils::asciiOutput = false;

void v2Utils::setAsciiOutput(bool doAO)
{
  asciiOutput = doAO;
}//setAsciiOutput

bool v2Utils::getDoAsciiOuput(void)
{
  return asciiOutput;
}//getDoAsciiOutput


string v2Utils::DSLtranscode(const XMLCh *DOMstr)
{
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<char> cStr(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(DOMstr));
  return cStr.get();
}


const XMLCh *v2Utils::getNodeTypeAsString(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
    if(node!=NULL) {
      switch(node->getNodeType()){
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE:{
	return textElement;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:{
	return textAttribute;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:{
	return textText;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE:{
	return textCdataSection;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE:{
	return textEntityReference;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_NODE:{
	return textEntity;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE:{
	return textProcessingInstruction;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE:{
	return textComment;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE:{
	return textDocument;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_TYPE_NODE:{
	return textDocumentType;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_FRAGMENT_NODE:{
	return textDocumentFragment;
      }
      case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::NOTATION_NODE:{
	return textNotation;
      }
      default:{
	return textUnknown;
      }
      }
    }
    else {
      return textNullNode;
    }
}

XMLCh *v2Utils::getStringFromNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  if(node!=NULL)
  {
  switch(node->getNodeType()) {

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE:

    return XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(node->getNodeValue());

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_FRAGMENT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {

    // FIXME: This recursive approach is very expensive near the top of a tree
    XMLChBuilder textcontent;
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getFirstChild();
    while(child!=NULL) {
      textcontent.appendAndDelete(v2Utils::getStringFromNode(child));
      child = child->getNextSibling();
    }

    //dont know if this constructor works but it seems to??
    /*    DOM_NodeIterator i = node.getOwnerDocument().
      createNodeIterator(node, DOM_NodeFilter::SHOW_ALL, 0, true);

    const XMLCh *textcontent = "";
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *next = i.nextNode();
    while(next!=NULL) {
      textcontent = appendXMLCh(textcontent, next->getNodeValue());
      next = i.nextNode();
      }*/

    return textcontent.fetch();
  }

  default:
    throw(DataItemException("v2utils::getStringFromNode", "Unknown node type"));
  }
}
  else
  {
    throw(DataItemException("v2utils::getStringFromNode", "Null node pointer"));
  }
  }

XSFloat v2Utils::getDoubleFromNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> text(getStringFromNode(node));
  return NumUtils::stringAsDouble(text.get());
}

XMLCh *v2Utils::printNodeToString(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  // Get the name and value out for convenience
  //  DOMString nodeNamespaceURI = toWrite.getNamespaceURI();
  //  DOMString nodePrefix = toWrite.getPrefix();
  //  DOMString nodeLocalName = toWrite.getLocalName();
  const XMLCh *nodeName = node->getNodeName();
  const XMLCh *nodeValue = node->getNodeValue();
  XMLChBuilder result;

  switch (node->getNodeType()) {
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
    // Output the element start tag.
    result.appendLiteral("<");
    result.append(nodeName);

    // Output any attributes on this element
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attributes = node->getAttributes();
    int attrCount = attributes->getLength();
    for (int i = 0; i < attrCount; i++) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *attribute = attributes->item(i);

      result.appendLiteral(" ");
      result.append(attribute->getNodeName());
      result.appendLiteral("=\"");
      //  Note that "<" must be escaped in attribute values.
      result.append(attribute->getNodeValue());
      result.appendLiteral("\"");
    }

    //  There were no children.  Output the close of the
    //  empty-element start tag.
    result.appendLiteral("/>");
    break;
  }
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE: {
    result.append(nodeValue);
    break;
  }
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child;

    for (child = node->getFirstChild(); child != 0; child = child->getNextSibling()){
      result.appendAndDelete(printNodeToString(child));
      result.appendLiteral(" ");
    }
    break;
  }
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE : {
    result.appendLiteral("<?");
    result.append(nodeName);
    result.appendLiteral(" ");
    result.append(nodeValue);
    result.appendLiteral("?>");
    break;
  }
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE: {
    result.appendLiteral("<![CDATA[");
    result.append(nodeValue);
    result.appendLiteral("]]>");
    break;
  }
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE: {
    result.appendLiteral("<!--");
    result.append(nodeValue);
    result.appendLiteral("-->");
    break;
  }
  default:
    result.appendLiteral("Unrecognized node type");
  } // end of switch

  return result.fetch();
}

bool v2Utils::compareNodeLists(const DeadNodeList *myNodes,
                               const DeadNodeList *otherNodes)
{
  bool found = false;

    // FIXME: This really isn't efficient!
  for(DeadNodeList::const_iterator myNode = myNodes->begin();
        !found && myNode != myNodes->end(); myNode++) {

    XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> myString(v2Utils::getStringFromNode(*myNode));

    for(DeadNodeList::const_iterator otherNode = otherNodes->begin();
          !found && otherNode != otherNodes->end(); otherNode++) {

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*otherNode));

      if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(myString.get(), otherString.get())==0) {
        found = true;
      }
    }
  }

  return found;
}

bool v2Utils::compareNodeListsLessThan(const DeadNodeList *myNodes,
                                       const DeadNodeList *otherNodes)
{

  if((myNodes->getLength() == 0) || (otherNodes->getLength() == 0)) {

    cerr << "no nodes";
    return false;
  }
  else {
    XSFloat maxInThis = getDoubleFromNode(*myNodes->begin());
    XSFloat minInOther = getDoubleFromNode(*otherNodes->begin());

    for(DeadNodeList::const_iterator myNode = myNodes->begin();
	myNode != myNodes->end(); myNode++) {
      XSFloat temp = getDoubleFromNode(*myNode);
      if(!NumUtils::isNaN(temp)) {
	if(!NumUtils::isGreaterThan(maxInThis, temp)) {
	  maxInThis = temp;
	}
      }
    }

    for(DeadNodeList::const_iterator otherNode = otherNodes->begin();
	otherNode != otherNodes->end(); otherNode++) {
      XSFloat temp = getDoubleFromNode(*otherNode);
      if(!NumUtils::isNaN(temp)) {
	if(!NumUtils::isLessThan(minInOther, temp)) {
	  minInOther = temp;
	}
      }
    }


    return NumUtils::isLessThan(maxInThis, minInOther);
  }
}

bool v2Utils::compareNodeListsGreaterThan(const DeadNodeList *myNodes,
					  const DeadNodeList *otherNodes)
{

  if((myNodes->getLength() == 0) || (otherNodes->getLength() == 0)) {

    return false;
  }
  else {
    XSFloat minInThis = getDoubleFromNode(*myNodes->begin());
    XSFloat maxInOther = getDoubleFromNode(*otherNodes->begin());

    for(DeadNodeList::const_iterator myNode = myNodes->begin();
	myNode != myNodes->end(); myNode++) {
      XSFloat temp = getDoubleFromNode(*myNode);
      if(!NumUtils::isNaN(temp)) {
	if(!NumUtils::isLessThan(minInThis, temp)) {
	  minInThis = temp;
	}
      }
    }

    for(DeadNodeList::const_iterator otherNode = otherNodes->begin();
	otherNode != otherNodes->end(); otherNode++) {
      XSFloat temp = getDoubleFromNode(*otherNode);
      if(!NumUtils::isNaN(temp)) {
	if(!NumUtils::isGreaterThan(maxInOther, temp)) {
	  maxInOther = temp;
	}
      }
    }


    return NumUtils::isGreaterThan(minInThis, maxInOther);
  }
}

int v2Utils::indexNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *indexNode)
{

  int index = -1;

  switch(indexNode->getNodeType()) {

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE:
    index = 1;
    break;

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
    const XMLCh *name = v2NSUtils::getLocalName( indexNode );
    const XMLCh *nspace = indexNode->getNamespaceURI();
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent = indexNode->getParentNode();

    if(parent==NULL) {
      return 1; // if the node nas no parent, it's index is one.
    }

    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = parent->getFirstChild();
    assert(child != 0);
    index = 1;

    while(child != indexNode) {
      if(child->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE
	 && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(name,  v2NSUtils::getLocalName( child ) )==0
	 && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nspace,  child->getNamespaceURI() )==0) {
	index++;
      }
      child = child->getNextSibling();
      assert(child != 0);
    }
    break;
  }

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE: {
    short type = indexNode->getNodeType();
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent = indexNode->getParentNode();
    if(parent==NULL) {
      return 1; // if a node has no parent, it's index is 1
    }

    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = parent->getFirstChild();
    assert(child != 0);
    index = 1;

    while(child != indexNode) {
      if(child->getNodeType() == type) {
	index++;
      }
      child = child->getNextSibling();
      assert(child != 0);
    }
    break;
  }
  default:
    assert("n" == "Unhandled node type in v2Utils::indexNode");
    throw DSLException("Unhandled node type in v2Utils::indexNode");

  }


  return index;
}

int v2Utils::indexNode(const DeadNodeList *nodeList)
{

 int index;

 if(nodeList->getLength() == 0) {
   index = 0;
 }
 else {
   XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *indexNode = nodeList->item(0);
   index = v2Utils::indexNode( indexNode );
 }


 return index;
}

int v2Utils::getMaxNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *candidate)
{

  int index = 0;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent = candidate->getParentNode();
  if(parent==NULL) {
    return 1; // no parent, no siblings
  }

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *current = parent->getFirstChild();
  const XMLCh *name = candidate->getNodeName();

  while (current != 0) {
    switch(current->getNodeType()) {

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
       if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(current->getNodeName(),name)==0) {
	 index++;
       }
       break;
    }

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:
    default: {
      break;
    }
    }

    current = current->getNextSibling();
  }

  return index;
}


bool v2Utils::isAncestor(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child)
{

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *cParent = child->getParentNode();

  while(cParent != 0) {
    if(cParent == parent) {

      return true;
    }
    cParent = cParent->getParentNode();
  }


  return false;
}

string v2Utils::UnicodeToEntity (unsigned int inUnicode)
{
	ostringstream ost;

	if (inUnicode < 0x80) {
		//Just return the character
		ost << (char)(inUnicode & 0x7f);
	} else {
		ost << "&#x";

		int length = 8;
		if (inUnicode < 0x100)
			length = 2;
		else if (inUnicode < 0x10000)
			length = 4;
		else if (inUnicode < 0x1000000)
			length = 6;
		else if (inUnicode <= 0x7fffffff)
			length = 8;

		ost.width(length);
		ost.fill('0');
		ost << hex << inUnicode << ";";
	}//if

	ost << ends;

  return ost.str();
}//UnicodeToEntity



//Convert a 32 bit Unicode character into UTF-8
//See RFC2279 and RFC2044 for details on how this works
string v2Utils::UnicodeToUTF8 (unsigned int inUnicode)
{
  int size;         //number of bytes in encoded string
  char tmpChar[7];  //temporary buffer holding the string

  if (inUnicode < 0x80)
    size = 1;
  else if (inUnicode < 0x800)
    size = 2;
  else if (inUnicode < 0x10000)
    size = 3;
  else if (inUnicode < 0x200000)
    size = 4;
  else if (inUnicode < 0x4000000)
    size = 5;
  else if (inUnicode <= 0x7fffffff)
    size = 6;
  else
    return 0;

  switch (size) {
    case 6:
      tmpChar[5] = 0x80 | (inUnicode & 0x3f);
      inUnicode = inUnicode >> 6;
      inUnicode |= 0x4000000;
      //fall through
    case 5:
      tmpChar[4] = 0x80 | (inUnicode & 0x3f);
      inUnicode = inUnicode >> 6;
      inUnicode |= 0x200000;
      //fall through
    case 4:
      tmpChar[3] = 0x80 | (inUnicode & 0x3f);
      inUnicode = inUnicode >> 6;
      inUnicode |= 0x10000;
      //fall through
    case 3:
      tmpChar[2] = 0x80 | (inUnicode & 0x3f);
      inUnicode = inUnicode >> 6;
      inUnicode |= 0x800;
      //fall through
    case 2:
      tmpChar[1] = 0x80 | (inUnicode & 0x3f);
      inUnicode = inUnicode >> 6;
      inUnicode |= 0xc0;
      //fall through
    case 1:
      tmpChar[0] = inUnicode;
  }

  tmpChar[size]='\0';

  return tmpChar;
}//UnicodeToUTF8

//Convert a string into a UTF-8 encoded version
string v2Utils::StringToUTF8(const XMLCh *instr)
{
  string retString; //the string to return

  //Get an handle on the raw 32bit data, and encode each character
  const XMLCh *tmpbuf = instr;
  for(unsigned int cnt=0; cnt<XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(instr); cnt++) {
	  if(asciiOutput == true) {
		  retString.append(UnicodeToEntity(tmpbuf[cnt]));
	  } else {
		retString.append(UnicodeToUTF8(tmpbuf[cnt]));
	  }//if
  }//for

  return retString;
}//stringToUTF8

//Return the actual Unicode character index of the byte pointed to by 'offset'. -1 is returned in case of error.
int v2Utils::UnicodeIndexOfUTF8(const char *origString, const char *offset)
{
  int retVal = 0;
  const char *tmpPos = origString;

  //Do nothing on a NULL string
  if(origString==NULL) {
    return -1;
  }//if

  //jump along each character until we hit (or pass) the one we want
  while(tmpPos < offset) {
    //how many bytes do we jump
    int bytesToJump;
    char tmpchar = (*tmpPos);

    //this is horrible -cam
    if(!(tmpchar & 0x80)) {
      bytesToJump = 1;
    } else {
      if(!(tmpchar & 0x20)) {
        bytesToJump = 2;
      } else {
        if(!(tmpchar & 0x10)) {
          bytesToJump = 3;
        } else {
          if(!(tmpchar & 0x08)) {
            bytesToJump = 4;
          } else {
            if(!(tmpchar & 0x04)) {
              bytesToJump = 5;
            } else {
              if(!(tmpchar & 0x02)) {
                bytesToJump = 6;
              } else {
                //This should never happen
                return -1;
              }//if
            }//if
          }//if
        }//if
      }//if
    }//if

    retVal++;
    tmpPos += bytesToJump;
  }//while

  return retVal;
}//UnicodeIndexOfUTF8


//Get pointer into resultUTF's buffer, starting at Unicode character matchStartIndex
string v2Utils::UTF8StringAtUnicodeIndex(const char *origString, unsigned int index)
{
  unsigned int position = 0;
  const char *tmpPos = origString;

  //Do nothing on a NULL string
  if(origString==NULL) {
    return "";
  }//if

  //jump along each character until we hit (or pass) the one we want
  while(position < index) {
    //how many bytes do we jump
    int bytesToJump;
    char tmpchar = (*tmpPos);

    //this is horrible -cam
    if(!(tmpchar & 0x80)) {
      bytesToJump = 1;
    } else {
      if(!(tmpchar & 0x20)) {
        bytesToJump = 2;
      } else {
        if(!(tmpchar & 0x10)) {
          bytesToJump = 3;
        } else {
          if(!(tmpchar & 0x08)) {
            bytesToJump = 4;
          } else {
            if(!(tmpchar & 0x04)) {
              bytesToJump = 5;
            } else {
              if(!(tmpchar & 0x02)) {
                bytesToJump = 6;
              } else {
                //This should never happen
                return "";
              }//if
            }//if
          }//if
        }//if
      }//if
    }//if

    position++;
    tmpPos += bytesToJump;
  }//while

  if(tmpPos != NULL) {
    return tmpPos;
  } else {
    return "";
  }//if
}//UTF8StringAtUnicodeIndex


void v2Utils::escapeNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  switch (node->getNodeType ()) {

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getFirstChild ();
    while(child != 0) {
      escapeNode(child);
      child = child->getNextSibling();
    }//while
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
    // Escape any attributes on this element
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attributes = node->getAttributes();
    int attrCount = attributes->getLength();
    for(int i = 0; i < attrCount; i++) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *attribute = attributes->item(i);
      escapeNode(attribute);
    }//for

    if(node->hasChildNodes()) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getFirstChild();
      while (child != 0) {
	escapeNode(child);
	child = child->getNextSibling();
      }//while
    }
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> value(escapeString(node->getNodeValue(), true));
    node->setNodeValue(value.get());
    break;
  }

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> value(escapeString(node->getNodeValue(), true));
    node->setNodeValue(value.get());
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child;
    for(child = node->getFirstChild(); child != 0; child = child->getNextSibling())
      escapeNode(child);
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_FRAGMENT_NODE: {
    if (node->hasChildNodes()) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getFirstChild();
      while (child != 0) {
	escapeNode(child);
	child = child->getNextSibling();
      }//while
    }//if
    break;
  }//case

  default: {
    break;
  }
  }//switch


}

XMLCh *v2Utils::escapeString(const XMLCh *str, bool escapeQuot)
{
  unsigned int bufferSize = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(str) * 2 + 16;

  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> buffer(new XMLCh[bufferSize]);

  unsigned int index = 0;
  while(*str!=XERCES_CPP_NAMESPACE_QUALIFIER chNull)
  {
    switch(*str) {
      case XERCES_CPP_NAMESPACE_QUALIFIER chAmpersand:
        // "&amp;"
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chAmpersand;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chSemiColon;
      break;

      case XERCES_CPP_NAMESPACE_QUALIFIER chOpenAngle:
        // "&lt;"
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chAmpersand;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chSemiColon;
      break;

      case XERCES_CPP_NAMESPACE_QUALIFIER chCloseAngle:
        // "&gt;"
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chAmpersand;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t;
        buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chSemiColon;
      break;

      case XERCES_CPP_NAMESPACE_QUALIFIER chDoubleQuote:
      if(escapeQuot) {
          // "&quot;"
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chAmpersand;
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_q;
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u;
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o;
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t;
          buffer[index++]=XERCES_CPP_NAMESPACE_QUALIFIER chSemiColon;
      }
        else
        {
          buffer[index++]=*str;
        }
        break;

    default:
        buffer[index++]=*str;
      break;
    }

    if(index+8>bufferSize)
    {
      bufferSize*=2;
      XMLCh* temp = new XMLCh[bufferSize];
      XERCES_CPP_NAMESPACE_QUALIFIER XMLString::moveChars(temp,buffer.get(),index);
      buffer.reset(temp);
  }

    ++str;
}
  buffer[index++]=0;

  return buffer.release();
}

XMLCh *v2Utils::serialiseNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *toWrite)
{
  XMLChBuilder result;

  // Get the name and value out for convenience
  const XMLCh *nodeName = toWrite->getNodeName ();
  const XMLCh *nodeValue = toWrite->getNodeValue ();

  switch (toWrite->getNodeType ()) {
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE: {
      result.appendLiteral("<?xml version='1.0' encoding='UTF-8' ?>\n");
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = toWrite->getFirstChild ();
      while (child != 0) {
	result.appendAndDelete(serialiseNode(child));
	child = child->getNextSibling ();
      }//while
      break;
    }//case

	case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_TYPE_NODE: {
		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentType *docTypeNode = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentType *)toWrite;

      result.appendLiteral("<!DOCTYPE ");
      result.append(docTypeNode->getName());

		//Add any ExternalID
		const XMLCh *pubID = docTypeNode->getPublicId();
		const XMLCh *sysID = docTypeNode->getSystemId();

		if(pubID!=NULL && pubID[0]!=0) {
        result.appendLiteral(" PUBLIC \"");
        result.append(pubID);
        result.appendLiteral("\" \"");
        result.append(sysID);
        result.appendLiteral("\"");
		} else {
			if(sysID!=NULL && sysID[0]!=0) {
          result.appendLiteral(" SYSTEM \"");
          result.append(sysID);
          result.appendLiteral("\"");
			}//if
		}//if

      result.appendLiteral(">");

		cerr << "Warning: A DOCTYPE tag was output -- contents of tag will be empty" << endl;
		break;
	}//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
    // Output the element start tag.
    /*result.reserve(15)*/;
      result.appendLiteral("<");
      result.append(nodeName);

    // Output any attributes on this element
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attributes = toWrite->getAttributes ();
    int attrCount = attributes->getLength ();
    for (int i = 0; i < attrCount; i++) {
      /*result.reserve(20)*/;
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *attribute = attributes->item(i);
        result.appendLiteral(" ");
        result.append(attribute->getNodeName());
        result.appendLiteral("=\"");
        result.append(escapeString(attribute->getNodeValue(), true));
        result.appendLiteral("\"");
    }//for

    //  Test for the presence of children, which includes both
    //  text content and nested elements.
    if (toWrite->hasChildNodes ()) {
      // There are children. Close start-tag, and output children.
        result.appendLiteral(">");
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = toWrite->getFirstChild ();
      while (child != 0) {
          result.appendAndDelete(serialiseNode(child));
        child = child->getNextSibling ();
      }//while

      // Done with children.  Output the end tag.
        result.appendLiteral("</");
        result.append(nodeName);
        result.appendLiteral(">");
    } else {
      //  There were no children.  Output the close of the
      //  empty-element start tag.
        result.appendLiteral("/>");
    }//if
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE: {
    /*result.reserve(30)*/;
      result.appendLiteral("{");
      result.append(toWrite->getNamespaceURI());
      result.appendLiteral("}");
      result.append(nodeName);
      result.appendLiteral("=\"");
      result.append(nodeValue);
      result.appendLiteral("\"");
    break;
  }

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE: {
      result.append(escapeString(nodeValue, true));
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE: {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child;

    for (child = toWrite->getFirstChild (); child != 0; child = child->getNextSibling ())
        result.appendAndDelete(serialiseNode(child));
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE: {
    /*result.reserve(30)*/;
      result.appendLiteral("<?");
      result.append(nodeName);
      result.appendLiteral(" ");
      result.append(nodeValue);
      result.appendLiteral("?>");
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE: {
    /*result.reserve(30)*/;
      result.appendLiteral("<![CDATA[");
      result.append(nodeValue);
      result.appendLiteral("]]>");
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE: {
    /*result.reserve(30)*/;
      result.appendLiteral("<!--");
      result.append(nodeValue);
      result.appendLiteral("-->");
    break;
  }//case

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_FRAGMENT_NODE: {
    if (toWrite->hasChildNodes ()) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = toWrite->getFirstChild ();
      while (child != 0) {
          result.appendAndDelete(serialiseNode(child));
	child = child->getNextSibling();
      }//while
    }//if
    break;
  }//case

  //  case DOM_Node::XPATH_NAMESPACE_NODE: {
  case XPATH_NAMESPACE_NODE: {
		/*result.reserve(30)*/;
      result.appendLiteral("[");
      result.append(nodeName);
      result.appendLiteral("=\"");
      result.append(toWrite->getNamespaceURI());
      result.appendLiteral("\"");
      result.appendLiteral("]");
		break;
	}//case


  default: {
    cerr  << "Unrecognized node type: "
	  << (long) toWrite->getNodeType () << endl;
	assert(false);
  }
  }//switch

  return result.fetch();
}

#ifdef TEST
// Test functions
#include "../DOMutils/XStr.hpp"

v2Utils::v2Utils(string name)
  : TestCase(name)
{


}


void v2Utils::setUp(void)
{

  // Nothing to do

}

Test* v2Utils::suite(void)
{

  TestSuite* testSuite = new TestSuite("v2Utils");

  testSuite->addTest(new TestCaller<v2Utils> ("getStringFromNode",
					      &v2Utils::testGetStringFromNode));
  testSuite->addTest(new TestCaller<v2Utils> ("compareNodeLists",
					      &v2Utils::testCompareNodeLists));
  testSuite->addTest(new TestCaller<v2Utils> ("compareNodeListsLessThan",
					      &v2Utils::testCompareNodeListsLessThan));

  testSuite->addTest(new TestCaller<v2Utils> ("compareNodeListsGreaterThan",
					      &v2Utils::testCompareNodeListsLessThan));


  return testSuite;
}

void v2Utils::testGetStringFromNode(void)
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("node test") ));

  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> text1(getStringFromNode(node));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(text1.get(),X("node test"))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *comment = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createComment(X("jimjimjim")));

  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> text2(getStringFromNode(comment));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(text2.get(),X("jimjimjim"))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("toshmonster") ));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("noggin") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *node4 = data->createElement(X("foo"));
  node4->appendChild(node3);
  node4->appendChild(node2);
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> text4(getStringFromNode(node4));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(text4.get(),X("noggintoshmonster"))==0);
  node4->setAttribute(X("jim"),X("frank"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr = node4->getAttributeNode(X("jim"));
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> text5(getStringFromNode(attr));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(text5.get(),X("frank"))==0);
}

void v2Utils::testCompareNodeLists(void)
{

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text1 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("node test") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem1 = data->createElement(X("foo"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("test") ));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("node ") ));
  elem1->appendChild(node3);
  elem1->appendChild(node2);

  elem1->setAttribute(X("jim"),X("node test"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = elem1->getAttributeNode(X("jim"));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("noggin") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem2 = data->createElement(X("foo"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node4 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("gin") ));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node5 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("nog") ));
  elem2->appendChild(node5);
  elem2->appendChild(node4);

  elem2->setAttribute(X("jim"),X("flibble"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = elem2->getAttributeNode(X("jim"));

  auto_ptr<DeadNodeList> adnl(new DeadNodeList());
  auto_ptr<DeadNodeList> bdnl(new DeadNodeList());
  DeadNodeList *a= adnl.get();
  DeadNodeList *b= bdnl.get();

  a->push_back(text1);
  b->push_back(elem1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(attr1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(attr1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(elem1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  b->push_back(attr1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(elem2);
  assert(!compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(text2);
  assert(!compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  b->push_back(attr2);
  assert(!compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  a->push_back(attr2);
  b->push_back(text2);
  b->push_back(elem1);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(elem2);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(text2);
  assert(compareNodeLists(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  assert(compareNodeLists(a,b));

  a->clear();
  assert(!compareNodeLists(a,b));

  /*
    text1==elem1==attr1=="node test"
    text2==elem2=="noggin"
    attr2=="flibble"

    (text1)==(elem1)
    (text1)==(attr1)
    (elem1)==(attr1)

    (text1)==(text1)
    (elem1)==(elem1)
    (attr1)==(attr1)

    (text1)!=(elem2)
    (elem1)!=(text2)
    (attr1)!=(attr2)

    (text1,attr2)==(text2,elem1)
    (elem2,attr2,attr2)==(attr2,attr2,attr2,text2)
    (attr1,attr2,attr2)==(attr2,attr2,attr2)
    ()!=(attr2,attr2,attr2)
   */

}

void v2Utils::testCompareNodeListsLessThan(void)
{

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text1 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("0") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem1 = data->createElement(X("a0"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("0") ));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("jim") ));
  elem1->appendChild(node3);
  elem1->appendChild(node2);

  elem1->setAttribute(X("jim"),X("0"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = elem1->getAttributeNode(X("jim"));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("10") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem2 = data->createElement(X("a10"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node4 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("10") ));
  elem2->appendChild(node4);

  elem2->setAttribute(X("jim"),X("-10"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = elem2->getAttributeNode(X("jim"));

  auto_ptr<DeadNodeList> adnl(new DeadNodeList());
  auto_ptr<DeadNodeList> bdnl(new DeadNodeList());
  DeadNodeList *a= adnl.get();
  DeadNodeList *b= bdnl.get();

  a->push_back(text1);
  b->push_back(elem1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(attr1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(attr1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(elem1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  b->push_back(attr1);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text2);
  assert(compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text2);
  assert(compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  b->push_back(attr2);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  a->push_back(attr2);
  b->push_back(text2);
  b->push_back(elem2);
  assert(compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(elem2);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(text2);
  assert(!compareNodeListsLessThan(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(text2);
  b->push_back(text2);
  b->push_back(text2);
  assert(compareNodeListsLessThan(a,b));

  /*
    text1==elem1==attr1=="0"
    text2==elem2=="10"
    attr2=="-10"

    (text1) >= (elem1)
    (text1) >= (attr1)
    (elem1) >= (attr1)

    (text1) >= (text1)
    (elem1) >= (elem1)
    (attr1) >= (attr1)

    (text1) < (text2)
    (elem1) < (text2)
    (attr1) >= (attr2)

    (text1,attr2) < (text2,elem2)
    (elem2,attr2,attr2) == (attr2,attr2,attr2,text2)
    (attr1,attr2,attr2) < (text2,text2,text2)
   */

}

void v2Utils::testCompareNodeListsGreaterThan(void)
{

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text1 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("0") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem1 = data->createElement(X("0"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("0") ));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("jim") ));
  elem1->appendChild(node3);
  elem1->appendChild(node2);

  elem1->setAttribute(X("jim"),X("0"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = elem1->getAttributeNode(X("jim"));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *text2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("10") ));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elem2 = data->createElement(X("10"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node4 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("10") ));
  elem2->appendChild(node4);

  elem2->setAttribute(X("jim"),X("-10"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = elem2->getAttributeNode(X("jim"));

  auto_ptr<DeadNodeList> adnl(new DeadNodeList());
  auto_ptr<DeadNodeList> bdnl(new DeadNodeList());
  DeadNodeList *a= adnl.get();
  DeadNodeList *b= bdnl.get();

  a->push_back(text1);
  b->push_back(elem1);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(attr1);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(elem1);
  b->push_back(attr1);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text2);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(text1);
  b->push_back(text2);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  b->push_back(attr2);
  assert(compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(text2);
  a->push_back(elem2);
  b->push_back(text1);
  b->push_back(attr2);
  assert(compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(elem2);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(attr2);
  b->push_back(text2);
  assert(!compareNodeListsGreaterThan(a,b));

  a->clear(); b->clear();
  a->push_back(attr1);
  a->push_back(attr2);
  a->push_back(attr2);
  b->push_back(text2);
  b->push_back(text2);
  b->push_back(text2);
  assert(!compareNodeListsGreaterThan(a,b));

}

//*********************
#endif
