/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _CONTEXT_HPP
#define _CONTEXT_HPP

#include <config.h>

#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNode.hpp>

#include "../dom3-xpath/XPathResult.hpp"

class BaseContextImpl;
class FunctionLookup;
class FuncFactory;
class VariableStore;

///Context contains the control data for an expression
class PATHAN_EXPORT Context
{
public:
  Context(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *current, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *trigger, VariableStore *varStore,
          const FunctionLookup *funcTable, XPathResult::resultType resultType);
  Context(BaseContextImpl *impl);
  Context();
  ~Context();
  Context(const Context &rhs);
  Context &operator=(const Context &rhs);

  enum ContextType{
    BASE_CONTEXT       = 1,
    XPATH_CONTEXT      = 2,
    DSLPATH_CONTEXT    = 3,
    RESERVED_1         = 4,
    RESERVED_2         = 5
  };

//  Context *clone() const;

  /* Get */
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *getTriggerContext() const;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *getCurrentContext() const;

  const VariableStore *getVariableStore() const;
  const FunctionLookup *getFunctionTable() const;

  unsigned int getContextSize() const;
  unsigned int getContextPosition() const;

  ContextType getContextType() const;

  bool getCreativeMode() const;

  bool isOrderedResultType() const;

  /* Helper methods */

  /** get the value of exprns in scope at this context */
  const XMLCh *getDefaultExprNS() const;

  /** Creates the expanded name of a QName (Deprecated, should be removed)*/
  XMLCh *getExpandedName(const XMLCh *qname) const;

  /** returns the URI that is bound in prefix in the current scope or zero
      length string otherwise */
  const XMLCh *getInheritedPrefixNS(const XMLCh *prefix) const;

  /** returns the prefix that is bound in uri in the current scope or zero
      length string otherwise (Stupid name, should be changed)*/
  const XMLCh *getInheritedNSPrefix(const XMLCh *uri) const;

  /** returns the default ns in scope at this context (Stupid name, should be changed)*/
  const XMLCh *getInheritedDefaultNS() const;


  /* Set */

  /** Changes Trigger Context to node */
  void changeTrigger(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  /** Changes Current Context to node */
  void changeCurrent(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  /** Changes Current Context to node. For use in Navigation, SCSs and during
      execution of context changing loop commands. */
  void changeCurrentInLoop(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  /** Does nothing. For use in Navigation, SCSs and at
      the start of context changing loop commands. */
  void prepareForLoop();

  void setContextSize(unsigned int size);
  void setContextPosition(unsigned int pos);

  void setCreativeMode(bool creative);

private:

  /** Boolean governing whether the expression is to
      create nodes on the tree */
  bool _creativeMode;

  /// current data context
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *_currentContext;
  /// current script context
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *_triggerContext;

  /// XPath context variables
  unsigned int _contextSize, _contextPosition;

  Context::ContextType _type;

  /// variable store
  VariableStore *_varStore;
  /// Function lookup table
  const FunctionLookup *_functionTable;
  /// The result type requested.
  XPathResult::resultType _resultType;
};

#endif

