/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DATAITEM_HPP
#define _DATAITEM_HPP

#include "config.h"
#include "../defs/XMLScriptTypeDefs.hpp"

#include <vector>
#include <string>
#include <stdio.h>
#include <iostream>
#include <math.h>

#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNode.hpp>

class DeadNodeList;
class Context;

#ifdef TEST
#include "../testharness/Test.h"
#include "../testharness/TestSuite.h"
#include "../testharness/TestCase.h"
#include "../testharness/TestCaller.h"
#endif

using namespace std;

/** class which is used as base for all the different data types */
class PATHAN_EXPORT DataItem
#ifdef TEST
  : public TestCase
#endif
{
public:

  ///enum for data types
  typedef enum {
    STRINGLITERAL,
    NUMERICLITERAL,
    NODELIST,
    DATABOOLEAN,
    FUNCTION,
    NAVIGATION,
    VARIABLE
  } whichType;

  DataItem();
  virtual ~DataItem();

  virtual bool asBool(const Context *context) const = 0;
  virtual const XMLCh *asString(const Context *context) const = 0;
  virtual XSFloat asNum(const Context *context) const = 0;
  /// Be careful with this method, as any NaN or infinity information is lost
  virtual int asInt(const Context *context) const;
  /// Returns a copy of the nodelist that the caller _must_ delete.
  virtual DeadNodeList *asNodeList(const Context *context, const bool singleNode) const = 0;

  /** This evaluates non-simple data items, and returns their answer as simple
      data items. Caller takes ownership of returned data item */
  virtual DataItem* asSimpleDataItem(const Context *context) const = 0;

  virtual bool isEqual(const DataItem *other,
		       const Context *context) const = 0;
  virtual bool isLessThan(const DataItem *other,
			  const Context *context) const = 0;
  virtual bool isGreaterThan(const DataItem *other,
			     const Context *context) const = 0;

  DataItem::whichType getType(void) const;

protected:
  void setType(DataItem::whichType t);

  DataItem(const DataItem& rhs);
  DataItem& operator=(const DataItem& rhs);

private:
  DataItem::whichType _type;

#ifdef TEST
public:
  /// constructor for testing
  DataItem(string name);
#endif
};

#endif
