/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DATAITEMBOOL_HPP
#define _DATAITEMBOOL_HPP

#include <config.h>


#include "DataItem.hpp"

/** deals with the bool data type */
class PATHAN_EXPORT DataItemBool : public DataItem
{

public:
  DataItemBool(bool b);
  ~DataItemBool();

  ///returns _value
  bool asBool(const Context *context) const;
  ///returns "true" if _value or "false" otherwise
  const XMLCh *asString(const Context *context) const;
  ///returns 1 if _value or 0 otherwise
  XSFloat asNum(const Context *context) const;
  ///returns an empty node list and will eventually give a warning
  DeadNodeList *asNodeList(const Context *context, const bool singleNode) const;

  DataItem *asSimpleDataItem(const Context *context) const;

  ///is the DataItem passed in equal to this
  bool isEqual(const DataItem *other, const Context *context) const;
  bool isLessThan(const DataItem *other, const Context *context) const;
  bool isGreaterThan(const DataItem *other, const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  DataItemBool(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testAsBool();
  ///Test function
  void testAsString();
  ///Test function
  void testAsNum();
  ///Test function
  void testAsInt();
  ///Test function
  void testAsNodeList();
  ///Test equals
  void testEquals();
  /// Test less than
  void testLessThan();
  /// Test greater than
  void testGreaterThan();
#endif

protected:

  DataItemBool(const DataItemBool& rhs);

private:
  bool _value;
};

#endif
