/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemFunction.hpp"
#include <memory>

DataItem *DataItemFunction::asSimpleDataItem(const Context *context) const
{
  return execute(context);
}

DataItemFunction::DataItemFunction(const vector<const DataItem*> &args)
  :
#ifdef TEST
  DataItem("function"),
#endif
  _args(args),
  _lastString(NULL)
{
  setType(DataItem::FUNCTION);
}

DataItemFunction::~DataItemFunction()
{
   delete[] (XMLCh *)_lastString;
   _lastString=NULL;

   for(vector<const DataItem*>::const_iterator argI = _args.begin();
       argI != _args.end();
       ++argI)
   {
      delete *argI;
   }
}

bool DataItemFunction::asBool(const Context *context) const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
  return result->asBool(context);
}

const XMLCh *DataItemFunction::asString(const Context *context) const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));

  // Careful for exceptions
  delete[] (XMLCh *)_lastString;
  _lastString = NULL;
  _lastString = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(result->asString(context));
  return _lastString;
}

XSFloat DataItemFunction::asNum(const Context *context) const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
  return result->asNum(context);
}

DeadNodeList *DataItemFunction::asNodeList(const Context *context, const bool singleNode) const
{
   auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
   return result->asNodeList(context, singleNode);
}

bool DataItemFunction::isEqual(const DataItem *other,
			       const Context *context)  const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
  return result->isEqual(other, context);
}

bool DataItemFunction::isLessThan(const DataItem *other,
				  const Context *context)  const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
  return result->isLessThan(other, context);
}

bool DataItemFunction::isGreaterThan(const DataItem *other,
				     const Context *context)  const
{
  auto_ptr<DataItem> result = static_cast< auto_ptr<DataItem> >(execute(context));
  return result->isGreaterThan(other, context);
}

