/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DATAITEMNODE_HPP
#define _DATAITEMNODE_HPP

#include <config.h>

#include "../dataItem/DeadNodeList.hpp"
#include "../dataItem/DataItem.hpp"
#include "../exceptions/DataItemException.hpp"
#include "../DOMutils/v2Utils.hpp"

/// deals with the node list data type
class PATHAN_EXPORT DataItemNodeList : public DataItem
{
public:
  DataItemNodeList(DeadNodeList *v);
  ~DataItemNodeList();

  ///returns true if there are any nodes in the list or false otherwise
  bool asBool(const Context *context) const;
  ///returns the name of the first node in the list or a zero length string otherwise
  const XMLCh *asString(const Context *context) const;
  ///returns the number of nodes in the list
  XSFloat asNum(const Context *context) const;
  ///returns the nodelist
  DeadNodeList *asNodeList(const Context *context, const bool singleNode) const;

  DataItem *asSimpleDataItem(const Context *context) const;

  ///is the DataItem passed in equal to this
  bool isEqual(const DataItem *other, const Context *context) const;
  ///is the DataItem passed in less than this
  bool isLessThan(const DataItem *other, const Context *context) const;
  ///is the DataItem passed in greater than this
  bool isGreaterThan(const DataItem *other, const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  DataItemNodeList(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testAsBool();
  ///Test function
  void testAsString();
  ///Test function
  void testAsNum();
  ///Test function
  void testAsNodeList();
  ///Test function
  void testAsInt();
  ///Test equals
  void testEquals();
  ///Test less than
  void testLessThan();
  ///Test greater than
  void testGreaterThan();
#endif

protected:
  DeadNodeList *_nodeList;
  mutable const XMLCh *_cachedString;
   
  DataItemNodeList(const DataItemNodeList& rhs);
};

#endif
