/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DATAITEMNUMERIC_HPP
#define _DATAITEMNUMERIC_HPP

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include "DataItem.hpp"
#include "../DOMutils/v2Utils.hpp"

#include "../defs/XMLScriptTypeDefs.hpp"

/** deals with the numeric data type */
class PATHAN_EXPORT DataItemNumeric : public DataItem
{

public:
  DataItemNumeric(XSFloat d);
  ~DataItemNumeric();

  ///returns false if _value is 0 else true
  bool asBool(const Context *context) const;
  ///returns the number as a string (up to 256 chars)
  const XMLCh *asString(const Context *context) const;
  ///returns _value
  XSFloat asNum(const Context *context) const;
  ///returns an empty node list and will eventually give a warning
  DeadNodeList *asNodeList(const Context *context, const bool singleNode) const;

  DataItem *asSimpleDataItem(const Context *context) const;

  ///is the DataItem passed in equal to this
  bool isEqual(const DataItem *other, const Context *context) const;
  ///is the DataItem passed in less than this
  bool isLessThan(const DataItem *other, const Context *context) const;
  ///is the DataItem passed in greater than this
  bool isGreaterThan(const DataItem *other, const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  DataItemNumeric(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testAsBool();
  ///Test function
  void testAsString();
  ///Test function
  void testAsNum();
  ///Test function
  void testAsInt();
  ///Test function
  void testAsNodeList();
  ///Test equals
  void testEquals();
  /// Test less than
  void testLessThan();
  /// Test greater than
  void testGreaterThan();
#endif

protected:

  DataItemNumeric(const DataItemNumeric& rhs);

private:
  XSFloat _value;
  mutable const XMLCh *_cachedString;
};

#endif
