/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DATAITEMSTRING_HPP
#define _DATAITEMSTRING_HPP

#include <config.h>

#include "DataItem.hpp"
#include "../DOMutils/v2Utils.hpp"

/** defines the behaviour for the string data type*/
class PATHAN_EXPORT DataItemString : public DataItem
{

public:

  // Initialises with an empty string
  DataItemString();

  // Takes ownership of string given to it
  DataItemString(const XMLCh *n);
  ~DataItemString();

  /// returns false if 0 length or true if not
  bool asBool(const Context *context) const;
  /// returns a clone of the value
  const XMLCh *asString(const Context *context) const;
  /// uses strtod to extract numerals or 0 if no numerals
  XSFloat asNum(const Context *context) const;
  /// returns an empty node list and will eventually give a warning
  DeadNodeList *asNodeList(const Context *context, const bool singleNode) const;

  DataItem *asSimpleDataItem(const Context *context) const;

  ///comparison test between this and the DataItem passed in
  bool isEqual(const DataItem *other, const Context *context) const;
  ///test if this is less than item passed
  bool isLessThan(const DataItem *other, const Context *context) const;
  ///test if this is greater than item passed
  bool isGreaterThan(const DataItem *other, const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  DataItemString(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testAsBool();
  ///Test function
  void testAsString();
  ///Test function
  void testAsNum();
  ///Test function
  void testAsInt();
  ///Test function
  void testAsNodeList();
  ///Test equals
  void testEquals();
  ///Test less than
  void testLessThan();
  /// Test greater than
  void testGreaterThan();
#endif

protected:

  DataItemString(const DataItemString &rhs);

private:

  DataItemString& operator=(const DataItemString &rhs);
  const XMLCh *_name;
};

#endif

