/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemVariable.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../context/Context.hpp"
#include "../exceptions/VariableStoreException.hpp"
#include "../DOMutils/v2Utils.hpp"
#include <xercesc/util/Janitor.hpp>

DataItemVariable::DataItemVariable(const XMLCh *prefix,
                                   const XMLCh *name)
  : _prefix(prefix), _name(name)
{
  setType(DataItem::VARIABLE);
}

DataItemVariable::~DataItemVariable()
{
   delete[] (XMLCh *)_prefix;
   _prefix=NULL;
   delete[] (XMLCh *)_name;
   _name=NULL;
}

bool DataItemVariable::asBool(const Context *context) const
{
  return lookUpVariable(context)->asBool(context);
}

const XMLCh *DataItemVariable::asString(const Context *context) const
{
  return lookUpVariable(context)->asString(context);
}

XSFloat DataItemVariable::asNum(const Context *context) const
{
  return lookUpVariable(context)->asNum(context);
}

DeadNodeList *DataItemVariable::asNodeList(const Context *context, const bool singleNode) const
{
  return lookUpVariable(context)->asNodeList(context, singleNode);
}

bool DataItemVariable::isEqual(const DataItem *other,
			       const Context *context) const
{
  return lookUpVariable(context)->isEqual(other, context);
}

bool DataItemVariable::isLessThan(const DataItem *other,
				  const Context *context) const
{
  return lookUpVariable(context)->isLessThan(other, context);
}

bool DataItemVariable::isGreaterThan(const DataItem *other,
				     const Context *context) const
{
  return lookUpVariable(context)->isGreaterThan(other, context);
}

DataItem *DataItemVariable::asSimpleDataItem(const Context *context) const
{
  return lookUpVariable(context)->asSimpleDataItem(context);
}

const DataItem *DataItemVariable::lookUpVariable(const Context *context) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> qname((XMLCh *)v2NSUtils::qualifyName(_prefix, _name)); // awaiting Xerces fix

  const DataItem* var = context->getVariableStore()->getVar(qname.get(), context);

  if(!var) {
    string msg = "Variable ";
    msg += v2Utils::DSLtranscode(qname.get());
    msg += " does not exist";
    throw VariableStoreException("DataItemVariable", msg);
  }

  return var;
}


