/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DEADNODELIST_HPP
#define _DEADNODELIST_HPP

#include <config.h>

#include <vector>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMDocument.hpp>

class DeadNodeList
{
public:

  typedef std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*>::iterator iterator;
  typedef std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*>::const_iterator const_iterator;
  typedef std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*>::reverse_iterator reverse_iterator;
  typedef std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*>::const_reverse_iterator const_reverse_iterator;

  DeadNodeList();
  ~DeadNodeList();

  //create empty list, while setting the document root
  //(for the purpose of working with hasDocumentChanged)
  DeadNodeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *docRoot);

  ///create using a DOM_NodeList
  DeadNodeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *domList);

  /**Convert to a XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList. An exception is thrown if all the nodes are not
     from the specified document, or if the nodelist contains attribute
     nodes.*/
  operator XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *() const;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *liveList() const;

  ///Returns the indexth item in the collection
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *item(unsigned int index) const;

  ///Returns the first node in the list in document order
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *first() const;

  ///Returns the number of nodes in the list
  unsigned int getLength(void) const;

  ///Returns whether any nodes exist in the list
  bool empty(void) const;

  ///Puts a DOM_Node in the store at the back
  void push_back(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  ///Puts a DOM_Node in the store at the front
  void push_front(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  ///clears the list
  void clear(void);

  ///adds nodeList to the this nodeList
  void addList(const DeadNodeList *nodeList);

  ///adds domList to the this nodeList
  void addList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *domList);

  ///has the document changed since the DeadNodeList was created
  bool hasDocumentChanged(void) const;

  ///Returns a copy of the store as a DeadNodeList
  DeadNodeList *clone(bool deep) const;

  ///Begin an iterator
  iterator begin(void);
  const_iterator begin(void) const;

  ///Find the end of the iterator
  iterator end(void);
  const_iterator end(void) const;

  ///Begin a reverse_iterator
  reverse_iterator rbegin(void);
  const_reverse_iterator rbegin(void) const;

  ///Find the end of a reverse_iterator
  reverse_iterator rend(void);
  const_reverse_iterator rend(void) const;

  /// Remove any duplicate nodes from the list.
  void removeDuplicates(bool ordered);

private:

  std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*> _vector;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *documentRoot; //used for keeping track of when the document changes

  //The changes() counter from the Document when the NodeList has its first Node added
  int _documentChanges;
  bool _documentChangesSet;

  //Called on when a node is added to the nodelist, this sets
  //_initialDocumentChanges if unset
  void setDocumentChanges(void);

  DeadNodeList(const DeadNodeList& rhs);
  DeadNodeList& operator=(const DeadNodeList& rhs);
};

#endif
