/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _NUMUTILS_HPP
#define _NUMUTILS_HPP

#include <config.h>


#ifdef TEST
#include "../testharness/Test.h"
#include "../testharness/TestSuite.h"
#include "../testharness/TestCase.h"
#include "../testharness/TestCaller.h"
#endif

#include <xercesc/util/XMLString.hpp>

#include "../defs/XMLScriptTypeDefs.hpp"

using namespace std;

/** class with static methods for dealing with floating point numbers */
class PATHAN_EXPORT NumUtils
#ifdef TEST
  : public TestCase
#endif
{
public:
  /** turns a double into a DOMString. caller takes ownership */
  static XMLCh *doubleAsString(XSFloat d);
  static XMLCh *uIntAsString(unsigned int d);
  static XMLCh *intAsString(unsigned int d);
  static XSFloat stringAsDouble(const XMLCh *s);
  static int stringAsInt(const XMLCh *s);

  ///turns a null terminated XMLCh* into a double
  static XSFloat stringAsDouble(const XMLCh *s, const unsigned int length);

  /** returns true if n is NaN */
  static bool isNaN(XSFloat n);

  /** returns true if n is Infinite */
  static bool isInfinite(XSFloat n);

  /** returns true if n is negative*/
  static bool isNegative(XSFloat n);

  /** compare two XSFloats for equality. Within a margin, epsilon */
  static bool isEqual(XSFloat x, XSFloat y);

  /** compare two XSFloats, and returns x < y. Within a margin, epsilon */
  static bool isLessThan(XSFloat x, XSFloat y);

  /** compare two XSFloats, and returns x > y. Within a margin, epsilon */
  static bool isGreaterThan(XSFloat x, XSFloat y);

#ifdef TEST
  /// constructor for testing
  NumUtils(string name);
  ///Test setup
  void setUp(void);
  ///Test body
  static Test *suite(void);
  void testStringAsDouble(void);
  void testDoubleAsString(void);
#endif
};

#endif // _NUMUTILS_HPP



