/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionAnd.hpp"
#include "../dataItem/DataItemBool.hpp"

FunctionAnd::FunctionAnd(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionAnd::execute(const Context *context) const
{
  return new DataItemBool(getBoolArg(0, context) && getBoolArg(1, context));
}

#ifdef TEST
// Test functions
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"

#include "../testharness/TestFunction.hpp"

void FunctionAnd::setUp()
{

  // Nothing to do

}
FunctionAnd::FunctionAnd(std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{
}

void FunctionAnd::testExecute()
{
  Context c;
  Context *context = &c;

  auto_ptr<DataItemNumeric> t(new DataItemNumeric(1));
  DataItem *tdi = t.get();

  auto_ptr<DataItemBool> f(new DataItemBool(false));
  DataItem *fdi = f.get();

  _args.push_back(tdi);
  _args.push_back(tdi);


  auto_ptr<DataItem> result(execute(context));
  assert(result->asBool(context));

  _args.clear();
  _args.push_back(tdi);
  _args.push_back(fdi);


  auto_ptr<DataItem> result2(execute(context));
  assert(!result2->asBool(context));

  _args.clear();
  _args.push_back(fdi);
  _args.push_back(tdi);


  auto_ptr<DataItem> result3(execute(context));
  assert(!result3->asBool(context));

  _args.clear();
  _args.push_back(fdi);
  _args.push_back(fdi);


  auto_ptr<DataItem> result4(execute(context));
  assert(!result4->asBool(context));

  _args.clear();
}

#endif
