/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#/*

  And operator function

*/

#ifndef _FUNCTIONAND_HPP
#define _FUNCTIONAND_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** Concat operator function*/
class PATHAN_EXPORT FunctionAnd : public XPathFunction
{
public:
  ///Constructor. Sets two arg defs as NOT_OPTIONAL and unlimited args to true.
  FunctionAnd(const vector<const DataItem*> &args);

  /** ands two booleans. Takes two booleans, and gives a boolean. Sets result
      to true if all arguments are true, false otherwise */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///TEST: Dummy constructor for test suite. Sets two arg defs as NOT_OPTIONAL and unlimited args to true.
  FunctionAnd(std::string name);
  ///TEST: Test setup
  void setUp();
  ///TEST: Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONAND_HPP

#if 0
<userDoc>
<section>
  <title><command>and</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>and(argument1, argument2, ...) -> boolean </emphasis>
  argument1  = boolean-expression #REQUIRED
  argument2  = boolean-expression #REQUIRED
  argument...= boolean-expression #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function takes at least two boolean arguments, and returns a boolean. The
    return value will be true if both of the arguments are true, and false
    otherwise. If any of the arguments are node lists, then the default behaviour is used
    to convert them to booleans.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" expression-syntax="dslpath">
      <output>
        a 1 # and(1,1) #
        b 0 # and(1,0) #
        c 0 # and(0,1) #
        d 1 # and(1,1,1,3) #
        e 1 # and(1,1,1,1) #
        f 1 # and(1,1,1,"1") #
        g 1 # and(1,1,1,"string") #
        h 1 # and(1,1 and 1 and 1) #
        i 1 # 1 and "string" #
      </output>
      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
              a 1 1
              b 0 0
              c 0 0
              d 1 1
              e 1 1
              f 1 1
              g 1 1
              h 1 1
              i 1 1
    </screen>.
    </para>
  </section>
</section>
</userDoc>

#endif


