/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Boolean function

*/

#ifndef _FUNCTIONBOOLEAN_HPP
#define _FUNCTIONBOOLEAN_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** Boolean operator function, and exists functions. Warning, you may need to think
    about this one, but in fact, these two functions have the same behaviour. */
class PATHAN_EXPORT FunctionBoolean : public XPathFunction
{
public:
  ///Constructor. Sets arg def to NOT_OPTIONAL
  FunctionBoolean(const vector<const DataItem*> &args);

  /** Returns the argument as a boolean. Sets result to value of getNextBool() **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///TEST: Dummy constructor for test suite. Sets arg def to NOT_OPTIONAL
  FunctionBoolean(std::string name);
  ///TEST: Test setup
  void setUp();
  ///TEST: Test boolean on a node
  void testExecute();
#endif
};

#endif // _FUNCTIONBOOLEAN_HPP

#if 0
<userDoc>
<section>
  <title><command>boolean</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>boolean(argument1) -> boolean</emphasis>
  argument1 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function takes an argument of any type, and returns a boolean. If the
    argument is a number, then false is returned if the number is zero or NaN.
    Otherwise, if the argument is a number other than 0 and NaN, then true is
    returned. If the argument is a string, it returns false if the
    string is of zero length, and true otherwise. If the argument is a node
    list, it returns false if the list is empty, and true otherwise.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">
      <data>
        <morue>
          <bacalao/>
        </morue>
        <morue/>
        <foo/>
      </data>

      <output>
      a 1 #boolean(4)#
      b 1 #boolean(1)#
      c 0 #boolean(0)#
      d 1 #boolean(-1)#
      e 1 #boolean("cod")#
      f 1 #boolean('atlantic cod')#
      g 1 #boolean(\xs:dd\xs:morue)#
      i 1 #boolean(\xs:dd\xs:morue\xs:bacalao)#
      j 0 #boolean(\xs:dd\xs:morue\IdoNotExist)#
      </output>
      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1 1
      b 1 1
      c 0 0
      d 1 1
      e 1 1
      f 1 1
      g 1 1
      i 1 1
      j 0 0
    </screen>.
    </para>
  </section>
</section>
</userDoc>

#endif

