/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionConcat.hpp"
#include "../DOMutils/XMLChBuilder.hpp"
#include "../dataItem/DataItemString.hpp"

#ifdef TEST
#include "../testharness/TestFunction.hpp"
#endif

FunctionConcat::FunctionConcat(const vector<const DataItem*> &args)
  : XPathFunction(2, UNLIMITED, args) // Between 2 and UNLIMITED arguments
{
  // Nothing to do
}

DataItem *FunctionConcat::execute(const Context *context) const
{
  XMLChBuilder result;

  for(unsigned int i = 0; i < getNumArgs(); ++i)
  {
    result.append(getStringArg(i, context));
  }

  return new DataItemString(result.fetch());
}

#ifdef TEST
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../DOMutils/XStr.hpp"

FunctionConcat::FunctionConcat(std::string name)
  : XPathFunction(2, UNLIMITED, vector<const DataItem*>())
{
}

// Test functions

void FunctionConcat::setUp()
{

  // Nothing to do

}

void FunctionConcat::testExecute()
{
  Context c;
  Context *context = &c;

  auto_ptr<DataItemString> str(new DataItemString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Test ") ));
  DataItem *strdi =  str.get();
  _args.push_back( strdi );

  auto_ptr<DataItemString> str2(new DataItemString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this") ));
  DataItem *strdi2 = str2.get();
  _args.push_back( strdi2 );

  auto_ptr<DataItem> result(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asString(context),X("Test this"))==0);
  _args.clear();

  auto_ptr<DataItemNumeric> num(new DataItemNumeric( 55.3 ));
  DataItem *numdi = num.get();
  _args.push_back( numdi );

  auto_ptr<DataItemBool> bol(new DataItemBool( true ));
  DataItem *boldi =  bol.get();
  _args.push_back( boldi );

  auto_ptr<DataItem> result2(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X("55.3true") ) == 0);
  _args.clear();

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));
  DeadNodeList *nl = new DeadNodeList;
  DeadNodeList *nl2 = new DeadNodeList;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr = data->createAttribute(X("frank"));
  attr->setValue(X("jim"));
  nl->push_back(attr);
  auto_ptr<DataItemNodeList> dinl(new DataItemNodeList(nl));
  DataItem *nodelist1 = dinl.get();

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *el = data->createElement(X("bob"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMText *tx = data->createTextNode(X("nancy"));
  el->appendChild(tx);
  nl2->push_back(el);
  auto_ptr<DataItemNodeList> dinl2(new DataItemNodeList(nl2));
  DataItem *nodelist2 = dinl2.get();

  _args.push_back(nodelist1);
  _args.push_back(nodelist2);

  auto_ptr<DataItem> result3(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X("jimnancy") )==0 );  _args.clear();

}

void FunctionConcat::testNullString()
{
  Context c;
  Context *context = &c;

  auto_ptr<DataItemString> nll(new DataItemString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("") ));
  DataItem *nlldi = nll.get();
  
  auto_ptr<DataItemString> str(new DataItemString( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Not null") ));
  DataItem *strdi = str.get();

  _args.push_back( nlldi );
  _args.push_back( nlldi );
  auto_ptr<DataItem> result(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asString(context), X("") )==0 );
  _args.clear();

  _args.push_back( nlldi );
  _args.push_back( strdi );
  auto_ptr<DataItem> result2(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context), X("Not null") )==0 );
  _args.clear();

  _args.push_back( strdi );
  _args.push_back( nlldi );
  auto_ptr<DataItem> result3(execute(context));

  assert( XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context), X("Not null") )==0 );
  _args.clear();
}

#endif
