/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Concat operator function

*/

#ifndef _FUNCTIONCONCAT_HPP
#define _FUNCTIONCONCAT_HPP


#ifdef WIN32
#pragma warning( disable: 4786 )
#endif

#include <config.h>

#include "XPathFunction.hpp"

/** Concat operator function*/
class PATHAN_EXPORT FunctionConcat : public XPathFunction
{
public:
  ///Constructor.
  FunctionConcat(const vector<const DataItem*> &args);

protected:
  /** concatenates two strings. Takes two strings, and gives a string */
  DataItem *execute(const Context *context) const;

#ifdef TEST
public:
  FunctionConcat(std::string name);
  ///TEST: Test setup - Does nothing
  void setUp();
  ///TEST: Test function
  void testExecute();
  ///TEST: Test function using empty strings
  void testNullString();
#endif
};

#endif // _FUNCTIONCONCAT_HPP

#if 0
<userDoc>
<section>
  <title><command>concat</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>concat(argument1, argument2, ...) -> string</emphasis>
  argument1  = #PCDATA #REQUIRED
  argument2  = #PCDATA #REQUIRED
  argument...= #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This funciton takes two or more strings, and returns the result of
    their concatenation. Another way of expressing this
    function is by using the '_' operator in DSLPath. (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <data>
        <saury>Testing</saury>
        <paparda>if we can concat properly</paparda>
      </data>

      <output>
      a #concat("123","456")#
      b #concat("","456","")#
      c #concat("","yu7","skljsdj29290")#
      d #concat(\xs:dd\xs:saury,\xs:dd\xs:paparda)#
      e #"123" _ "456"#
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 123456
      b 456
      c yu7skljsdj29290
      d Testingif we can concat properly
      e 123456
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif

