/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionContains.hpp"
#include "../dataItem/DataItemBool.hpp"


FunctionContains::FunctionContains(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionContains::execute(const Context *context) const 
{ 
  const XMLCh *container = getStringArg(0, context); 
  const XMLCh *pattern = getStringArg(1, context); 
 
  bool contains = XERCES_CPP_NAMESPACE_QUALIFIER 
XMLString::stringLen(pattern) == 0 || 
    XERCES_CPP_NAMESPACE_QUALIFIER XMLString::patternMatch(container, 
pattern) > -1; 
 
  return new DataItemBool(contains); 
} 

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"

// Test functions

void FunctionContains::setUp()
{

  // Nothing to do

}

FunctionContains::FunctionContains(std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{





}

void FunctionContains::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjimbobmavis")));
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjim")));
  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("jim")));
  auto_ptr<DataItem> test4(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("mavis")));
  auto_ptr<DataItem> test5(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("hello")));
  auto_ptr<DataItem> test6(new DataItemNodeList(new DeadNodeList));
  _args.push_back(test1.get());
  _args.push_back(test2.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(result1->asBool(context));

  _args.clear();
  _args.push_back(test2.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(!result2->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(result3->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test3.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(result4->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test4.get());


  auto_ptr<DataItem> result5(execute(context));

  assert(result5->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test5.get());


  auto_ptr<DataItem> result6(execute(context));

  assert(!result6->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test6.get());


  auto_ptr<DataItem> result7(execute(context));

  assert(result7->asBool(context));

  _args.clear();
  _args.push_back(test6.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result8(execute(context));

  assert(!result8->asBool(context));

  _args.clear();
  _args.push_back(test6.get());
  _args.push_back(test6.get());


  auto_ptr<DataItem> result9(execute(context));

  assert(result9->asBool(context));

  _args.clear();
}

//*********************
#endif
