/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Count function

*/

#ifndef _FUNCTIONCOUNT_HPP
#define _FUNCTIONCOUNT_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** Count function. */
class PATHAN_EXPORT FunctionCount : public XPathFunction
{
public:
  ///Constructor. Sets arg def to NOT_OPTIONAL.
  FunctionCount(const vector<const DataItem*> &args);

  /** Returns the number of nodes in the nodelist argument. **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///TEST: Dummy constructor for test suite. Constructor. Sets arg def to NOT_OPTIONAL.
  FunctionCount(std::string name);
  ///TEST: Test setup - Does nothing
  void setUp();
  ///TEST: Test count - Does nothing
  void testExecute();
#endif
};

#endif // _FUNCTIONCOUNT_HPP


#if 0
<userDoc>
<section>
  <title><command>count</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>count(argument1) -> numeral</emphasis>
  argument1 = expression-nodeList #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function takes a node list and returns the number of nodes in that list.
    This function is the same as the <emphasis>count()</emphasis> function described in XPath.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0"  xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <data>
        <amberjack>
          <rudderfish/>
          <seriole/>
          <carangidae/>
        </amberjack>
        <amberjack>
          <rudderfish/>
          <seriole/>
          <carangidae/>
        </amberjack>
      </data>

      <output>
      6 # count(\xs:dd\xs:amberjack\xs:*) #
      2 # count(\xs:dd\xs:amberjack) #
      3 # count(\xs:dd\xs:amberjack[2]\xs:*) #
      3 # count(\xs:dd\xs:amberjack[1]\xs:*) #
      2 # count(\xs:dd\xs:amberjack\xs:rudderfish) #
      0 # count(\xs:dd\xs:amberjack\xs:rudderfish\xs:Idontexist) #
      0 # count(\xs:dd\xs:amberjack\xs:rudderfish\xs:Idontexist\xs:*) #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      6 6
      2 2
      3 3
      3 3
      2 2
      0 0
      0 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






