/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionFloor.hpp"
#include "../dataItem/DataItemNumeric.hpp"

#include <math.h>

FunctionFloor::FunctionFloor(const vector<const DataItem*> &args)
  : XPathFunction(1, 1, args)
{
}

DataItem *FunctionFloor::execute(const Context *context) const
{
  XSFloat f = getNumeralArg(0, context);
  return new DataItemNumeric(floor(f));
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"

// Test functions

void FunctionFloor::setUp()
{

  // Nothing to do

}

FunctionFloor::FunctionFloor(std::string name)
  : XPathFunction(1, 1, vector<const DataItem*>())
{




}

void FunctionFloor::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemNumeric(5.5));
  auto_ptr<DataItem> test2(new DataItemNumeric(5.0));
  auto_ptr<DataItem> test3(new DataItemNumeric(5));
  auto_ptr<DataItem> test4(new DataItemNumeric(4.8));
  auto_ptr<DataItem> test5(new DataItemNumeric(-4.3));
  auto_ptr<DataItem> test6(new DataItemNumeric(-4));
  auto_ptr<DataItem> test7(new DataItemNumeric(0));
  _args.push_back(test1.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(result1->asNum(context) == 5);

  _args.clear();
  _args.push_back(test2.get()); 


  auto_ptr<DataItem> result2(execute(context));

  assert(result2->asNum(context) == 5);

  _args.clear();
  _args.push_back(test3.get()); 


  auto_ptr<DataItem> result3(execute(context));
    
  assert(result3->asNum(context) == 5);

  _args.clear();
  _args.push_back(test4.get()); 


  auto_ptr<DataItem> result4(execute(context));

  assert(result4->asNum(context) == 4);

  _args.clear();
  _args.push_back(test5.get()); 


  auto_ptr<DataItem> result5(execute(context));

  assert(result5->asNum(context) == -5);

  _args.clear();
  _args.push_back(test6.get()); 


  auto_ptr<DataItem> result6(execute(context));

  assert(result6->asNum(context) == -4);

  _args.clear();
  _args.push_back(test7.get()); 


  auto_ptr<DataItem> result7(execute(context));

  assert(result7->asNum(context) == 0);

  _args.clear();
}

//*********************
#endif
