/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   Id function

*/

#ifndef _FUNCTIONID_HPP
#define _FUNCTIONID_HPP

#include <config.h>


#include "XPathFunction.hpp"
#include "../dataItem/DeadNodeList.hpp"

/** Id function */
class PATHAN_EXPORT FunctionId : public XPathFunction
{
public:
  ///Constructor.
  FunctionId(const vector<const DataItem*> &args);

  /** gives back the element with appropriate ID*/
  DataItem *execute(const Context *context) const;

private:
  /** helper function to run the id query */
  DeadNodeList *findID(XMLCh *values,
                       const Context *context) const;

public:
#ifdef TEST
  ///TEST: Dummy constructor for test suite. Constructor. Sets arg defs to NOT_OPTIONAL and OPTIONAL_NOCONTEXT.
  FunctionId(std::string name);
  ///TEST: Test setup - Does nothing
  void setUp();
  ///TEST: Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONID_HPP


#if 0

<userDoc>
<section>
  <title><command>id</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>id(argument1) -> nodelist</emphasis>
  argument1 = object  #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>The id function selects elements by their unique ID. When the argument to id is of type nodelist, then the result is the union of the result of applying id to the string-value of
        each of the nodes in the argument nodelist. When the argument to id is of any other type, the argument is converted to a string as if by a call to the string function; the string is split into a
        whitespace-separated list of tokens (whitespace is any sequence of characters matching the production S); the result is a nodelist containing the elements in the same document as the context node
        that have a unique ID equal to any of the tokens in the list.
    </para>
  </section>

  <section>
    <title>Warning</title>
    <para>This function will not work with the xtract command line processor&apos;s data document,
          as it is a constructed document, and not a parsed document.
    </para>
  </section>
</section>
</userDoc>

#endif






