/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionLang.hpp"

#include "../DOMutils/v2Utils.hpp"
#include "../context/Context.hpp"
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include "../dataItem/DataItemBool.hpp"

// "lang"
static const XMLCh textLang[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "http://www.w3.org/XML/1998/namespace"
static const XMLCh textLangNamespace[] =
{
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_h, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chColon, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chPeriod, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_3, XERCES_CPP_NAMESPACE_QUALIFIER chPeriod,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_X, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_M, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_L, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
  XERCES_CPP_NAMESPACE_QUALIFIER chDigit_1, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_9, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_9, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_8, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e,
  XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

FunctionLang::FunctionLang(const vector<const DataItem*> &args)
  : XPathFunction(1, 1, args)
{
}

DataItem *FunctionLang::execute(const Context *context) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *root = NULL;
  bool result = false;
  const XMLCh *xmlLangValue = getStringArg(0, context);
  unsigned int langLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(xmlLangValue);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = context->getCurrentContext();

  if(node!=NULL ){
    root = node->getOwnerDocument();
  }

  while(node!=NULL  && node != root){
    XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *myNode = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node);
    const XMLCh *value = myNode->getAttributeNS( textLangNamespace,textLang);
    if(value != NULL){
      if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareNString(value, xmlLangValue, langLen)==0) {
	  result = true;
	}
      break;
    }
    node = node->getParentNode();
  }

  return new DataItemBool(result);
}


#ifdef TEST

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../simpleVariables/VariableStore.hpp"
#include "../functionAPI/FunctionLookup.hpp"

// Test functions

void FunctionLang::setUp()
{

  // Nothing to do

}

FunctionLang::FunctionLang(std::string name)
  : XPathFunction(1, 1, vector<const DataItem*>())
{




}

// Not tests
void FunctionLang::testExecute()
{
  TestFunction test;

  XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser xmlparser;
  test.initDOMParser(xmlparser);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *script = test.parseXML(xmlparser, strdup("lang.data.xml"));
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(test.initDataTree());
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> out(test.initOutputTree());

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *node = script->getDocumentElement();

  Context c(node, node, new VariableStore, new FunctionLookup, XPathResult::ANY_TYPE);
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode = context->getCurrentContext();
  contextNode = contextNode->getFirstChild()->getFirstChild()->getFirstChild();
  context->changeCurrent(contextNode);

  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("zoo")));
  _args.push_back(test1.get());



  auto_ptr<DataItem> result(execute(context));
  assert(result->asBool(context));

  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("zooo")));
  _args.clear();
  _args.push_back(test2.get());



  result.reset(execute(context));
  assert(!result->asBool(context));

  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("en-us")));
  _args.clear();
  _args.push_back(test3.get());



  result.reset(execute(context));
  assert(!result->asBool(context));

  _args.clear();
}

//*********************
#endif
