/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  lang function

*/

#ifndef _FUNCTIONLANG_HPP
#define _FUNCTIONLANG_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** lang function. */
class PATHAN_EXPORT FunctionLang : public XPathFunction
{
public:
  FunctionLang(const vector<const DataItem*> &args);

  /** The lang function returns true or false depending on whether the language of
      the context node as specified by xml:lang attributes is the same as or is a
      sublanguage of the language specified by the argument string. The language of
      the context node is determined by the value of the xml:lang attribute on the
      context node, or, if the context node has no xml:lang attribute, by the value
      of the xml:lang attribute on the nearest ancestor of the context node that has
      an xml:lang attribute.**/

  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionLang(std::string name);
  ///Test setup
  void setUp();
  void testExecute();
#endif
};

#endif // _FUNCTIONLANG_HPP


#if 0
<userDoc>
<section>
  <title><command>lang</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>lang(argument1) -> boolean</emphasis>
  argument1 = string #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
      The lang function returns true or false depending on whether the language of
      the context node as specified by xml:lang attributes is the same as or is a
      sublanguage of the language specified by the argument string. The language of
      the context node is determined by the value of the xml:lang attribute on the
      context node, or, if the context node has no xml:lang attribute, by the value
      of the xml:lang attribute on the nearest ancestor of the context node that has
      an xml:lang attribute.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">


      <data>
        <angler_fish xml:lang="en-us-fish">
          <bass xml:lang="gr-zoo"><sardine /></bass>
        </angler_fish>
      </data>

      <output>

      <foreach node="\xs:dd\xs:angler_fish\xs:bass\xs:sardine">
      <output>
      TEST 1:
        <if test='lang("en-us")'>
        <then>
          language of the context node as specified by xml:lang attributes is
          the same as or is a sublanguage of the language specified by the
          argument string
        </then>
        <else>
          language of the context node as specified by xml:lang attributes is
          not the same as or is not a sublanguage of the language specified by
          the argument string
        </else>
        </if>
      </output>
      </foreach>

      <foreach node="\xs:dd\xs:angler_fish\xs:bass\xs:sardine">
      <output>
      TEST 2:
        <if test='lang("gr")'>
        <then>
          language of the context node as specified by xml:lang attributes is
          the same as or is a sublanguage of the language specified by the
          argument string
        </then>
        <else>
          language of the context node as specified by xml:lang attributes is
          not the same as or is not a sublanguage of the language specified by
          the argument string
        </else>
        </if>
      </output>
      </foreach>


      </output>


      </xst>
    ]]></programlisting>
    should produce the output
    <screen>

      TEST 1:

      language of the context node as specified by xml:lang attributes is
      not the same as or is not a sublanguage of the language specified by
      the argument string

      TEST 2:

      language of the context node as specified by xml:lang attributes is
      the same as or is a sublanguage of the language specified by the
      argument string

    </screen>
    </para>
  </section>
</section>
</userDoc>
#endif






