/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   Localname function

*/

#ifndef _FUNCTIONLOCALNAME_HPP
#define _FUNCTIONLOCALNAME_HPP

#include <config.h>


#include "XPathFunction.hpp"
#include "../dataItem/DeadNodeList.hpp"

/** Localname function */
class PATHAN_EXPORT FunctionLocalname : public XPathFunction
{
public:
  FunctionLocalname(const vector<const DataItem*> &args);

  /** Returns the namespace unaware name of the node. Takes an optional nodelist,
      and returns an empty string if the node has no name. Uses the current context
      if a nodelist is not specified. */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionLocalname(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONLOCALNAME_HPP


#if 0
<userDoc>
<section>
  <title><command>localname</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>localname(argument1) -> string</emphasis>
  argument1 = #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the local name (name without prefix) of the node. The first node of the
    optional argument is used to retrieve the local name from. If the argument is not present, then the
    current context is used as the input.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath" xmlns:aa="aaa" xmlns:bb="bbb">

      <data>
        <blackfish>
          <aa:carpa/>
          <bb:tautogue_noir/>
          <tautoga_onitis/>
        </blackfish>
      </data>

      <foreach node="\xs:dd\xs:blackfish\*:*" >
        <output>
          # localname(.) #
        </output>
      </foreach>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
          carpa

          tautogue_noir

          tautoga_onitis
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






