/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionLookup.hpp"
#include "../factory/FuncFactoryTemplate.hpp"
#include "../exceptions/FunctionException.hpp"
#include "../DOMutils/v2Utils.hpp"

FunctionLookup::FunctionLookup()
{
}

FunctionLookup::~FunctionLookup()
{
   // Delete the factories in the list
   for(std::map<const XMLCh *, FuncFactory *, XMLChLess>::iterator i = _funcTable.begin();
       i != _funcTable.end(); ++i) {
      delete[] (XMLCh *)i->first;
      delete i->second;
   }
}

void FunctionLookup::insertFunction(const XMLCh *name, FuncFactory *func)
{
  _funcTable[name] = func;
}

DataItem *FunctionLookup::lookUpFunction(const XMLCh *fname,
                                         const std::vector<const DataItem*> &args) const
{
   map<const XMLCh *, FuncFactory *, XMLChLess>::const_iterator funcI = _funcTable.find(fname);
   if(funcI != _funcTable.end())
   {
      return funcI->second->createInstance(args);
   }
   throw(FunctionException("lookup", string("Tried to lookup unknown function: ")
                           + v2Utils::DSLtranscode(fname)));
}
