/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _FLOOKUP_HPP
#define _FLOOKUP_HPP

#include <config.h>

#ifdef WIN32
#pragma warning( disable : 4786 )
#endif

#include "../factory/FuncFactory.hpp"
#include "../DOMutils/XMLChCompare.hpp"

#include <map>

/** Abstract base class, implementing a lookup table for functions */
class FunctionLookup
{
public:

  FunctionLookup();
  virtual ~FunctionLookup();

  ///adds a function to the function table
  void insertFunction(const XMLCh *name, FuncFactory *func);

  ///returns the approriate Function object
  DataItem *lookUpFunction(const XMLCh *fname,
                           const std::vector<const DataItem*> &args) const;

private:
  /// Quick lookup for functions by name. Functions pointed to are not unique.
  std::map<const XMLCh *, FuncFactory *, XMLChLess> _funcTable;

  FunctionLookup(const FunctionLookup &rhs);
  FunctionLookup &operator=(const FunctionLookup& rhs);
};

#endif
