/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   Namespace function

*/

#ifndef _FUNCTIONNAMESPACE_HPP
#define _FUNCTIONNAMESPACE_HPP

#include <config.h>


#include "XPathFunction.hpp"
#include "../dataItem/DeadNodeList.hpp"

/** Namespace function */
class PATHAN_EXPORT FunctionNamespace : public XPathFunction
{
public:
  FunctionNamespace(const vector<const DataItem*> &args);

  /** Returns the namespace of the node. Takes an optional nodelist,
      and returns an empty string if the node has no name. Uses the current context
      if a nodelist is not specified. */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionNamespace(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};


#if 0
<userDoc>
<section>
  <title><command>namespace</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>namespace(argument1) -> string</emphasis>
  argument1 = #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the namespace of the node. The first node of the optional argument
    is used to retrieve the local name from. If the argument is not present, then the
    current context is assumed to be the input.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath" xmlns:aa="aaa" xmlns:bb="bbb">

      <data>
        <hoki>
          <aa:whiptail/>
          <bb:blue_haki/>
          <merluccidae/>
        </hoki>
      </data>

      <foreach node="\xs:dd\xs:hoki\*:*" >
        <output>
          # namespace(.) #
        </output>
      </foreach>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
          aaa

          bbb

          http://www.xmlscript.org/2.0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif





#endif // _FUNCTIONNAMESPACE_HPP


