/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionNormalizeSpace.hpp"

#include "../DOMutils/v2Utils.hpp"
#include "../context/Context.hpp"
#include "../dataItem/DataItemString.hpp"
#include <xercesc/util/XMLUniDefs.hpp>

FunctionNormalizeSpace::FunctionNormalizeSpace(const vector<const DataItem*> &args)
  : XPathFunction(0, 1, args)
{
}

DataItem *FunctionNormalizeSpace::execute(const Context *context) const
{
  XMLCh *result = NULL;
  try
  {
  if(getNumArgs() == 1) {
      result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(getStringArg(0, context));
  }
  else {
      result = v2Utils::getStringFromNode(context->getCurrentContext());
  }

    // Set whitespace to 1 here to ensure no space is placed at the start
    int whitespace = 1;

    const XMLCh *in = result;
    XMLCh *out = result;

    while(*in!=XERCES_CPP_NAMESPACE_QUALIFIER chNull) {
      XMLCh ch = *in++;

      if(ch==XERCES_CPP_NAMESPACE_QUALIFIER chHTab || ch==XERCES_CPP_NAMESPACE_QUALIFIER chLF || ch==XERCES_CPP_NAMESPACE_QUALIFIER chCR || ch==XERCES_CPP_NAMESPACE_QUALIFIER chSpace)
      {
      whitespace++;
        ch = XERCES_CPP_NAMESPACE_QUALIFIER chSpace;
    }
      else
      {
      whitespace = 0;
    }

      if(whitespace < 2)
      {
        *out++ = ch;
  }
  }

    if(out!=result && out[-1]==XERCES_CPP_NAMESPACE_QUALIFIER chSpace)
  {
      --out;
    }

    *out++ = XERCES_CPP_NAMESPACE_QUALIFIER chNull;
     }
  catch(...)
  {
    delete result;
    throw;
  }

  return new DataItemString(result);
}


#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

// Test functions

void FunctionNormalizeSpace::setUp()
{

  // Nothing to do

}

FunctionNormalizeSpace::FunctionNormalizeSpace(std::string name)
  : XPathFunction(0, 1, vector<const DataItem*>())
{




}

// Not tests
void FunctionNormalizeSpace::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("\n\n\n\n\n  I      am    a silly      man     ")));
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("  I\t\t\t      am\n\n\n\n\n\n    a silly      man\n\n\n\n     ")));
  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Dead dogs lie all over the lawn")));
  auto_ptr<DataItem> test4(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("\n\n\n\n\t\t\t\t\t\t\t               ")));
  _args.push_back(test1.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result1->asString(context),X("I am a silly man"))==0);

  _args.clear();
  _args.push_back(test2.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X("I am a silly man"))==0);
  _args.clear();
  _args.push_back(test3.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X("Dead dogs lie all over the lawn"))==0);
  _args.clear();
  _args.push_back(test4.get());


  auto_ptr<DataItem> result4(execute(context));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result4->asString(context),X(""))==0);
  _args.clear();
}

//*********************
#endif
