/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  normalize-space function

*/

#ifndef _FUNCTIONNORMALIZESPACE_HPP
#define _FUNCTIONNORMALIZESPACE_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** lang function. */
class PATHAN_EXPORT FunctionNormalizeSpace : public XPathFunction
{
public:
  FunctionNormalizeSpace(const vector<const DataItem*> &args);

  /** **/

  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionNormalizeSpace(std::string name);
  ///Test setup
  void setUp();
  ///Test normalize-space
  void testExecute();
#endif
};

#endif // _FUNCTIONNORMALIZESPACE_HPP

/* This combination of hashes and quotes confuses gcc 3.3's preprocessor

#if 0
<userDoc>
<section>
  <title><command>normalize-space</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>normalize-space(argument1) -> boolean</emphasis>
argument1 = string #PCDATA #OPTIONAL : current context
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
     The normalize-space function returns the argument string with
     whitespace normalized by stripping leading and trailing whitespace
     and replacing sequences of whitespace characters by a single space.
     If the argument is omitted, it defaults to the context node
     converted to a string, in other words the string-value of the
     context node.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <data><salmon>  mayfly do     not

      like

      it	in		here</salmon></data>

      <output>
      SALMON<with node="\xs:dd\xs:salmon">#normalize-space()#</with>BELLY


      BEGIN# normalize-space( "     I am     a



        silly     fish    ") #END
      BEGIN# normalize-space( "There     is
      		no    business  like fish-mongery") #END
      BEGIN# normalize-space( "	Weather 			Report		" ) #END
      NOT# normalize-space( "
      " ) #HING
      BEGIN# normalize-space( "




				     " ) #END
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      BEGINI am a silly fishEND
      BEGINThere is no business like fish-mongeryEND
      BEGINWeather ReportEND
      NOTHING
      BEGINEND
    </screen>
    </para>
  </section>
</section>
</userDoc>
#endif
*/
