/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpDivide operator function

*/

#ifndef _FUNCTIONOPDIVIDE_HPP
#define _FUNCTIONOPDIVIDE_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpDivide operator function*/
class PATHAN_EXPORT FunctionOpDivide : public XPathFunction
{
public:
  FunctionOpDivide(const vector<const DataItem*> &args);

  /** Divides two dataitems. **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpDivide(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPDIVIDE_HPP


#if 0
<userDoc>
<section>
  <title><command>divide</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>divide(numerator, denominator) -> numeral</emphasis>
  numerator   = #PCDATA #REQUIRED
  denominator = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the value of 'numerator' divided by 'denominator'. The function may also
    be represented using 'div' or '/' (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>
      # divide (123, 0) #
      # divide ("123", 123) #
      # divide ("123", 246) #
      # "123" / "369" #
      # "numerator" div "denominator" #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      inf
      1
      0.5
      0.333333333
      nan
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






