/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpEqualsEquals operator function

*/

#ifndef _FUNCTIONOPEQEQ_HPP
#define _FUNCTIONOPEQEQ_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpEqualsEquals operator function*/
class PATHAN_EXPORT FunctionOpEqualsEquals : public XPathFunction
{
public:
  FunctionOpEqualsEquals(const vector<const DataItem*> &args);

  /** Returns whether two dataItems are equal. */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpEqualsEquals(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPPLUS_HPP


#if 0
<userDoc>
<section>
  <title><command>equals_equals</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>equals_equals(argument1, argument2) -> boolean</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns true if the numerical value of argument1 and argument2 are the same.
    Otherwise false is returned. This function may also be represented using '==' (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>
      a # equals_equals (123, 0) #
      b # equals_equals ("123", 123) #
      c # "123" == "369" #
      d # "123" == "123" #
      e # "numerator" == "numerator" #
      f # "numerator" == "nu1merator" #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 0
      b 1
      c 0
      d 1
      e 1
      f 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






