/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpGreaterThan operator function

*/

#ifndef _FUNCTIONOPGT_HPP
#define _FUNCTIONOPGT_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpLessThan operator function*/
class PATHAN_EXPORT FunctionOpGreaterThan : public XPathFunction
{
public:
  FunctionOpGreaterThan(const vector<const DataItem*> &args);

  /** OpLessThan Takes two DataItems a and b and returns (a > b)*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpGreaterThan(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPGT_HPP

#if 0
<userDoc>
<section>
  <title><command>greater_than</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>greater_than(argument1, argument2) -> boolean</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns true if the numerical value of argument1 is greater than the numerical
    value of argument2. Note that this function will not perform a string comparison. This function
    may be represented by 'gt' (see example below)
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>

      a # greater_than("5","4") #
      b # greater_than("4","5") #
      c # greater_than(4,4) #
      d # greater_than("one", "two") #
      e # "two" gt "one" #

      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1
      b 0
      c 0
      d 0
      e 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







