/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionOpGreaterThanEqual.hpp"
#include "../dataItem/DataItemBool.hpp"

FunctionOpGreaterThanEqual::FunctionOpGreaterThanEqual(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionOpGreaterThanEqual::execute(const Context *context) const
{
  return new DataItemBool(getDataItemArg(0, context)->isGreaterThan(getDataItemArg(1, context), context) || getDataItemArg(0, context)->isEqual(getDataItemArg(1, context), context));
}

#ifdef TEST
// Test functions
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
void FunctionOpGreaterThanEqual::setUp()
{

  // Nothing to do

}

FunctionOpGreaterThanEqual::FunctionOpGreaterThanEqual( std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{






}

void FunctionOpGreaterThanEqual::testExecute()
{

  //only a sample test here as the functionality is provided in DataItem

  Context c;
  Context *context = &c;

  auto_ptr<DataItemNumeric> num(new DataItemNumeric(1));
  DataItem *item1 = num.get();

  auto_ptr<DataItemNumeric> num2(new DataItemNumeric(2));
  DataItem *item2 = num2.get();

  auto_ptr<DataItemNumeric> num3(new DataItemNumeric(2));
  DataItem *item3 = num3.get();

  _args.push_back(item1);
  _args.push_back(item2);


  auto_ptr<DataItem> result(execute(context));
  assert(result->asBool(context) == false);

  _args.clear();
  _args.push_back(item2);
  _args.push_back(item1);


  auto_ptr<DataItem> result2(execute(context));
  assert(result2->asBool(context) == true);

  _args.clear();
  _args.push_back(item2);
  _args.push_back(item3);


  auto_ptr<DataItem> result3(execute(context));
  assert(result3->asBool(context) == true);

  _args.clear();
}

#endif
