/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpGreaterThanEqual operator function

*/

#ifndef _FUNCTIONOPGTEQ_HPP
#define _FUNCTIONOPGTEQ_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** OpGreaterThanEqual operator function*/
class PATHAN_EXPORT FunctionOpGreaterThanEqual : public XPathFunction
{
public:
  FunctionOpGreaterThanEqual(const vector<const DataItem*> &args);

  /** OpGreaterThanEqual Takes two DataItems a and b and returns (a < b)*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpGreaterThanEqual(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPGTEQ_HPP

#if 0
<userDoc>
<section>
  <title><command>greater_than_equal</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>greater_than_equal(argument1, argument2) -> boolean</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns true if the numerical value of argument1 is greater than or equal to, the
    numerical value of argument2. Note that this function will not perform a string comparison. This
    function may also be represented by 'gte' (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>

      a # greater_than_equal("5","4") #
      b # greater_than_equal("4","5") #
      c # greater_than_equal(4,4) #
      d # greater_than_equal("one", "two") #
      e # "two" gte "one" #

      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1
      b 0
      c 1
      d 0
      e 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







