/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionOpLessThan.hpp"
#include "../dataItem/DataItemBool.hpp"

FunctionOpLessThan::FunctionOpLessThan(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionOpLessThan::execute(const Context *context) const
{
  return new DataItemBool(getDataItemArg(0, context)->isLessThan(getDataItemArg(1, context), context));
}

#ifdef TEST
// Test functions
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"

void FunctionOpLessThan::setUp()
{

  // Nothing to do

}

FunctionOpLessThan::FunctionOpLessThan( std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{






}

void FunctionOpLessThan::testExecute()
{

  //only a sample test here as the functionality is provided in DataItem

  Context c;
  Context *context = &c;

  auto_ptr<DataItemNumeric> num(new DataItemNumeric(1));
  DataItem *item1 = num.get();

  auto_ptr<DataItemNumeric> num2(new DataItemNumeric(2));
  DataItem *item2 = num2.get();

  _args.push_back(item1);
  _args.push_back(item2);


  auto_ptr<DataItem> result(execute(context));
  assert(result->asBool(context));
  assert(result->asBool(context) == true);


  _args.clear();
  _args.push_back(item2);
  _args.push_back(item1);


  auto_ptr<DataItem> result2(execute(context));
  assert(result2->asBool(context) == false);

  _args.clear();
}


#endif
