/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpLessThan operator function

*/

#ifndef _FUNCTIONOPLTEQ_HPP
#define _FUNCTIONOPLTEQ_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpLessThanEqual operator function*/
class PATHAN_EXPORT FunctionOpLessThanEqual : public XPathFunction
{
public:
  FunctionOpLessThanEqual(const vector<const DataItem*> &args);

  /** OpLessThanEqual Takes two DataItems a and b and returns (a <= b)*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpLessThanEqual(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPLTEQ_HPP

#if 0
<userDoc>
<section>
  <title><command>less_than_equal</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>less_than_equal(argument1, argument2) : Boolean</emphasis>
  argument1 : #PCDATA #REQUIRED
  argument2 : #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns true if the numerical value of 'argument1' is less than, or equal to, the
    numerical value of 'argument2'. Note that this function will not perform a string comparison. This
    function may also be represented by 'lte' (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>

      a # less_than_equal("5","4") #
      b # less_than_equal("4","5") #
      c # less_than_equal(4,4) #
      d # less_than_equal("one", "two") #
      e # "two" lte "one" #

      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 0
      b 1
      c 1
      d 0
      e 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







