/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpMinus operator function

*/

#ifndef _FUNCTIONOPMINUS_HPP
#define _FUNCTIONOPMINUS_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpMinus operator function*/
class PATHAN_EXPORT FunctionOpMinus : public XPathFunction
{
public:
  FunctionOpMinus(const vector<const DataItem*> &args);

  /** OpMinusenates two DataItems. Takes two DataItems and returns the difference of them*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpMinus(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPPLUS_HPP

#if 0
<userDoc>
<section>
  <title><command>minus</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>minus(argument1, argument2) -> numeral</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the numeric value of argument1 minus argument2. This function may be
    represented using '-' (see example below, taking note of white-space).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>
      a # minus("5","4") #
      b # minus("4","5") #
      c # minus(4,4) #
      d # minus("one", "two") #
      e # 2 - 4 #
      f # 5 - 0 #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1
      b -1
      c 0
      d 0
      e -2
      f 5
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







