/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpMod operator function

*/

#ifndef _FUNCTIONOPMOD_HPP
#define _FUNCTIONOPMOD_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpMod operator function*/
class PATHAN_EXPORT FunctionOpMod : public XPathFunction
{
public:
  FunctionOpMod(const vector<const DataItem*> &args);

  /** OpModenates two DataItems. Takes two DataItems and returns the mod of them*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpMod(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPMOD_HPP

#if 0
<userDoc>
<section>
  <title><command>mod</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>mod(argument1, argument2) -> numeral</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns value of argument1 modulus argument2. Note that argument2 must be
    positive (when converted to an integer), otherwise a warning is given, and a null value returned.
    This function may also be represented by 'mod' or '%' (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>
      a # mod("5","4") #
      b # mod("4","5") #
      c # 4 mod 4 #
      d # 8 % 5 #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1
      b 4
      c 0
      d 3
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







