/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   OpNotEquals operator function

*/

#ifndef _FUNCTIONOPNOTEQ_HPP
#define _FUNCTIONOPNOTEQ_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** OpNotEquals operator function*/
class PATHAN_EXPORT FunctionOpNotEquals : public XPathFunction
{
public:
  FunctionOpNotEquals(const vector<const DataItem*> &args);

  /** OpNotEqualsenates two DataItems. Takes two DataItems and returns true if they are not equal*/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOpNotEquals(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOPNOTEQ_HPP

#if 0
<userDoc>
<section>
  <title><command>not_equals</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>not_equals(argument1, argument2) -> boolean</emphasis>
  argument1 = #PCDATA #REQUIRED
  argument2 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns true if argument1 and argument2 are numerically different. Otherwise, false is returned.
    Note that this will do a string comparison. This function may also be represented by 'neq' or '!=' (see
    example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
      <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
           expression-syntax="dslpath">

      <output>
      a # not_equals("5","4") #
      b # not_equals("one","two") #
      c # not_equals("one","one") #
      d # 5 neq 4 #
      e # 4 != 4 #
      </output>

      </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      a 1
      b 1
      c 0
      d 1
      e 0
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







