/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionOpPlus.hpp"
#include "../dataItem/DataItemNumeric.hpp"

FunctionOpPlus::FunctionOpPlus(const vector<const DataItem*> &args)
  : XPathFunction(2, UNLIMITED, args)
{
  // Nothing to do
}

DataItem *FunctionOpPlus::execute(const Context *context) const
{
  XSFloat result = getNumeralArg(0, context);
  for(unsigned int i = 1; i < getNumArgs(); i++) {
    result += getNumeralArg(i, context);
  }
  return new DataItemNumeric(result);
}

#ifdef TEST
// Test functions

#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"

#include "../testharness/TestFunction.hpp"

void FunctionOpPlus::setUp()
{

  // Nothing to do

}

FunctionOpPlus::FunctionOpPlus( std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{
}

void FunctionOpPlus::testExecute()
{
  Context c;
  Context *context = &c;

  auto_ptr<DataItemNumeric> num(new DataItemNumeric(2));
  DataItem *item1 = num.get();
  _args.push_back(item1);

  auto_ptr<DataItemNumeric> num2(new DataItemNumeric(3));
  DataItem *item2 = num2.get();
  _args.push_back(item2);




  auto_ptr<DataItem> result(execute(context));

  assert(result->asNum(context) == 5);

  _args.clear();
}
//*********************
#endif
