/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Or operator function

*/

#ifndef _FUNCTIONOR_HPP
#define _FUNCTIONOR_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** Or operator function*/
class PATHAN_EXPORT FunctionOr : public XPathFunction
{
public:
  FunctionOr(const vector<const DataItem*> &args);

  /** ors two booleans. Takes two booleans, and gives a boolean */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionOr(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONOR_HPP

#if 0
<userDoc>
<section>
  <title><command>or</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>or(argument1, argument2, ...) : boolean</emphasis>
  argument1  = boolean-expression  #PCDATA #REQUIRED
  argument2  = boolean-expression  #PCDATA #REQUIRED
  argument...= boolean-expression   #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function calculates the logical OR of its parameters. The return value will
    be true if any of the arguments are true. If no argument is true, then false is returned.
    May also be used as a binary operator.
    (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

      <data><halibut/></data>
      <output>

      1: # or(1,0) #
      2: # or("swordfish", 0) #
      3: # or(\xs:dd\xs:halibut, 0)#
      4: # or(\xs:dd\xs:halibut\xs:dab, 0,"","plaice")#
      5: # or("","") #
      6: # -1 or 0  #
      7: # \xs:dd\xs:halibut\xs:dab  or "conger_eel" #

      </output>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: 1
      2: 1
      3: 1
      4: 1
      5: 0
      6: 1
      7: 1
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







