/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionRound.hpp"

#include <math.h>
#include "../DOMutils/v2Utils.hpp"
#include "../defs/NumUtils.hpp"
#include "../dataItem/DataItemNumeric.hpp"

FunctionRound::FunctionRound(const vector<const DataItem*> &args): XPathFunction(1, 1, args)
{
}

DataItem *FunctionRound::execute(const Context *context) const
{
  XSFloat f = getNumeralArg(0, context);
  
  if (!NumUtils::isNaN(f) && !NumUtils::isInfinite(f) && fabs(f) != 0.0L){
    if(f < 0.0L && f > -0.5L) {
      f = -0.0L;
    } else {
      f = floor(f + 0.5L);
    }
  }

  return new DataItemNumeric(f);
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"

// Test functions

void FunctionRound::setUp()
{

  // Nothing to do

}

FunctionRound::FunctionRound(std::string name)
  : XPathFunction(1, 1, vector<const DataItem*>())
{




}

void FunctionRound::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemNumeric(5.5));
  auto_ptr<DataItem> test2(new DataItemNumeric(5.0));
  auto_ptr<DataItem> test3(new DataItemNumeric(-4.3));
  auto_ptr<DataItem> test4(new DataItemNumeric(+0.0));
  auto_ptr<DataItem> test5(new DataItemNumeric(-0.0));
  auto_ptr<DataItem> test6(new DataItemNumeric(atof("inf")));
  auto_ptr<DataItem> test7(new DataItemNumeric(atof("-inf")));
  auto_ptr<DataItem> test8(new DataItemNumeric(-0.25));

  _args.push_back(test1.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(result1->asNum(context) == 6);

  _args.clear();
  _args.push_back(test2.get()); 


  auto_ptr<DataItem> result2(execute(context));
 
  assert(result2->asNum(context) == 5);

  _args.clear();
  _args.push_back(test3.get()); 


  auto_ptr<DataItem> result3(execute(context));

  assert(result3->asNum(context) == -4);

  _args.clear();
  _args.push_back(test4.get()); 


  auto_ptr<DataItem> result4(execute(context));

  assert(!NumUtils::isNegative(result4->asNum(context)));
  assert(result4->asNum(context) == 0);
  /*
  _args.clear();
  _args.push_back(test5.get()); 


  auto_ptr<DataItem> result5(execute(context));

#ifndef WIN32
  assert(NumUtils::isNegative(result5->asNum(context)));
#endif
  assert(result4->asNum(context) == 0);
  */
  /*
  _args.clear();
  _args.push_back(test6.get()); 


  auto_ptr<DataItem> result6(execute(context));
  
  assert(NumUtils::isInfinite(result6->asNum(context)));
  assert(!NumUtils::isNegative(result6->asNum(context)));

  _args.clear();
  _args.push_back(test7.get()); 


  auto_ptr<DataItem> result7(execute(context));

  assert(NumUtils::isInfinite(result7->asNum(context)));
  assert(NumUtils::isNegative(result7->asNum(context)));
  */
  _args.clear();
  _args.push_back(test8.get()); 


  auto_ptr<DataItem> result8(execute(context));

  assert(result8->asNum(context) == XSFloat(-0.0));

  _args.clear();
}

//*********************
#endif
