/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionStartsWith.hpp"
#include "../dataItem/DataItemBool.hpp"
#include <xercesc/util/XMLUniDefs.hpp>

FunctionStartsWith::FunctionStartsWith(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionStartsWith::execute(const Context *context) const
{
  const XMLCh *search = getStringArg(0, context);
  const XMLCh *prefix = getStringArg(1, context);

  bool startswith = true;
  if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(prefix) > XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(search)) {
    // prefix is longer than search
    startswith = false;
  } else {
    // do a character wise comparison, assuming they are the same
    for(unsigned int i = 0; search[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull && prefix[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull; i++) {
      if(search[i] != prefix[i]) {
	startswith = false;
	break;
      }
    }
  }

  return new DataItemBool(startswith);
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"

// Test functions

void FunctionStartsWith::setUp()
{

  // Nothing to do

}

FunctionStartsWith::FunctionStartsWith(std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{





}

void FunctionStartsWith::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjimbobmavis")));
  _args.push_back(test1.get());
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjim")));
  _args.push_back(test2.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(result1->asBool(context));

  _args.clear();
  _args.push_back(test2.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(!result2->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(result3->asBool(context));

  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("hello")));
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test3.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(!result4->asBool(context));

  auto_ptr<DataItem> test4(new DataItemNodeList(new DeadNodeList()));
  _args.clear();
  _args.push_back(test4.get());
  _args.push_back(test4.get());


  auto_ptr<DataItem> result5(execute(context));

  assert(result5->asBool(context));

  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test4.get());


  auto_ptr<DataItem> result6(execute(context));

  assert(result6->asBool(context));

  _args.clear();
  _args.push_back(test4.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result7(execute(context));

  assert(!result7->asBool(context));

  _args.clear();
}

//*********************
#endif
