/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  String function

*/

#ifndef _FUNCTIONSTRING_HPP
#define _FUNCTIONSTRING_HPP

#include <config.h>


#include "XPathFunction.hpp"

/** String operator function. */
class PATHAN_EXPORT FunctionString : public XPathFunction
{
public:
  FunctionString(const vector<const DataItem*> &args);

  /** Returns the argument as a String **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionString(std::string name);
  ///Test setup
  void setUp();
  ///Test String on a node
  void testExecute();
#endif
};

#endif // _FUNCTIONSTRING_HPP

#if 0
<userDoc>
<section>
  <title><command>string</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>string(argument1) -> string</emphasis>
  argument1 = any #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns argument1 as a string
    (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

      <data><halibut>I am not a real fish<anchovy/></halibut></data>
      <output>

      1: # string("haddock") #
      2: # string(\xs:dd\xs:halibut) #
      3: # string(34526) #

      </output>


    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: haddock
      2: I am not a real fish
      3: 34526
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif

