/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionStrlen.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../context/Context.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include <xercesc/util/Janitor.hpp>

FunctionStrlen::FunctionStrlen(const vector<const DataItem*> &args)
  : XPathFunction(0, 1, args)
{
}

DataItem *FunctionStrlen::execute(const Context *context) const
{
  if(getNumArgs() == 1) {
    return new DataItemNumeric(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getStringArg(0, context)));
  } else {
    XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> string(v2Utils::getStringFromNode(context->getCurrentContext()));
    return new DataItemNumeric(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(string.get()));
  }
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../simpleVariables/VariableStore.hpp"

// Test functions

void FunctionStrlen::setUp()
{

  // Nothing to do

}

FunctionStrlen::FunctionStrlen(std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{






}

// Not tests
void FunctionStrlen::testExecute()
{
  Context c;
  Context *context = &c;

  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("1234567890")));
  _args.push_back(test1.get());


  auto_ptr<DataItem> result1(execute(context));

  assert(result1->asNum(context) == 10);

  _args.clear();
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("12345")));
  _args.push_back(test2.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(result2->asNum(context) == 5);

  _args.clear();
  auto_ptr<DataItem> test3(new DataItemNodeList(new DeadNodeList()));
  _args.push_back(test3.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(result3->asNum(context) == 0);

  _args.clear();
  auto_ptr<DataItem> test4(new DataItemString(NULL));
  _args.push_back(test4.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(result4->asNum(context) == 0);

  _args.clear();
}

#endif
