/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Strlen function

*/

#ifndef _FUNCTIONSTRLEN_HPP
#define _FUNCTIONSTRLEN_HPP


#include "XPathFunction.hpp"

/** Strlen function. */
class PATHAN_EXPORT FunctionStrlen : public XPathFunction
{
public:
  FunctionStrlen(const vector<const DataItem*> &args);

  /** Returns the readable length of the string argument. **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionStrlen(std::string name);
  ///Test setup
  void setUp();
  ///Test count
  void testExecute();
#endif
};

#endif // _FUNCTIONSTRLEN_HPP

#if 0
<userDoc>
<section>
  <title><command>strlen/string-length</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>strlen/string-length(argument1) -> numeral</emphasis>
  argument1 = string #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the length of argument1. If no argument is given, then the length
    of the current context is returned (i.e. the length of the total
    text content of the current context).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

        <data>
          fish smell badly in hot weather
          <halibut>
            I am not a real fish
            <anchovy/>
          </halibut>
        </data>

        <output>

        1: # strlen("haddock") #
        2: # string-length(\xs:dd\xs:halibut) #
        3: # strlen() #

        </output>



    </xst>
    ]]></programlisting>
    should produce output similar to
    <screen>
      1: 7
      2: 20
      3: 51
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif







