/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionSubstr.hpp"

#include <climits>

#include "../defs/NumUtils.hpp"
#include "../dataItem/DataItemString.hpp"
#include <xercesc/util/XMLUniDefs.hpp>

FunctionSubstr::FunctionSubstr(const vector<const DataItem*> &args)
  : XPathFunction(2, 3, args)
{
}

DataItem *FunctionSubstr::execute(const Context *context) const
{
  const XMLCh *str = getStringArg(0, context);

  int start = 0;
  const DataItem *secondArg = getDataItemArg(1, context);
  if(NumUtils::isInfinite(secondArg->asNum(context))) {
      return new DataItemString();
  } else if(NumUtils::isNaN(secondArg->asNum(context))) {
    return new DataItemString();
  } else {
    start = secondArg->asInt(context) - 1;
  }

  int end = INT_MAX;
  if(getNumArgs() > 2) {
    const DataItem *thirdArg = getDataItemArg(2, context);
    if(NumUtils::isInfinite(thirdArg->asNum(context))) {
      if(thirdArg->asNum(context) < 0) {
	return new DataItemString();
      }
    } else if(NumUtils::isNaN(thirdArg->asNum(context))) {
      return new DataItemString();
    } else {
      end = start + thirdArg->asInt(context);
    }
  }

  if(start < 0) {
    start = 0;
  }

  int length = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(str);
  if(end > length) {
    end = length;
  }

  if(end <= start) {
    return new DataItemString();
  }

  XMLCh *result = new XMLCh[end-start+1];
  XMLCh *out = result;
  while(str[start]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull && start<end)
  {
    *out++ = str[start++];
  }
  *out++ = 0;

  return new DataItemString(result);
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

// Test functions

void FunctionSubstr::setUp()
{

  // Nothing to do

}

FunctionSubstr::FunctionSubstr(std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{
}

// Not tests
void FunctionSubstr::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjimbobmavis")));
  _args.push_back(test1.get());
  auto_ptr<DataItem> start1(new DataItemNumeric(6.0));
  _args.push_back(start1.get());
  auto_ptr<DataItem> length1(new DataItemNumeric(3.4));
  _args.push_back(length1.get());

  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result1->asString(context),X("jim"))==0);
  _args.clear();
  _args.push_back(test1.get());
  auto_ptr<DataItem> start2(new DataItemNumeric(11.8));
  _args.push_back(start2.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X("mavis"))==0);
  _args.clear();
  _args.push_back(test1.get());
  auto_ptr<DataItem> start3(new DataItemNumeric(0.0));
  _args.push_back(start3.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X("frankjimbobmavis"))==0);
  _args.clear();
  _args.push_back(test1.get());
  auto_ptr<DataItem> start4(new DataItemNumeric(1.0));
  _args.push_back(start4.get());
  auto_ptr<DataItem> length4(new DataItemNumeric(0.0));
  _args.push_back(length4.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result4->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test1.get());
  auto_ptr<DataItem> start5(new DataItemNumeric(1.0));
  _args.push_back(start5.get());
  auto_ptr<DataItem> length5(new DataItemNumeric(-12.0));
  _args.push_back(length5.get());


  auto_ptr<DataItem> result5(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result5->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test1.get());
  auto_ptr<DataItem> start6(new DataItemNumeric(11.8));
  _args.push_back(start6.get());
  auto_ptr<DataItem> length6(new DataItemNumeric(100.0));
  _args.push_back(length6.get());


  auto_ptr<DataItem> result6(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result6->asString(context),X("mavis"))==0);
  _args.clear();
  auto_ptr<DataItem> test7(new DataItemNodeList(new DeadNodeList()));
  _args.push_back(test7.get());
  auto_ptr<DataItem> start7(new DataItemNumeric(6.0));
  _args.push_back(start7.get());
  auto_ptr<DataItem> length7(new DataItemNumeric(3.4));
  _args.push_back(length7.get());


  auto_ptr<DataItem> result7(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result7->asString(context),X(""))==0);
  _args.clear();
}

//*********************
#endif
