/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  Substr function

*/

#ifndef _FUNCTIONSUBSTR_HPP
#define _FUNCTIONSUBSTR_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** Substr function. */
class PATHAN_EXPORT FunctionSubstr : public XPathFunction
{
public:
  FunctionSubstr(const vector<const DataItem*> &args);

  /** Returns the substring to the right of parm2 until optional parm3 from string parm1 **/
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionSubstr(std::string name);
  ///Test setup
  void setUp();
  ///Test count
  void testExecute();
#endif
};

#endif // _FUNCTIONSUBSTR_HPP


#if 0
<userDoc>
<section>
  <title><command>substr/substring/rightstr</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>substr/substring/rightstr(argument1,argument2,argument3) -> string</emphasis>
  argument1 = string  #PCDATA #REQUIRED
  argument2 = numeral #PCDATA #REQUIRED
  argument3 = numeral #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns those characters of argument1 which start from an index
    value specified in argument2 up to the end of the string or an amount of characters
    specified in argument3. Note that all three forms (substr, substring, rightstr) are
    functionally equivalent.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

      <output>

      1: # substr("haddock", 4 ) #
      2: # substring("baracuda", 5, 3 ) #
      3: # rightstr(substring(substr("plankton", 1, 5), 1, 4), 2) #

      </output>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: dock
      2: cud
      3: lan
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






