/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionSubstringBefore.hpp"
#include "../dataItem/DataItemString.hpp"

FunctionSubstringBefore::FunctionSubstringBefore(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionSubstringBefore::execute(const Context *context) const
{
  const XMLCh *container = getStringArg(0, context);
  const XMLCh *pattern = getStringArg(1, context);

  unsigned int containerLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(container);
  unsigned int patternLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(pattern);

  bool contains = false;
  unsigned int start = 0;;
  if(patternLen==0){
	  contains=true;
  } else {
    if(containerLen >= patternLen) {
      // search pattern must be shorter than string being searched
      for (unsigned int i = 0; i <= containerLen-patternLen ; ++i){
	// find match for the first letter of the pattern
	if (container[i] == pattern[0]) {
	  contains = true;
	  start = i;
	  // check for the remaining letters of the pattern
	  for(unsigned int j = 1; j < patternLen ; ++j){
	    if (container[i+j] != pattern[j]){
	      contains = false;
	      break;
	    }
	  }
	  if(contains) {
	    break;
	  }
	}
      }
    }
  }

  if(contains){
    XMLCh *result = new XMLCh[start+1];
    XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(result, container, 0, start);
    return new DataItemString(result);
  } else {
    return new DataItemString();
  }
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

// Test functions

void FunctionSubstringBefore::setUp()
{

  // Nothing to do

}

FunctionSubstringBefore::FunctionSubstringBefore(std::string name)
  : XPathFunction(2, 2, vector<const DataItem*>())
{





}

void FunctionSubstringBefore::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjimbobmavis")));
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank")));
  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("jim")));
  auto_ptr<DataItem> test4(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("bm")));
  auto_ptr<DataItem> test5(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("h")));
  auto_ptr<DataItem> test6(new DataItemNodeList(new DeadNodeList));
  _args.push_back(test1.get());
  _args.push_back(test2.get());

  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result1->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test2.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test3.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result4->asString(context),X("frank"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test4.get());


  auto_ptr<DataItem> result5(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result5->asString(context),X("frankjimbo"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test5.get());


  auto_ptr<DataItem> result6(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result6->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test6.get());


  auto_ptr<DataItem> result7(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result7->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test6.get());
  _args.push_back(test1.get());


  auto_ptr<DataItem> result8(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result8->asString(context),X(""))==0);
  _args.clear();
  _args.push_back(test6.get());
  _args.push_back(test6.get());


  auto_ptr<DataItem> result9(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result9->asString(context),X(""))==0);
  _args.clear();
}

//*********************
#endif
