/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionSumNodes.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../defs/NumUtils.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../context/Context.hpp"
#include <xercesc/util/Janitor.hpp>
#include <memory>

FunctionSumNodes::FunctionSumNodes(const vector<const DataItem*> &args)
  : XPathFunction(1, 1, args)
{
}

DataItem *FunctionSumNodes::execute(const Context *context) const
{
  auto_ptr<DeadNodeList> nlist(getNodeListArg(0, context));
  XSFloat sum = 0;
  for(DeadNodeList::const_iterator i = nlist->begin();
      i != nlist->end(); ++i) {
    XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> string(v2Utils::getStringFromNode(*i));
    sum += NumUtils::stringAsDouble(string.get());
  }
  return new DataItemNumeric(sum);
}

#ifdef TEST
// Test functions

#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../DOMutils/XStr.hpp"

#include "../testharness/TestFunction.hpp"

void FunctionSumNodes::setUp()
{

  // Nothing to do

}

FunctionSumNodes::FunctionSumNodes(std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{





}

void FunctionSumNodes::testExecute()
{


  DeadNodeList *wibble= new DeadNodeList;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *impl = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> doc(impl->createDocument(X("http://www.xmlscript.org/2.0"), X("xs:__data__"), NULL));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *temp = NULL;
  temp = doc->createAttribute(X("node"));
  temp->setNodeValue(X("1"));
  wibble->push_back(temp);
  temp = doc->createAttribute(X("node"));
  temp->setNodeValue(X("2"));
  wibble->push_back(temp);
  temp = doc->createAttribute(X("node"));
  temp->setNodeValue(X("3"));
  wibble->push_back(temp);
  temp = doc->createAttribute(X("node"));
  temp->setNodeValue(X("4"));
  wibble->push_back(temp);
  temp = doc->createAttribute(X("node"));
  temp->setNodeValue(X("5"));
  wibble->push_back(temp);

  auto_ptr<DataItemNodeList> wibbledi(new DataItemNodeList(wibble));
  _args.push_back(wibbledi.get());

  Context context;
  auto_ptr<DataItem> result(execute(&context));
  assert(result->asInt(&context) == 15);

  _args.clear();
}
#endif
