/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
   SumNodes function -- takes a nodelist, evaluates it's nodes
   as numbers and sums them
*/

#ifndef _FUNCTIONSUMNODES_HPP
#define _FUNCTIONSUMNODES_HPP


#include "XPathFunction.hpp"

/** SumNodes operator function*/
class PATHAN_EXPORT FunctionSumNodes : public XPathFunction
{
public:
  FunctionSumNodes(const vector<const DataItem*> &args);

  /** Takes a list of nodes and sums their values */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionSumNodes(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONSUMNODES_HPP

#if 0
<userDoc>
<section>
  <title><command>sumNodes</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>sumNodes(argument1) -> numeral</emphasis>
  argument1 = expression-nodelist  #PCDATA #OPTIONAL
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function converts each node in the given nodelist to a number, and returns
    the sum of these values.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

        <data>59
          <halibut>500
            <anchovy/>
          </halibut>
          <halibut name = "jules">500
            <pollock>25</pollock>
            <pollock>25</pollock>
            <pollock>25</pollock>
          </halibut>
          <halibut>1000</halibut>
        </data>

        <output>
        1: # sumNodes(\xs:dd\xs:halibut) #
        2: # sumNodes(\xs:dd\xs:halibut\xs:anchovy) #
        3: # sumNodes() #
        <with node="\xs:dd\xs:halibut{.name == 'jules'}">4: # sumNodes(xs:pollock) #
        </with >

        </output>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: 2000
      2: 0
      3: 59
      4: 75
    </screen>
    </para>
  </section>
</section>
</userDoc>




#endif



