/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   Tag function

*/

#ifndef _FUNCTIONTAG_HPP
#define _FUNCTIONTAG_HPP

#include <config.h>

#include "XPathFunction.hpp"
#include "../dataItem/DeadNodeList.hpp"

/** Tag function */
class PATHAN_EXPORT FunctionTag : public XPathFunction
{
public:
  FunctionTag(const vector<const DataItem*> &args);

  /** Returns the namespace unaware name of the node. Takes an optional nodelist,
      and returns an empty string if the node has no name. Uses the current context
      if a nodelist is not specified. */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionTag(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONTAG_HPP


#if 0
<userDoc>
<section>
  <title><command>tag</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>tag(argument1) -> string</emphasis>
  argument1 = nodelist-expression  #PCDATA #OPTIONAL : current context
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the namespace aware name of the first node in
    the node list. The default behaviour, when no argument is given, is to return the
    namespace aware name of the current context.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

      <data><xs:halibut><xs:anchovy/></xs:halibut></data>
      <output>

      2: # tag() #
      3: # tag(\xs:dd\xs:halibut) #
      4: # tag(\xs:dd\xs:halibut\xs:anchovy) #
          <foreach node="\xs:dd\xs:halibut"><output>
          1: # tag() #</output></foreach>
      </output>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: xs:halibut

      2: xs:dd
      3: xs:halibut
      4: xs:anchovy
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif






