/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionTranslate.hpp"
#include <xercesc/util/XMLUniDefs.hpp>
#include "../dataItem/DataItemString.hpp"
#include <xercesc/util/Janitor.hpp>

#include <map>
#include <set>

FunctionTranslate::FunctionTranslate(const vector<const DataItem*> &args)
  : XPathFunction(3, 3, args)
{
}

DataItem *FunctionTranslate::execute(const Context *context) const
{
  const XMLCh *source = getStringArg(0, context);
  const XMLCh *search = getStringArg(1, context);
  const XMLCh *replace = getStringArg(2, context);

  typedef map<XMLCh,XMLCh> Translation;
  Translation translation;

  unsigned int i = 0;
  while(search[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull && replace[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull)
  {
    translation[search[i]] = replace[i];
    ++i;
  }
  while(search[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull)
  {
    translation[search[i]] = XERCES_CPP_NAMESPACE_QUALIFIER chNull;
    ++i;
  }

  XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> result(new XMLCh[XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(source)+1]);
  XMLCh *dest = result.get();
    while(*source!=XERCES_CPP_NAMESPACE_QUALIFIER chNull)
    {
      Translation::const_iterator translateI = translation.find(*source);
      if(translateI==translation.end())
      {
        *dest++ = *source;
      }
      else if(translateI->second!=XERCES_CPP_NAMESPACE_QUALIFIER chNull)
      {
        *dest++ = translateI->second;
      }
      ++source;
      }
    *dest++ = XERCES_CPP_NAMESPACE_QUALIFIER chNull;

  return new DataItemString(result.release());
}

#ifdef TEST

#include <xercesc/dom/DOMDocument.hpp>
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

// Test functions

void FunctionTranslate::setUp()
{

  // Nothing to do

}

FunctionTranslate::FunctionTranslate(std::string name)
  : XPathFunction(3, 3, vector<const DataItem*>())
{






}

void FunctionTranslate::testExecute()
{
  auto_ptr<DataItem> test1(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frankjimbobmavis")));
  auto_ptr<DataItem> test2(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("abc")));
  auto_ptr<DataItem> test3(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("ABC")));
  auto_ptr<DataItem> test4(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("abci")));
  auto_ptr<DataItem> test5(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("bcd")));
  auto_ptr<DataItem> test6(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("aaa")));
  _args.push_back(test1.get());
  _args.push_back(test2.get());
  _args.push_back(test3.get());


  Context c;
  Context *context = &c;

  auto_ptr<DataItem> result1(execute(context));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result1->asString(context),X("frAnkjimBoBmAvis"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test4.get());
  _args.push_back(test3.get());


  auto_ptr<DataItem> result2(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X("frAnkjmBoBmAvs"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test2.get());
  _args.push_back(test5.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X("frbnkjimcocmbvis"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test6.get());
  _args.push_back(test5.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result4->asString(context),X("frdnkjimbobmdvis"))==0);
  _args.clear();
  _args.push_back(test1.get());
  _args.push_back(test5.get());
  _args.push_back(test4.get());


  auto_ptr<DataItem> result5(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result5->asString(context),X("frankjimaoamavis"))==0);
  _args.clear();
}

//*********************
#endif
