/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

   Unary Minus operator function

*/

#ifndef _FUNCTIONUNARYMINUS_HPP
#define _FUNCTIONUNARYMINUS_HPP


#include "XPathFunction.hpp"

/** Unary Minus operator function*/
class PATHAN_EXPORT FunctionUnaryMinus : public XPathFunction
{
public:
  FunctionUnaryMinus(const vector<const DataItem*> &args);

  /** The unary minus operator function. */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionUnaryMinus(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONUNARYMINUS_HPP

#if 0
<userDoc>
 <section>
  <title><command>unaryMinus</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>unaryMinus(argument1) -> negation</emphasis>
  argument1 = #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function returns the negative value of the given argument.
    (see example below).
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

      <data><halibut>45</halibut></data>
      <output>

      1: # unaryMinus(1) #
      2: # unaryMinus("56") #
      3: # unaryMinus(\xs:dd\xs:halibut) #
      4: # unaryMinus(-345) #
      5: # unaryMinus("-4") #


      </output>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
      1: -1
      2: -56
      3: -45
      4: 345
      5: 4
    </screen>
    </para>
  </section>
</section>
</userDoc>

#endif // _FUNCTIONUNARYMINUS_HPP






