/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
   SumNodes function -- takes a nodelist, evaluates it's nodes
   as numbers and sums them
*/

#ifndef _FUNCTIONUNION_HPP
#define _FUNCTIONUNION_HPP

#include <config.h>

#include "XPathFunction.hpp"

/** Union operator function*/
class PATHAN_EXPORT FunctionUnion : public XPathFunction
{
public:
  FunctionUnion(const vector<const DataItem *> &args);

  /** Takes two lists and returns the union of them (with each node unique in the resulting list) */
  DataItem *execute(const Context *context) const;

#ifdef TEST
  ///Dummy constructor for test suite
  FunctionUnion(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testExecute();
#endif
};

#endif // _FUNCTIONUNION_HPP

#if 0
<userDoc>
<section>
  <title><command>union</command></title>

  <section>
  <title>Synopsis</title>
  <para>
  <synopsis>
  <emphasis>union(argument1, argument2) -> expression-nodelist</emphasis>
  argument1 = expression-nodelist  #PCDATA #REQUIRED
  argument2 = expression-nodelist  #PCDATA #REQUIRED
  </synopsis>
  </para>
  </section>

  <section>
    <title>Description</title>
    <para>
    This function takes two node lists, merges them together, and returns that result.
    The node lists are merged such that there are no duplicate nodes. There is no
    guaranteed ordering of the resulting list.
    </para>
  </section>

  <section>
    <title>Examples</title>
    <para>
    The program:
    <programlisting><![CDATA[
    <xst xmlns="http://www.xmlscript.org/2.0" xmlns:xs="http://www.xmlscript.org/2.0"
         expression-syntax="dslpath">

    <data>

    <tuna size="1" age="3"/>
    <salmon size="1" age="3"/>
    <trout size="1" age="2"/>
    <bass size="1" age="2"/>
    <pike size="2" age="2"/>
    <trout size="2" age="2"/>

    <fishBucket/>

    </data>

    <insert into="\xs:dd\xs:fishBucket">
    <eval>  union(\xs:dd\**{.size == "1"}, \xs:dd\**{.age == "2"}) </eval>
    </insert>

    <foreach node="\xs:dd\xs:fishBucket\*:**">
    <output><eval> . </eval>
    </output>
    </foreach>

    </xst>
    ]]></programlisting>
    should produce the output
    <screen>
    <x_ns0:tuna size="1" age="3" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    <x_ns0:salmon size="1" age="3" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    <x_ns0:trout size="1" age="2" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    <x_ns0:bass size="1" age="2" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    <x_ns0:pike size="2" age="2" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    <x_ns0:trout size="2" age="2" xmlns:x_ns0="http://www.xmlscript.org/2.0"/>
    </screen>
    </para>
  </section>
</section>
</userDoc>




#endif



