/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "GeneralFunction.hpp"
#include "../defs/XMLScriptTypeDefs.hpp"
#include "../exceptions/FunctionException.hpp"


const unsigned int GeneralFunction::UNLIMITED = 40000000; // A very large number

GeneralFunction::GeneralFunction(unsigned int argsFrom, unsigned int argsTo,
				 const vector<const DataItem*> &args)
  : DataItemFunction(args)
{
#ifndef TEST
  if((argsFrom != UNLIMITED && argsFrom > args.size()) ||
     (argsTo != UNLIMITED && args.size() > argsTo)) {
    throw FunctionException("GeneralFunction constructor","Wrong number of arguments");
  }
#endif
}

GeneralFunction::~GeneralFunction()
{
}

bool GeneralFunction::getBoolArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asBool(context);
}

const XMLCh *GeneralFunction::getStringArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asString(context);
}

XSFloat GeneralFunction::getNumeralArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asNum(context);
}

DeadNodeList *GeneralFunction::getNodeListArg(unsigned int argNum,
                                                    const Context *context) const
{
  return _args[argNum]->asNodeList(context, false);
}

const DataItem *GeneralFunction::getDataItemArg(unsigned int argNum,
                                                const Context *context) const
{
  return _args[argNum];
}

int GeneralFunction::getIntArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asInt(context);
}

unsigned int GeneralFunction::getNumArgs() const
{
  return _args.size();
}

#ifdef TEST
void GeneralFunction::setUp()
{
}
#endif


















