/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _GENERALFUNCTION_HPP
#define _GENERALFUNCTION_HPP

#ifdef WIN32
#pragma warning( disable: 4786 )
#endif

#include "../dataItem/DataItemFunction.hpp"

/** Virtual interface class for functions */
class PATHAN_EXPORT GeneralFunction : public DataItemFunction
{
public:
  /// constructor, checks for the correct number of arguments.
  GeneralFunction(unsigned int argsFrom, unsigned int argsTo,
		  const vector<const DataItem*> &args);
  virtual ~GeneralFunction();

protected:
  /** returns argument argNum as a boolean */
  bool getBoolArg(unsigned int argNum, const Context *context) const;
  /** returns argument argNum as a string */
  const XMLCh *getStringArg(unsigned int argNum,
			 const Context *context) const;
  /** returns argument argNum as a numeral */
  XSFloat getNumeralArg(unsigned int argNum, const Context *context) const;
  /** returns argument argNum as a nodelist, that the callet _must_ delete */
  DeadNodeList *getNodeListArg(unsigned int argNum,
			      const Context *context) const;
  /** returns argument argNum as a data item. caller does NOT own */
  const DataItem *getDataItemArg(unsigned int argNum,
                                 const Context *context) const;
  /** returns argument argNum as an integer */
  int getIntArg(unsigned int argNum, const Context *context) const;

  /** returns the number of parameters passed into the function */
  unsigned int getNumArgs() const;

  static const unsigned int UNLIMITED;

#ifdef TEST
public:
  void setUp();
#endif
};

#endif // _GENERALFUNCTION_HPP














