/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathFunction.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../defs/NumUtils.hpp"
#include <memory>
#include <xercesc/util/Janitor.hpp>

XPathFunction::XPathFunction(unsigned int argsFrom, unsigned int argsTo,
				 const vector<const DataItem*> &args)
  : GeneralFunction(argsFrom, argsTo, args)
{
  // Nothing to do
}

bool XPathFunction::getBoolArg(unsigned int argNum,
			      const Context *context) const
{
  //No need for special case, as makes no difference
  return _args[argNum]->asBool(context);
}

const XMLCh *XPathFunction::getStringArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asString(context);
}

XSFloat XPathFunction::getNumeralArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asNum(context);
}

DeadNodeList *XPathFunction::getNodeListArg(unsigned int argNum,
			      const Context *context) const
{
  return _args[argNum]->asNodeList(context, false);
}

const DataItem *XPathFunction::getDataItemArg(unsigned int argNum,
                                              const Context *context) const
{
  return _args[argNum];
}

int XPathFunction::getIntArg(unsigned int argNum,
			      const Context *context) const
{
  auto_ptr<DataItem> valueItem
     = static_cast< auto_ptr<DataItem> >(_args[argNum]->asSimpleDataItem(context));

  XSFloat d = 0.0L;

  if(valueItem->getType() == DataItem::NODELIST) {
    auto_ptr<DeadNodeList> nl(valueItem->asNodeList(context, false));
    if(nl->getLength() == 0) {
      return 0; //loss of information
    } else {
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> string(v2Utils::getStringFromNode(nl->first()));
      d = NumUtils::stringAsDouble(string.get());
    }
  } else {
    d = valueItem->asNum(context);
  }

  if(d < 0) {
    d -= 0.5;
    return static_cast<int>(ceil(d));
  }
  else {
    d += 0.5;
    return static_cast<int>(floor(d));
  }
}
