/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathFunctionManifest.hpp"
#include "../factory/FuncFactoryTemplate.hpp"

// Node-set functions
#include "../functionAPI/FunctionLast.hpp"
#include "../functionAPI/FunctionPosition.hpp"
#include "../functionAPI/FunctionCount.hpp"
#include "../functionAPI/FunctionId.hpp"
#include "../functionAPI/FunctionLocalname.hpp"
#include "../functionAPI/FunctionNamespace.hpp"
#include "../functionAPI/FunctionTag.hpp"

// String functions
#include "../functionAPI/FunctionString.hpp"
#include "../functionAPI/FunctionConcat.hpp"
#include "../functionAPI/FunctionStartsWith.hpp"
#include "../functionAPI/FunctionContains.hpp"
#include "../functionAPI/FunctionSubstringBefore.hpp"
#include "../functionAPI/FunctionSubstringAfter.hpp"
#include "../functionAPI/FunctionSubstr.hpp"
#include "../functionAPI/FunctionStrlen.hpp"
#include "../functionAPI/FunctionNormalizeSpace.hpp"
#include "../functionAPI/FunctionTranslate.hpp"

// Boolean functions
#include "../functionAPI/FunctionBoolean.hpp"
#include "../functionAPI/FunctionNot.hpp"
#include "../functionAPI/FunctionTrue.hpp"
#include "../functionAPI/FunctionFalse.hpp"
#include "../functionAPI/FunctionLang.hpp"

// Number functions
#include "../functionAPI/FunctionNumber.hpp"
#include "../functionAPI/FunctionSumNodes.hpp"
#include "../functionAPI/FunctionFloor.hpp"
#include "../functionAPI/FunctionCeiling.hpp"
#include "../functionAPI/FunctionRound.hpp"

void XPathFunctionManifest::populate(FunctionLookup *lookup)
{
   // Node-set functions
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("last"),
                          new FuncFactoryTemplate<FunctionLast>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("position"),
                          new FuncFactoryTemplate<FunctionPosition>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("count"),
                          new FuncFactoryTemplate<FunctionCount>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("id"),
                          new FuncFactoryTemplate<FunctionId>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("local-name"),
                          new FuncFactoryTemplate<FunctionLocalname>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("namespace-uri"),
                          new FuncFactoryTemplate<FunctionNamespace>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("name"),
                          new FuncFactoryTemplate<FunctionTag>());

   // String functions
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("string"),
                          new FuncFactoryTemplate<FunctionString>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("concat"),
                          new FuncFactoryTemplate<FunctionConcat>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("starts-with"),
                          new FuncFactoryTemplate<FunctionStartsWith>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("contains"),
                          new FuncFactoryTemplate<FunctionContains>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("substring-before"),
                          new FuncFactoryTemplate<FunctionSubstringBefore>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("substring-after"),
                          new FuncFactoryTemplate<FunctionSubstringAfter>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("substring"),
                          new FuncFactoryTemplate<FunctionSubstr>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("string-length"),
                          new FuncFactoryTemplate<FunctionStrlen>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("normalize-space"),
                          new FuncFactoryTemplate<FunctionNormalizeSpace>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("translate"),
                          new FuncFactoryTemplate<FunctionTranslate>());

   // Boolean functions
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("boolean"),
                          new FuncFactoryTemplate<FunctionBoolean>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("not"),
                          new FuncFactoryTemplate<FunctionNot>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("true"),
                          new FuncFactoryTemplate<FunctionTrue>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("false"),
                          new FuncFactoryTemplate<FunctionFalse>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("lang"),
                          new FuncFactoryTemplate<FunctionLang>());

   // Number functions
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("number"),
                          new FuncFactoryTemplate<FunctionNumber>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("sum"),
                          new FuncFactoryTemplate<FunctionSumNodes>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("floor"),
                          new FuncFactoryTemplate<FunctionFloor>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("ceiling"),
                          new FuncFactoryTemplate<FunctionCeiling>());
   lookup->insertFunction(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("round"),
                          new FuncFactoryTemplate<FunctionRound>());
}
