/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathNSResolverImpl.hpp"


XPathNSResolverImpl::XPathNSResolverImpl(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *resNode)
{
	//Create the map of namspaces that are in scope of the given node
	namespaceBindings = v2NSUtils::createNamespaceMap(resNode);

	resolverNode = resNode;
}//constructor


XPathNSResolverImpl::~XPathNSResolverImpl()
{
  for(namespaceMapType::iterator namespaceI = namespaceBindings->begin();
      namespaceI != namespaceBindings->end();
      ++namespaceI)
  {
    delete[] (XMLCh *)namespaceI->first;
    delete[] (XMLCh *)namespaceI->second;
  }
  delete namespaceBindings, namespaceBindings=NULL;
}//destructor


const XMLCh *XPathNSResolverImpl::lookupNamespaceURI(const XMLCh *prefix) const
{
	namespaceMapType::const_iterator resolvedPrefix = namespaceBindings->find(prefix);
	if(resolvedPrefix == namespaceBindings->end()) {
		return NULL;
	}
	return resolvedPrefix->second;

	// Note: the above produces the same as:
	//   return namespaceBindings[prefix];
	// but does not make new entries in the map
}//lookupNamespaceURI

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *XPathNSResolverImpl::getResolverNode() const
{
  return resolverNode;
}//getResolverNode
