/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathNamespaceImpl.hpp"
#include "../dom3-xpath/XPathNamespace.hpp"

#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/impl/DOMDocumentImpl.hpp>
#include <xercesc/util/XercesVersion.hpp>
#include <xercesc/dom/impl/DOMCasts.hpp>
#include <xercesc/dom/impl/DOMNodeImpl.hpp>

//Main constructor
XPathNamespaceImpl::XPathNamespaceImpl(const XMLCh *nsPrefix, const XMLCh *nsUri,
                                       XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *owner, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *docOwner)
   : fNode(docOwner)
{
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeImpl *argImpl = /*castToNodeImpl(this)*/&fNode;
    argImpl->fOwnerNode = owner;
    argImpl->isOwned(true);
    argImpl->setIsLeafNode(true);

    XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl *docImpl = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl *)docOwner;

    uri = docImpl->getPooledString(nsUri);
    prefix = docImpl->getPooledString(nsPrefix);

    //	this->ownerElement = owner;
}//constructor


//Copy constructor
XPathNamespaceImpl::XPathNamespaceImpl(const XPathNamespaceImpl &other)
	: fNode(other.fNode), uri(other.uri), prefix(other.prefix)
{
}//copy constructor


//Destructor
XPathNamespaceImpl::~XPathNamespaceImpl()
{
	//Nothing
}//destructor


XERCES_CPP_NAMESPACE_QUALIFIER DOMNode * XPathNamespaceImpl::cloneNode(bool deep) const
{
	throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NOT_SUPPORTED_ERR;
}//cloneNode

const XMLCh *XPathNamespaceImpl::getNodeName() const
{
	return prefix;
}//getNodeName

short XPathNamespaceImpl::getNodeType() const
{
  //	return XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::XPATH_NAMESPACE_NODE;
  return XPATH_NAMESPACE_NODE;
}//getNodeType

const XMLCh *XPathNamespaceImpl::getPrefix() const
{
	return prefix;
}//getPrefix


const XMLCh *XPathNamespaceImpl::getNamespaceURI() const
{
	return uri;
}//getNamespaceURI

XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *XPathNamespaceImpl::getOwnerElement() const
{
    return (XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *) (fNode.isOwned() ? fNode.fOwnerNode : 0);
}//getOwnerElement

const XMLCh *XPathNamespaceImpl::getNodeValue() const
{
	return NULL;
}//getNodeVale

void XPathNamespaceImpl::setNodeValue(const XMLCh *value)
{
	throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
}//setNodeValue


void* XPathNamespaceImpl::setUserData(const XMLCh* key, void* data, XERCES_CPP_NAMESPACE_QUALIFIER DOMUserDataHandler* handler)
{
	throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
}

void XPathNamespaceImpl::setPrefix(const XMLCh *nsPrefix)
{
	throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
}//setPrefix




//Should be moved into XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument
//Produce a new XPathNamespace object
XPathNamespace *XPathNamespaceImpl::createXPathNamespace(const XMLCh *prefix, const XMLCh *uri, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *owner)
{
    XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *ownerDocument = owner->getOwnerDocument();
    XPathNamespaceImpl *impl = new ((XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl *)ownerDocument, (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType)XPATH_NAMESPACE_OBJECT) XPathNamespaceImpl(prefix, uri, owner, ownerDocument);
    return impl;
};


void  XPathNamespaceImpl::release() {
    if (fNode.isOwned() && !fNode.isToBeReleased())
        throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::INVALID_ACCESS_ERR,0);

    XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl* doc = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl*) getOwnerDocument();
    if (doc) {
        fNode.callUserDataHandlers(XERCES_CPP_NAMESPACE_QUALIFIER DOMUserDataHandler::NODE_DELETED, 0, 0);
        doc->release(this, (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType)XPATH_NAMESPACE_OBJECT);
    }
    else {
        // shouldn't reach here
        throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::INVALID_ACCESS_ERR,0);
    }

}

//
//   Functions inherited from Node
//

           XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap* XPathNamespaceImpl::getAttributes() const                   {return fNode.getAttributes (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument*     XPathNamespaceImpl::getOwnerDocument() const                {return fNode.getOwnerDocument (); };
           bool             XPathNamespaceImpl::hasAttributes() const                   {return fNode.hasAttributes(); };
           bool             XPathNamespaceImpl::isEqualNode(const XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* arg) const  {return fNode.isEqualNode(arg); };

           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::appendChild(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newChild)          {return fNode.appendChild (newChild); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList*     XPathNamespaceImpl::getChildNodes() const                   {return fNode.getChildNodes (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getFirstChild() const                   {return fNode.getFirstChild (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getLastChild() const                    {return fNode.getLastChild (); };
     const XMLCh*           XPathNamespaceImpl::getLocalName() const                    {return fNode.getLocalName (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getNextSibling() const                  {return fNode.getNextSibling (); };

//The exception is wrong - we need to sort out all these methods when we can compile again.
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getParentNode() const                   { 	throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);};
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getPreviousSibling() const              {throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);};
           bool             XPathNamespaceImpl::hasChildNodes() const                   {return fNode.hasChildNodes (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::insertBefore(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newChild, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *refChild)
                                                                                    {return fNode.insertBefore (newChild, refChild); };
           void             XPathNamespaceImpl::normalize()                             {fNode.normalize (); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::removeChild(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *oldChild)          {return fNode.removeChild (oldChild); };
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::replaceChild(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newChild, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *oldChild)
                                                                                    {return fNode.replaceChild (newChild, oldChild); };
           bool             XPathNamespaceImpl::isSupported(const XMLCh *feature, const XMLCh *version) const
                                                                                    {return fNode.isSupported (feature, version); };
           bool             XPathNamespaceImpl::isSameNode(const XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* other) const {return fNode.isSameNode(other); };
           void*            XPathNamespaceImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
           const XMLCh*     XPathNamespaceImpl::getBaseURI() const                      {return fNode.getBaseURI(); };
           short            XPathNamespaceImpl::compareTreePosition(const XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* other) const {return fNode.compareTreePosition(other); };
           const XMLCh*     XPathNamespaceImpl::getTextContent() const                  {return fNode.getTextContent(); };
           void             XPathNamespaceImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
#if _XERCES_VERSION >= 20100
           const XMLCh*     XPathNamespaceImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
           bool             XPathNamespaceImpl::isDefaultNamespace(const XMLCh* namespaceURI) const {return fNode.isDefaultNamespace(namespaceURI); };
           const XMLCh*     XPathNamespaceImpl::lookupNamespaceURI(const XMLCh* nsPrefix) const {return fNode.lookupNamespaceURI(nsPrefix); };
#else
           const XMLCh*     XPathNamespaceImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
           bool             XPathNamespaceImpl::isDefaultNamespace(const XMLCh* namespaceURI) {return fNode.isDefaultNamespace(namespaceURI); };
           const XMLCh*     XPathNamespaceImpl::lookupNamespaceURI(const XMLCh* prefix) {return fNode.lookupNamespaceURI(prefix); };
#endif
           XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*         XPathNamespaceImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); };
