/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef __XPATHRESULTIMPL_HPP
#define __XPATHRESULTIMPL_HPP

#include <config.h>

#include <dom3-xpath/XPathResult.hpp>

#include <xercesc/util/XMLString.hpp>

#include <dataItem/DataItem.hpp>
#include <dataItem/DeadNodeList.hpp>
#include <context/Context.hpp>

class XPathResultImpl : public XPathResult
{
public:

  XPathResultImpl(DataItem *result, const XPathResult::resultType resultType,
                  Context *context, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *documentRoot);
  ~XPathResultImpl();

  XPathResult::resultType getResultType() const;

  double getNumberValue() const throw (XPathException);

  const XMLCh *getStringValue() const throw (XPathException);

  bool getBooleanValue() const throw (XPathException);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *getSingleNodeValue() const throw (XPathException);

  bool getInvalidIteratorState() const;

  unsigned long getSnapshotLength() const throw(XPathException);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *iterateNext() throw (XPathException, XERCES_CPP_NAMESPACE_QUALIFIER DOMException);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *snapshotItem(unsigned long index) const throw(XPathException);

protected:

  const std::string errorMessage(XPathResult::resultType requestedType,
                                 XPathResult::resultType resultType) const;
  const std::string typeName(XPathResult::resultType type) const;

  DataItem *result;
  DeadNodeList *resultNodeList;
  XPathResult::resultType resultType;
  Context *context;

  unsigned int curIndex;

}; //XPathResultImpl

#endif //__XPATHRESULTIMPL_HPP
