/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "LexerDOMStringStream.hpp"
#include "xercesc/util/XMLString.hpp"

using namespace std;  //Note: Needed to make windows happy

LexerDOMStringStream::LexerDOMStringStream(const XMLCh *inputBuffer)
{
  position = 0;
  length = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(inputBuffer);
  last_retrieved = 0;

  buffer = inputBuffer;
}//constructor

LexerDOMStringStream::~LexerDOMStringStream()
{
}//destructor

int LexerDOMStringStream::read(unsigned char *ptr, unsigned int numBytes)
{
  unsigned short *inputBuffer = (unsigned short *)ptr;

  if(position == length) {
    inputBuffer[0] = 0;
    position++;
    last_retrieved = 0;
    return 0;
  }//if

  int towrite;

  //Note: numBytes is in terms of bytes, not Unicode characters
  numBytes /= sizeof(unsigned short);

  //Figure out how many characters to read
  if(numBytes > length - position) {
    towrite = length - position;
  } else {
    towrite = numBytes;
  }//if

  //Copy the characters into ptr
  int outpos = 0;
  for(unsigned int cnt=position; cnt<towrite+position; cnt++) {
    inputBuffer[outpos] = buffer[cnt];
    outpos++;
  }//for

  //Update markers
  last_retrieved = towrite;
  position += towrite;

  return towrite;
}//read

bool LexerDOMStringStream::eof() const
{
  //Note: This hopefully mimics what istream does with eof()
  if(position == length+1) {
    return true;
  } else {
    return false;
  }//if
}//eof
