/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
  Interface to XPathLexer that gives a yylex function as
  required by Bison.
*/

#ifndef _XPATHLEXER_HPP
#define _XPATHLEXER_HPP

#include <config.h>


#include "../navigation/NavStep.hpp"
#include "../parser/ArgHolder.hpp"
#include "../dataItem/DataItemNav.hpp"
#include "../defs/XMLScriptTypeDefs.hpp"
#include "../navigation/PredicateStepExprXPath.hpp"
//#include "../navigation/PredicateStep.hpp"
#include "LexerDOMStringStream.hpp"
#include <stdlib.h>

#ifndef YY_CHAR
#define YY_CHAR unsigned short
#endif

using namespace std;

typedef union {

  XSFloat number;
  DataItem *dataItem;
  DataItemNav *dataItemNav;
  YY_CHAR *dString;
  XMLCh *wString;
  NavStep *navStep;
  ArgHolder *argHolder;
  PredicateStepExpr *predStep;
  vector<PredicateStep*> *predStore;

} yystype;

#define YYSTYPE yystype


/** Adapter for XPathLexer that gives a yylex function as required
    by Bison. Basically Bison expects an integer as the return type,
    with 0 meaning end of input. If the returned token type has an
    associated chunk of data, we should store it in lvalp. The
    pcontrol pointer is the only way to pass control data to the yylex
    function, and is cast back to a DSLPathParserControl structure,
    then the Lexer instance included in that is used to actually grab
    tokens. **/
int yylex(YYSTYPE *lvalp, void *pcontrol);

#endif









