/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "AttributeAxis.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMDocument.hpp>

DeadNodeList *AttributeAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
  list->clear();
  if(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attrs = node->getAttributes();
      int attrs_length = 0;

      if (attrs != 0) {
        attrs_length = attrs->getLength();
      }//if

      for (int k = 0; k < attrs_length; k++) {
        assert(attrs->item(k)!=NULL);
        list->push_back(attrs->item(k));
      }//for
  }//if

  return list;
}//makeList

/* If you make a change in this method, don't forget to copy it to
   InheritedAttributeAxis::doCreativeMethod as well */
XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *AttributeAxis::doCreativeMode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const Context *context) const
{
  //we only create one if we have a name and uri and type. Also node must be an element
  if(!(getNameWildcard() || getNamespaceWildcard() || getTypeWildcard())
     && node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {

    XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *target = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node);

    const XMLCh *prefix = getNodePrefix();

    const XMLCh *uri = NULL;
    if(prefix==NULL || prefix[0]==0) {
      uri = context->getDefaultExprNS();
    }
    else {
      uri = context->getInheritedPrefixNS(prefix);
    }

    if(!XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(uri) && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getNodePrefix())) {
      throw NavigationException("The prefix \"" + v2Utils::DSLtranscode(getNodePrefix()) + "\" is not bound to a URI.");
    }

    XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr = target->getOwnerDocument()->createAttributeNS(uri, getNodeName());
    target->setAttributeNodeNS(attr);
    return attr;
  }
  return NULL;
}






