/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "ChildAxis.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMText.hpp>
#include <xercesc/dom/DOMComment.hpp>

DeadNodeList *ChildAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
  list->clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *kid = node->getFirstChild();

  if(!(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) && node->hasChildNodes()) {
    list->addList(node->getChildNodes());
  }
  return list;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ChildAxis::doCreativeMode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const Context *context) const
{
  //we only create one if we have a name and uri and nodetype. Node must also be an element
  if(!(getNameWildcard() || getNamespaceWildcard() || getTypeWildcard())
     && (node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE)) {

    assert(getNodeType() != 0);
    int type = getNodeType();

    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newNode = NULL;
    XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc = node->getOwnerDocument();

    switch(type) {

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE:{
      const XMLCh *prefix = getNodePrefix();

      const XMLCh *uri = NULL;
      if(prefix==NULL || prefix[0]==0) {
        uri = context->getDefaultExprNS();
      }
      else {
        uri = context->getInheritedPrefixNS(prefix);
      }

      if(!XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(uri) && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getNodePrefix())) {
        throw NavigationException("The prefix \"" + v2Utils::DSLtranscode(getNodePrefix()) + "\" is not bound to a URI.");
      }

      newNode = doc->createElementNS(uri, getNodeName());
      node->appendChild(newNode);
      break;
    }

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:{
      newNode = doc->createTextNode(NULL);
      node->appendChild(newNode);
      break;
    }

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE:{
      newNode = doc->createComment(NULL);
      node->appendChild(newNode);
      break;
    }

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::NOTATION_NODE:
    //case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::XML_DECL_NODE:
    default: {
      break;
    }
    }

    return newNode;

  }

  //we
  return NULL;
}




