/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FollowingAxis.hpp"
#include <xercesc/dom/DOMAttr.hpp>

DeadNodeList *FollowingAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
  list->clear();
  makeListHelper(node, list);

  return list;
}//makeList


void FollowingAxis::makeListHelper(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *retList) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *currentNode = node;

  while(currentNode!=NULL) {
    //First get the next sibling, if any, otherwise get the parent and keep going from there
    if(currentNode->getNextSibling()==NULL) {
      if(currentNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
	currentNode = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(currentNode)->getOwnerElement();
      } else {
	currentNode = currentNode->getParentNode();
      }
      continue;
    }//if

    //otherwise there is a sibling, so keep on chugging
    currentNode = currentNode->getNextSibling();

    //recursively add descendants to the list
    retList->push_back(currentNode);

    if(currentNode->hasChildNodes()) {
      addTree(currentNode, retList);
    }//if
  }//while
}//makeListHelper

void FollowingAxis::addTree(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *retList) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getFirstChild();
  while (child!=NULL) {
    //recursively add descendants to the list
    retList->push_back(child);

    if(child->hasChildNodes()) {
      addTree(child, retList);
    }//of

    child = child->getNextSibling();
  }//while
}//addTree

