/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "ForwardAxis.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include <deque>
#include<iostream>
#include <xercesc/dom/DOMNode.hpp>

#include "../DOMutils/v2Utils.hpp"

ForwardAxis::ForwardAxis()
{
  setAxisOrientation(FORWARD);
}


void ForwardAxis::doStep(DeadNodeList *candidateList, const Context *context, const bool creativeMode, const bool firstMatch, DeadNodeList *result) const
{
  const XMLCh *uri = context->getInheritedPrefixNS(getNodePrefix());

  if((!XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(uri) || getNamespaceWildcard()) && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getNodePrefix())) {
    throw NavigationException("The prefix \"" + v2Utils::DSLtranscode(getNodePrefix()) + "\" is not bound to a URI.");
  }

  const XMLCh *defaultExprNS = NULL;

  //for default namespace in expressions
  if(getConsiderDefault()) {
    defaultExprNS = context->getDefaultExprNS();
  }

  DeadNodeList nodeListToFilter;
  DeadNodeList filteredNodeList;
  DeadNodeList::iterator i, end;
  end = candidateList->end();

  // Iterate over the candidates
  for(i = candidateList->begin(); i != end; ++i) {

    // Filter on type name and namespace
    makeList(*i, &nodeListToFilter);
    filterNodes(&nodeListToFilter, uri, defaultExprNS, &filteredNodeList);


    // Do creative mode
    if(creativeMode && filteredNodeList.getLength() == 0) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newNode = doCreativeMode(*i, context);
      if(newNode!=NULL) {
        filteredNodeList.push_back(newNode);
      }
    }
    // If this is firstMatch is set exit checkPredicates as soon as a
    // positive is found
    if(firstMatch) {
      checkPredicates(&filteredNodeList, context, true, result);

      //If we have a match leave the loop
      if(!result->empty()) {
        return;
      }
    }
    // Otherwise checkPredicates finds all matches
    else {
      checkPredicates(&filteredNodeList, context, false, result);
    }

    filteredNodeList.clear();
    nodeListToFilter.clear();
  }

}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ForwardAxis::doCreativeMode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const Context *context) const
{
  return NULL;
}







