/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "InheritedAttributeAxis.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMDocument.hpp>

DeadNodeList *InheritedAttributeAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
  list->clear();

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *nodeIter = node;

  if(nodeIter->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
    //Add the attributes from all parents up to root
    while(nodeIter->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attrs = nodeIter->getAttributes();
      const XMLCh *namespaceURI = nodeIter->getNamespaceURI();

      //Go up tree it this node doesn't have any attributes
      if(attrs == NULL) {
        continue;
      }//if

      //Go through each attribute for this node, and add it, if it hasn't been already
      bool toAdd = true;
      for(unsigned int cnt=0; cnt<attrs->getLength(); cnt++) {
        XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = attrs->item(cnt);

        //Get the attribute's info
        const XMLCh *nodeName = tmpNode->getNodeName();
        const XMLCh *nodeURI = tmpNode->getNamespaceURI();

        //Compare against what's already been added
        for(DeadNodeList::iterator iter = list->begin();
	    iter != list->end(); ++iter) {
          const XMLCh *tmpName = (*iter)->getNodeName();
          const XMLCh *tmpURI = (*iter)->getNamespaceURI();

          if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodeName, tmpName)==0
             && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodeURI,tmpURI)==0) {
            toAdd = false;
            break;
          }//if
        }//for iter

        //Add it, if it isn't already in the list
        if(toAdd == true) {
          list->push_front(attrs->item(cnt));
        }//if
      }//for cnt

      //go up one level in the tree
      nodeIter = nodeIter->getParentNode();
    }//while
  }//if

  return list;
}//makeList

/* If you make a change in this method, don't forget to copy it to
   AttributeAxis::doCreativeMethod as well */
XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *InheritedAttributeAxis::doCreativeMode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const Context *context) const
{
  //we only create one if we have a name and uri and type. Also node must be an element
  if(!(getNameWildcard() || getNamespaceWildcard() || getTypeWildcard())
     && node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {

    XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *target = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node);

    const XMLCh *prefix = getNodePrefix();

    const XMLCh *uri = NULL;
    if(prefix==NULL || prefix[0]==0) {
      uri = context->getDefaultExprNS();
    }
    else {
      uri = context->getInheritedPrefixNS(prefix);
    }

    if(!XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(uri) && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getNodePrefix())) {
      throw NavigationException("The prefix \"" + v2Utils::DSLtranscode(getNodePrefix()) + "\" is not bound to a URI.");
    }

    XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr = target->getOwnerDocument()->createAttributeNS(uri, getNodeName());
    target->setAttributeNodeNS(attr);
    return attr;
  }
  return NULL;
}//doCreativeMode



