/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "NamespaceAxis.hpp"
#include "../dom3-xpath/XPathNamespace.hpp"
#include "../interface/XPathNamespaceImpl.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include <xercesc/dom/DOM.hpp>
#include <algorithm>

//Stuff for creating the namespace nodes
static DeadNodeList *tmpNodeList;
static XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *parentNode;
static void createNSNode(pair<const XMLCh * const, const XMLCh *> tmpIter);

DeadNodeList *NamespaceAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
	list->clear();  

	//Note: We don't care what kind of node 'node' is.
	//createNamespaceMap does the necessary sanity checks
	namespaceMapType *namespaceMap = v2NSUtils::createNamespaceMap(node);

	//Create nodes for all the namespaces in scope
	tmpNodeList = list;
	parentNode = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node);
	for_each(namespaceMap->begin(), namespaceMap->end(), createNSNode);
        delete namespaceMap;

	return list;
}//makeList


static void createNSNode(pair<const XMLCh * const, const XMLCh *> tmpIter)
{
	const XMLCh *prefix = tmpIter.first;
	const XMLCh *uri = tmpIter.second;

  if (parentNode->getOwnerDocument() != NULL) {
     parentNode->getOwnerDocument()->getNodeName();
  }

	//XPathNamespace *newNSNode = parentNode->getOwnerDocument()->createXPathNamespace(prefix, uri, parentNode);
	XPathNamespace *newNSNode = XPathNamespaceImpl::createXPathNamespace(prefix, uri, parentNode);

	tmpNodeList->push_back(newNSNode);

}//createNSNode



