/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "NavStep.hpp"
#include "../DOMutils/v2NSUtils.hpp"

#include "PredicateStepRange.hpp"
#include "PredicateStep.hpp"
#include <xercesc/dom/DOMNodeList.hpp>
#include "../dom3-xpath/Pathan.hpp"

#include<memory>

static bool myUseWholeText = false;


NavStep::NavStep(void) {
  _name = NULL;
  _prefix = NULL;
  _type = 0;
  _wildcardName = false;
  _wildcardNamespace = false;
  _wildcardType = false;
  _axisOrientation = FORWARD;
  _considerDefault = true; // Changed from false to true, 2001-08-30 - jpcs
}

NavStep::~NavStep(void) {

   delete[] (XMLCh *)_name;
   _name=NULL;
   delete[] (XMLCh *)_prefix;
   _prefix=NULL;

   for(vector<PredicateStep*>::iterator predicateI = _predList.begin();
       predicateI != _predList.end();
       ++predicateI)
   {
      delete *predicateI;
   }
}

void NavStep::doStep(DeadNodeList *candidateList, const Context *context, const bool creativeMode, const bool firstMatch, DeadNodeList *result) const
{
  checkPredicates(candidateList, context, firstMatch, result);
}

void NavStep::filterNodes(const DeadNodeList *nodeList, const XMLCh *uri, const XMLCh *defaultExprNS, DeadNodeList *result) const
{
   filterNodesHelper(nodeList, uri, defaultExprNS, false, result);
}//filterNodes

void NavStep::filterNodesHelper(const DeadNodeList *nodeList, const XMLCh *uri, const XMLCh *defaultExprNS, bool prevIsText, DeadNodeList *result) const
{
  DeadNodeList::const_iterator i, end;
  end = nodeList->end();

//  DOM_Node lastNode;

//  if(nodeList.empty() == false) {
//	  lastNode = nodeList.item(0);
//  }//if

  for(i = nodeList->begin(); i != end; ++i) {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *curNode = *i;

	if(curNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE) {
           auto_ptr<DeadNodeList> entityChilds
              = static_cast< auto_ptr<DeadNodeList> >(new DeadNodeList(curNode->getChildNodes()));
           filterNodesHelper(entityChilds.get(), uri, defaultExprNS, prevIsText, result);
	} else {
		if(checkNodeType(curNode) && checkNodeName(curNode) && checkNodeNamespace(curNode, uri, defaultExprNS)) {
			//We only want to add a text node, if the last node we looked at wasn't
			//a text node sibling of the current node
			if( ((curNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE) ||
				(curNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE)) && (myUseWholeText == false)) {

//				DOM_Node prevSib = curNode.getPreviousSibling();

				//If there was a previous sibling, that was the last node looked at, and
				//that node was a text node, then do nothing; otherwise add the node to the result
//				if((prevSib.isNull() == false) && (prevSib == lastNode) &&
//						((lastNode.getNodeType() == DOM_Node::TEXT_NODE) ||
//						 (lastNode.getNodeType() == DOM_Node::CDATA_SECTION_NODE))) {
				if(prevIsText == false) {
					result->push_back(curNode);
				}//if
			} else {
				//Some non-text node to be added
				result->push_back(curNode);
			}//if
		}//if
	}//if

	if((curNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE) || (curNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE)) {
		prevIsText = true;
	} else {
		prevIsText = false;
	}//if


//	lastNode = curNode;
  }//for
}//filterNodesHelper


void Pathan::useWholeText(bool wholeText)
{
	myUseWholeText = wholeText;
}//setWholeText


bool NavStep::checkNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const
{
  if(_wildcardType) {
    return true;
  } else {
	int nodeType = node->getNodeType();

    assert(getNodeType() != 0);

	//CDATA nodes are the same as Text, so we need to keep that in mind
	if((_type == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE) && (nodeType == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::CDATA_SECTION_NODE)) {
		return true;
	}//if

	//Otherwise, for everything else, we do the usual
    return node->getNodeType() == _type;
  }//if
}//checkNodeType

bool NavStep::checkNodeName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const
{
  if(_wildcardName) {
    return true;
  }
  else {
    assert(_name && _name[0]);
    if(_type == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE || _type == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE
       || _type == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE){
      //to make sure we are ok with node created with DOM level 1
      const XMLCh *nodeName = node->getLocalName();
      if(nodeName==NULL || nodeName[0]==0) {
	nodeName = node->getNodeName();
      }
      return XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodeName, _name)==0;
    } else {
      return true;
    }
  }
}


bool NavStep::checkNodeNamespace(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const XMLCh *uri, const XMLCh *defaultExprNS) const
{
  if(_wildcardNamespace) {
    return true;
  }
  else if(_type != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE && _type != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
    return true;
  }
  else if(!_considerDefault) {
    return (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(node->getNamespaceURI(),uri)==0);
  }
  //this is for Decisionsofts DSLPath and XPath+. These both take into account
  //a default expression namespace
  else {
    if(defaultExprNS==NULL || defaultExprNS[0]==0 || node->getNodeType()==XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
      if(uri==NULL || uri[0]==0) {
        const XMLCh *nsUri = node->getNamespaceURI();
        return (nsUri==NULL || nsUri[0]==0);
      }
      else {
	return (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(node->getNamespaceURI(),uri)==0);
      }
    }
    else {
      if(uri==NULL || uri[0]==0) {
	return (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(node->getNamespaceURI(),defaultExprNS)==0);
      }
      else {
	return (XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(node->getNamespaceURI(),uri)==0);
      }
    }
  }
}

void NavStep::setConsiderDefaultExprNamespace(void)
{
  _considerDefault = true;
}

void NavStep::addPredicate(PredicateStep *step)
{
  _predList.push_back(step);
}

void NavStep::addPredicates(const std::vector<PredicateStep*> &steps)
{
  vector<PredicateStep*>::const_iterator i, end;
  end = steps.end();

  for(i = steps.begin(); i != end; ++i) {
    _predList.push_back(*i);
  }

}

void NavStep::setAxisOrientation(const AxisOrientation ori)
{
  _axisOrientation = ori;
}


void NavStep::setNodeName(const XMLCh *name)
{
  _name = name;
}

void NavStep::setNodePrefix(const XMLCh *name)
{
  _prefix = name;
}

void NavStep::setNodeType(int type)
{
  _type = type;
}

void NavStep::setNameWildcard(void)
{
  _wildcardName = true;
}

void NavStep::setNamespaceWildcard(void)
{
  _wildcardNamespace = true;
}

void NavStep::setTypeWildcard(void)
{
  _wildcardType = true;
}

bool NavStep::getConsiderDefault(void) const
{
  return _considerDefault;
}

int NavStep::getNodeType(void) const
{
  return _type;
}

const XMLCh *NavStep::getNodePrefix(void) const
{
  return _prefix;
}

const XMLCh *NavStep::getNodeName(void) const
{
  return _name;
}

bool NavStep::getNameWildcard(void) const
{
  return _wildcardName;
}

bool NavStep::getNamespaceWildcard(void) const
{
  return _wildcardNamespace;
}

bool NavStep::getTypeWildcard(void) const
{
  return _wildcardType;
}

const std::vector<PredicateStep*> &NavStep::getPredicates(void) const
{
  return _predList;
}

bool NavStep::getTypeSet(void) const
{
  return _type != 0;
}

void NavStep::checkPredicates(DeadNodeList *candidateList, const Context *context, bool leaveOnMatch, DeadNodeList *result) const
{
    if(_predList.size() == 0) {
        result->addList(candidateList);
        return;
    }

  Context newcontext(*context);

  vector<PredicateStep*>::const_iterator j, jend;
  jend = _predList.end();

  DeadNodeList::const_iterator i, iend;
  unsigned int contextPosition;
  DeadNodeList tmpList;
  DeadNodeList *dblist[2];
  int db = 0;
  dblist[db] = candidateList;
  dblist[1] = &tmpList;

  //check all the predicates
  if(_axisOrientation == FORWARD) {
    // iterate over the predicates
    for(j = _predList.begin();j != jend; ++j) {
      newcontext.setContextSize(dblist[db]->getLength());
      contextPosition = 1;
      iend = dblist[db]->end();

      // iterate over the candidates
      for(i = dblist[db]->begin(); i != iend; ++i) {
	newcontext.changeCurrentInLoop(*i);
	newcontext.setContextPosition(contextPosition);

	if((*j)->doPredicate(&newcontext)) {
	  dblist[1-db]->push_back(*i);
	  if(leaveOnMatch) {
		result->addList(dblist[1-db]);
	  }
	}
	++contextPosition;
      }

      // Double buffer the lists
      dblist[db]->clear();
      db = 1 - db;
    }
  }
  else { // REVERSE
    // iterate over the predicates
    for(j = _predList.begin();j != jend; ++j) {
      newcontext.setContextSize(dblist[db]->getLength());
      contextPosition = newcontext.getContextSize();
      iend = dblist[db]->end();

      // iterate over the candidates
      for(i = dblist[db]->begin(); i != iend; ++i) {
	newcontext.changeCurrentInLoop(*i);
	newcontext.setContextPosition(contextPosition);

	if((*j)->doPredicate(&newcontext)) {
	  dblist[1-db]->push_back(*i);
	  if(leaveOnMatch) {
		result->addList(dblist[1-db]);
	  }
	}
	--contextPosition;
      }

      // Double buffer the lists
      dblist[db]->clear();
      db = 1 - db;
    }
  }
  result->addList(dblist[db]);

}






