/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _NAVIGATIONSTEP_HPP
#define _NAVIGATIONSTEP_HPP

#include <config.h>

#ifdef WIN32
#pragma warning( disable: 4786 )
#pragma warning( disable : 4251 )
#endif

#include <vector>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include "../dataItem/DeadNodeList.hpp"
#include "../context/Context.hpp"
#include "PredicateStep.hpp"
#include <assert.h>

class FunctionLookup;

/**
This class stores information about a Location path step. When doStep is called the candidate
list passed in is filtered based on predictes.
 */
class PATHAN_EXPORT NavStep
{
public:
  NavStep();
  virtual ~NavStep();

  ///this acts on the candidate list and returns the result
  virtual void doStep(DeadNodeList *candidateList, const Context *context, const bool creativeMode, const bool firstMatch, DeadNodeList *result) const;

  ///add a predicate to the step
  void addPredicate(PredicateStep *step);

  ///add a predicate to the step
  void addPredicates(const std::vector<PredicateStep*> &steps);

  ///set function for private member
  void setNodeName(const XMLCh *name);

  ///set function for private member
  void setNodePrefix(const XMLCh *name);

  ///set node type looked for
  void setNodeType(int type);

  ///sets the wildcard nodeName bool
  void setNameWildcard();

  ///sets the prefix wildcard bool
  void setNamespaceWildcard();

  ///sets the type wildcard bool
  void setTypeWildcard();

  ///after this is called, the default expression namespace is considered
  void setConsiderDefaultExprNamespace();

  ///sets the axis orientation used in predicates
  void setAxisOrientation(const AxisOrientation ori);

  ///gets the node type we are selecting
  int getNodeType() const;

  ///gets the prefix of the node we are selecting
  const XMLCh *getNodePrefix() const;

  ///gets the name of the node we are selecting
  const XMLCh *getNodeName() const;

  ///returns true if the Node Name ia a wildcard
  bool getNameWildcard() const;

  ///returns true if the Node prefix ia a wildcard
  bool getNamespaceWildcard() const;

  ///returns true if the type ia a wildcard
  bool getTypeWildcard() const;

  ///returns true is we are considering a deafult namespace in a the expression
  bool getConsiderDefault() const;

  ///returns the predicate list to be applied to each node we select
  const std::vector<PredicateStep*> &getPredicates() const;

  ///retuns whether the type has been set in this NavStep
  bool getTypeSet() const;


protected:
  ///filters the nodeList based on name namespace and type
  void filterNodes(const DeadNodeList *nodeList, const XMLCh *uri, const XMLCh *defaultExprNS, DeadNodeList *result) const;
  void filterNodesHelper(const DeadNodeList *nodeList, const XMLCh *uri, const XMLCh *defaultExprNS, bool prevIsText, DeadNodeList *result) const;

  ///check a node's type against _type, considers wildcard
  bool checkNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const;

  ///check a node's name against _name, considers wildcard
  bool checkNodeName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const;

  /// returns whether node has the same uri as _uri, considers wildcard
  bool checkNodeNamespace(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const XMLCh *uri, const XMLCh *defaultExprNS) const;

  ///discard members of candidateList which don't conform to current predicates
  void checkPredicates(DeadNodeList *candidateList, const Context *context, bool leaveOnMatch, DeadNodeList *result) const;


private:
  std::vector<PredicateStep*> _predList;
  const XMLCh *_name;
  const XMLCh *_prefix;
  int _type;
  bool _wildcardName, _wildcardNamespace, _wildcardType, _considerDefault, _wholeText;
  AxisOrientation _axisOrientation;

  NavStep(const NavStep& rhs);
  NavStep& operator=(const NavStep& rhs);
};

#endif
