/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "PrecedingAxis.hpp"
#include <xercesc/dom/DOMAttr.hpp>

DeadNodeList *PrecedingAxis::makeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *list) const
{
  list->clear();
  makeListHelper(node, list);

  return list;
}

void PrecedingAxis::makeListHelper(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *retList) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *currentNode = node;

  while(currentNode!=NULL) {

    if(currentNode->getPreviousSibling()==NULL) {
      if(currentNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
	currentNode = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(currentNode)->getOwnerElement();
      } else {
	currentNode = currentNode->getParentNode();
      }
      continue;
    }//if

    //otherwise there is a sibling, so keep on chugging
    currentNode = currentNode->getPreviousSibling();

    //recursively add descendants to the list
    if(currentNode->hasChildNodes()) {
      addTree(currentNode, retList);
    }//if

    retList->push_back(currentNode);
  }//while

}//makeListHelper

void PrecedingAxis::addTree(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, DeadNodeList *retList) const
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child = node->getLastChild();
  while (child!=NULL) {
    //recursively add descendants to the list
    if(child->hasChildNodes()) {
      addTree(child, retList);
    }//of

    retList->push_back(child);

    child = child->getPreviousSibling();
  }//while
}//addTree

