/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _PREDICATESTEP_HPP
#define _PREDICATESTEP_HPP

#include <config.h>

#include "../functionAPI/FunctionLookup.hpp"
//#include "../context/Context.hpp"
class Context;
#include "../DOMutils/v2Utils.hpp"

using namespace std;

enum PredType {
  PRED_EXPR,
  PRED_INDEX
};

typedef enum {
  FORWARD, REVERSE
} AxisOrientation;

/** This is a base class for all predicate steps - doPredicate should
    be overridden by sub classes*/
class PATHAN_EXPORT PredicateStep
{

public:
  PredicateStep();
  virtual ~PredicateStep();

  /** Assess whether a node meets some criteria. These criteria might
      reflect the node's position in the tree (e.g. index selections)
      or some more complex test. The function returns true if the
      criteria are met, false otherwise. This is a pure virtual
      function - implementations are left to subclasses. */
  virtual bool doPredicate(const Context *context) const = 0;

  /// predicates need to be able to return their type
  virtual PredType returnType(void) const = 0;

private:

  PredicateStep(const PredicateStep& rhs);
  PredicateStep& operator=(const PredicateStep& rhs);
};

#ifdef _INLINE_FUNCTIONS
#include "PredicateStep.inl"
#endif

#endif // _PREDICATESTEP_HPP




