/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "PredicateStepExprXPath.hpp"
#include "../defs/NumUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include "../DOMutils/v2Utils.hpp"
#include <memory>

PredicateStepExprXPath::PredicateStepExprXPath(DataItem *expr)
  : PredicateStepExpr(expr)
{
}

PredicateStepExprXPath::~PredicateStepExprXPath()
{
}

bool PredicateStepExprXPath::doPredicate(const Context *context) const
{
  // Evaluate the expression
   auto_ptr<DataItem> simpleData
        = static_cast< auto_ptr<DataItem> >(_expr->asSimpleDataItem(context));

  //Check if the result is a number, if so the predicate is
  //satisfied if the context position of the node is equal to said
  //number
  if(simpleData->getType() == DataItem::NUMERICLITERAL) {
    return NumUtils::isEqual(simpleData->asNum(context),
			     static_cast<XSFloat>(context->getContextPosition()));
  }

  //If not a numeric literal the expression is evaluated as a boolean
  else {
    return simpleData->asBool(context);
  }

  return false;
}
