/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
  NavStepPredicateRange -- NavStepPredicate that selects nodes based
  on a range.  Very basic functionality for now -- single indices and
  ranges only, specified by integers.
*/

#include "PredicateStepRange.hpp"

#include "../functionAPI/FunctionLookup.hpp"
#include "../context/Context.hpp"
#include <assert.h>
#include "../DOMutils/v2NSUtils.hpp"
#include "../dataItem/DeadNodeList.hpp"

PredicateStepRange::PredicateStepRange()
{
  _doSingleLast = false;
}

PredicateStepRange::~PredicateStepRange()
{
   vector<DataItem*>::iterator iter;
   for(iter = _lastVec.begin();
       iter != _lastVec.end();
       ++iter)
   {
      delete *iter;
   }
   for(iter = _doubleFrom.begin();
       iter != _doubleFrom.end();
       ++iter)
   {
      delete *iter;
   }
   for(iter = _doubleTo.begin();
       iter != _doubleTo.end();
       ++iter)
   {
      delete *iter;
   }
   for(iter = _singleFrom.begin();
       iter != _singleFrom.end();
       ++iter)
   {
      delete *iter;
   }
}

void PredicateStepRange::addRangeAtEnd()
{
  _doSingleLast = true;
}

void PredicateStepRange::addRangeAtEnd(DataItem *min)
{
  _lastVec.push_back(min);
}


void PredicateStepRange::addRange(DataItem *From)
{
  _singleFrom.push_back(From);
}

void PredicateStepRange::addRange(DataItem *exprFrom, DataItem *exprTo)
{
  _doubleFrom.push_back(exprFrom);
  _doubleTo.push_back(exprTo);
}

bool PredicateStepRange::doPredicate(const Context *context) const
{
  //get the index
  int candIndex = v2Utils::indexNode(context->getCurrentContext());

  if(checkSingleton(context, candIndex)) {
    return true;
  }

  if(checkRange(context, candIndex)) {
    return true;
  }

  if(checkDollar(context, candIndex)) {
    return true;
  }

  return false;
}

bool PredicateStepRange::checkSingleton(const Context *context,
					int candIndex) const
{
  //check the singleton ranges
  for(vector<DataItem*>::const_iterator k = _singleFrom.begin();
      k != _singleFrom.end(); k++) {
    int value = (*k)->asInt(context);
    if(value == candIndex) {
      return true;
    }
  }

  return false;
}

bool PredicateStepRange::checkRange(const Context *context,
				    int candIndex) const
{
  //check the ranges
  vector<DataItem*>::const_iterator i, j;
  for(i = _doubleFrom.begin(), j = _doubleTo.begin();
      i != _doubleFrom.end(); i++, j++) {
    int from = (*i)->asInt(context);
    int to = (*j)->asInt(context);

    if(to < from) {
      int temp = from; from = to; to = temp;
    }

    if(from < 1) {
      // we don't return anything, coz richard says -1 refers to the
      // end of the string, not anything else.
      break;
    }

    if(candIndex <= to && candIndex >= from) {
      return true;
    }
  }

  return false;
}

bool PredicateStepRange::checkDollar(const Context *context,
				     int candIndex) const
{
  //must now be a $ thingy
  for(vector<DataItem*>::const_iterator l = _lastVec.begin();
      l != _lastVec.end(); l++) {
    int min = (*l)->asInt(context);

    if(candIndex >= min) {
      return true;
    }
  }

  if(_doSingleLast) {
    if(candIndex == v2Utils::getMaxNode(context->getCurrentContext())) {
      return true;
    }
  }

  return false;
}

PredType PredicateStepRange::returnType(void) const
{
  return PRED_INDEX;
}












