/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _PREDICATESTEPRANGE_HPP
#define _PREDICATESTEPRANGE_HPP

#ifndef WIN32
#include <config.h>
#else
#pragma warning(disable : 4251)
#endif

#include "../functionAPI/FunctionLookup.hpp"
//#include "../context/Context.hpp"
class Context;
#include "PredicateStep.hpp"

using namespace std;

/** PredicateStepRange is solely used for DSLstyle ranges */
class PATHAN_EXPORT PredicateStepRange : public PredicateStep
{
public:
  /** The constructor. */
  PredicateStepRange();
  /** The destructor. */
  ~PredicateStepRange();

  /** Add a singleton range, consisting of a single index - as an
      expression. */
  void addRange(DataItem *expr);

  /** Add a pair range, consisting of an index to start and an index
      to finish - as an expression. */
  void addRange(DataItem *from, DataItem *to);

  /// indicate we are using a dollar sign.
  void addRangeAtEnd();

  /// if it is in a range, pass along it's minimum value
  void addRangeAtEnd(DataItem *expr);

   /** Carry out the navigation step, returning a list of nodes. This
      function should be passed an initial candidate list, an object
      representing the current context (for evaluating expressions), a
      pointer to the function lookup table (also for evaluating
      expressions), creative mode no longer exists */
  bool doPredicate(const Context *context) const;

  PredType returnType(void) const;

private:

  ///check all singleton ranges
  bool checkSingleton(const Context *context, int candIndex) const;

  ///check all paired ranges
  bool checkRange(const Context *context, int candIndex) const;

  ///check any $ related ranges
  bool checkDollar(const Context *context, int candIndex) const;

  bool _doSingleLast;
  vector<DataItem*> _lastVec;
  vector<DataItem*> _doubleFrom;
  vector<DataItem*> _doubleTo;
  vector<DataItem*> _singleFrom;
};

#endif // _PREDICATESTEPRANGE_HPP




