/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "ReverseAxis.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/NavigationException.hpp"
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <deque>

ReverseAxis::ReverseAxis()
{
  setAxisOrientation(REVERSE);
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ReverseAxis::getParent(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node) const
{
  switch(node->getNodeType()) {

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE:
  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE:
    return node->getParentNode();
    break;

  case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
    return static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(node)->getOwnerElement();
    break;
  default:
    return NULL;
    break;
  }
}

void ReverseAxis::doStep(DeadNodeList *candidateList, const Context *context, const bool creativeMode, const bool firstMatch, DeadNodeList *result) const
{
  const XMLCh *uri = context->getInheritedPrefixNS(getNodePrefix());

  if((!XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(uri) || getNamespaceWildcard()) && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(getNodePrefix())) {
    throw NavigationException("The prefix \"" + v2Utils::DSLtranscode(getNodePrefix()) + "\" is not bound to a URI.");
  }

  const XMLCh *defaultExprNS = NULL;

  //for default namespace in expressions
  if(getConsiderDefault()) {
    defaultExprNS = context->getDefaultExprNS();
  }

  DeadNodeList nodeListToFilter;
  DeadNodeList filteredNodeList;
  DeadNodeList::iterator i, end;
  end = candidateList->end();

  for(i = candidateList->begin(); i != end; ++i) {
    // Filter on type name and namespace
    makeList(*i, &nodeListToFilter);
    filterNodes(&nodeListToFilter, uri, defaultExprNS, &filteredNodeList);

    // Do creative mode
    if(creativeMode && filteredNodeList.getLength() == 0) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *newNode = doCreativeMode(*i, context);
      if(newNode!=NULL) {
        filteredNodeList.push_back(newNode);
      }
    }

    // If this is firstMatch is set exit checkPredicates as soon as a
    // positive is found
    if(firstMatch) {
      checkPredicates(&filteredNodeList, context, true, result);

      //If we have a match leave the loop
      if(!result->empty()) {
	return;
      }
    }
    // Otherwise checkPredicates finds all matches
    else {
      checkPredicates(&filteredNodeList, context, false, result);
    }

    //    result.addList(checkPredicates(output, context));
  }
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ReverseAxis::doCreativeMode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node, const Context *context) const
{
  return NULL;
}
