%{
#include <xercesc/util/XercesDefs.hpp>
#include <config.h>

#include "../dataItem/DataItem.hpp"
#include "../dataItem/DataItemNav.hpp"
#include "../dataItem/DataItemFunction.hpp"
#include "../dataItem/DataItemVariable.hpp"
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../functionAPI/FunctionUnaryMinus.hpp"
#include "../navigation/PredicateStepExprXPath.hpp"

#include <vector>
#include <xercesc/util/XMLString.hpp>

#include "../lexer/XPathLexer.hpp"

#include "../navigation/NavStep.hpp"
#include "../navigation/AncestorAxis.hpp"
#include "../navigation/AncestorOrSelfAxis.hpp"
#include "../navigation/AttributeAxis.hpp"
#include "../navigation/ChildAxis.hpp"
#include "../navigation/DescendantAxis.hpp"
#include "../navigation/DescendantOrSelfAxis.hpp"
#include "../navigation/FollowingAxis.hpp"
#include "../navigation/FollowingSiblingAxis.hpp"
#include "../navigation/InheritedAttributeAxis.hpp"
#include "../navigation/NamespaceAxis.hpp"
#include "../navigation/ParentAxis.hpp"
#include "../navigation/PrecedingAxis.hpp"
#include "../navigation/PrecedingSiblingAxis.hpp"
#include "../navigation/SelfAxis.hpp"

#include "../functionAPI/FunctionOr.hpp"
#include "../functionAPI/FunctionAnd.hpp"
#include "../functionAPI/FunctionOpEqualsEquals.hpp"
#include "../functionAPI/FunctionOpNotEquals.hpp"
#include "../functionAPI/FunctionOpPlus.hpp"
#include "../functionAPI/FunctionOpMinus.hpp"
#include "../functionAPI/FunctionOpTimes.hpp"
#include "../functionAPI/FunctionOpDivide.hpp"
#include "../functionAPI/FunctionOpLessThan.hpp"
#include "../functionAPI/FunctionOpGreaterThan.hpp"
#include "../functionAPI/FunctionOpLessThanEqual.hpp"
#include "../functionAPI/FunctionOpGreaterThanEqual.hpp"
#include "../functionAPI/FunctionUnaryMinus.hpp"
#include "../functionAPI/FunctionUnion.hpp"
#include "../functionAPI/FunctionOpMod.hpp"

#include "../dom3-xpath/XPathNamespace.hpp"

#define YYPARSE_PARAM parm
#define YYLEX_PARAM parm

inline vector<const DataItem*> packageArgs(DataItem *arg1, DataItem *arg2);
void importStep(NavStep *step1, NavStep *step2);
void yyerror(const char* s);

#include "XPathParserControl.hpp"
#include "../exceptions/XPathParseException.hpp"

#define YYDEBUG 1
#include <cstdio>

%}


/* Make parser thread safe */
%pure_parser
%expect 1


%type <dataItem>    _MainExpr
%type <dataItem>    _Expr
%type <dataItem>    _OrExpr
%type <dataItem>    _AndExpr
%type <dataItem>    _EqualityExpr
%type <dataItem>    _RelationalExpr
%type <dataItem>    _AdditiveExpr
%type <dataItem>    _MultiplicativeExpr
%type <dataItem>    _UnaryExpr
%type <dataItem>    _UnionExpr
%type <dataItem>    _PathExpr
%type <dataItem>    _FilterExpr
%type <dataItem>    _PrimaryExpr
%type <dataItem>    _VariableReference
%type <wString>     _Literal
%type <dataItem>    _Number
%type <dataItem>    _FunctionCall
%type <dString>      _FunctionName
%type <argHolder>    _ArgumentList
%type <dataItemNav>    _LocationPath
%type <dataItemNav>    _ContextSpecifier
%type <dataItemNav>    _AbsoluteLocationPath
%type <dataItemNav>    _AbbreviatedAbsoluteLocationPath
%type <dataItemNav>    _RelativeLocationPath
%type <dataItemNav>    _AbbreviatedRelativeLocationPath
%type <navStep>     _Step
%type <navStep>      _AbbreviatedStep
%type <predStore>    _PredicateList
%type <predStep>    _Predicate
%type <navStep>      _NodeTest
%type <navStep>      _NodeType
%type <navStep>      _NameTest
%type <navStep>      _AxisSpecifier
%type <navStep>     _AbbreviatedAxisSpecifier
%type <navStep>     _AxisName
%type <dString>                 _NCName

%token      TOKEN_BANG
%token  <dString>  TOKEN_LITERAL
%token  <number>  TOKEN_NUMERAL
%token      TOKEN_LEFT_BRACE
%token      TOKEN_RIGHT_BRACE
%token      TOKEN_LEFT_SQUARE
%token      TOKEN_RIGHT_SQUARE
%token      TOKEN_DOT_DOT
%token      TOKEN_AT
%token      TOKEN_COMMA
%token      TOKEN_DOT
%token      TOKEN_DOLLAR_DOLLAR
%token      TOKEN_DOLLAR
%token      TOKEN_PERCENT
/*%token      TOKEN_COLON_COLON*/ /*Note: These are used for the axis specifiers and are handled in the lexer*/
%token      TOKEN_COLON
%token      TOKEN_STAR
%token      TOKEN_SLASH_SLASH
%token      TOKEN_SLASH
%token      TOKEN_BAR
%token      TOKEN_PLUS
%token      TOKEN_MINUS
%token      TOKEN_EQUALS
%token      TOKEN_NOT_EQUALS
%token      TOKEN_LT
%token      TOKEN_LTE
%token      TOKEN_GT
%token      TOKEN_GTE
%token      TOKEN_OPERATOR_AND
%token      TOKEN_OPERATOR_OR
%token      TOKEN_OPERATOR_MOD
%token      TOKEN_OPERATOR_DIV
%token      TOKEN_OPERATOR_LT
%token      TOKEN_OPERATOR_LTE
%token      TOKEN_OPERATOR_GT
%token      TOKEN_OPERATOR_GTE
%token      TOKEN_AXISNAME_ANCESTOR_OR_SELF
%token      TOKEN_AXISNAME_INHERITED_ATTRIBUTE
%token      TOKEN_AXISNAME_ANCESTOR
%token      TOKEN_AXISNAME_ATTRIBUTE
%token      TOKEN_AXISNAME_CHILD
%token      TOKEN_AXISNAME_DESCENDANT_OR_SELF
%token      TOKEN_AXISNAME_DESCENDANT
%token      TOKEN_AXISNAME_FOLLOWING_SIBLING
%token      TOKEN_AXISNAME_FOLLOWING
%token      TOKEN_AXISNAME_NAMESPACE
%token      TOKEN_AXISNAME_PARENT
%token      TOKEN_AXISNAME_PRECEDING_SIBLING
%token      TOKEN_AXISNAME_PRECEDING
%token      TOKEN_AXISNAME_SELF
%token      TOKEN_NODETYPE_COMMENT
%token      TOKEN_NODETYPE_TEXT
%token      TOKEN_NODETYPE_PROCESSING_INSTRUCTION
%token      TOKEN_NODETYPE_NODE
%token  <dString>  TOKEN_NCNAME
%token      TOKEN_HAT



%%




_MainExpr:

  /*
    MainExpr        ::= Expr?
  */

  /* EMPTY */ {
    ((XPathParserControl *)parm)->result = new DataItemNodeList(new DeadNodeList());
  }

  | _Expr {
    ((XPathParserControl *)parm)->result = $1;
  }
;


_Expr:

  /*
    Expr    ::= OrExpr
  */

  _OrExpr {
    $$ = $1;
  }

;

_OrExpr:

  /*
    OrExpr          ::= AndExpr | OrExpr OrOp AndExpr
  */

  _AndExpr {
    $$ = $1;
  }

  | _OrExpr TOKEN_OPERATOR_OR _AndExpr {
    $$ = new FunctionOr(packageArgs($1, $3));
  }
;


_AndExpr:

  /*
    AndExpr         ::= EqualExpr | AndExpr AndOp EqualExpr
  */

  _EqualityExpr {
    $$ = $1;
  }

  | _AndExpr TOKEN_OPERATOR_AND _EqualityExpr {
    $$ = new FunctionAnd(packageArgs($1, $3));
  }
;


_EqualityExpr:

  /*
            EqualityExpr
                               ::=
                                 RelationalExpr
                                 | EqualityExpr '=' RelationalExpr
                                 | EqualityExpr '!=' RelationalExpr
  */


  _RelationalExpr {
    $$ = $1;
  }

  | _EqualityExpr TOKEN_EQUALS _RelationalExpr {
    $$ = new FunctionOpEqualsEquals(packageArgs($1, $3));
  }

  | _EqualityExpr TOKEN_NOT_EQUALS _RelationalExpr {
    $$ = new FunctionOpNotEquals(packageArgs($1, $3));
  }

;

_RelationalExpr: /*~*/

  /*
                   RelationalExpr
                               ::=
                                AdditiveExpr
                                | RelationalExpr ('<' | OpLT)  AdditiveExpr
                                | RelationalExpr ('>' | OpGT) AdditiveExpr
                                | RelationalExpr ('<=' | OpLTE) AdditiveExpr
                                | RelationalExpr ('>=' | OpGTE) AdditiveExpr

  */


  _AdditiveExpr {
         $$ = $1;
  }

  | _RelationalExpr TOKEN_OPERATOR_LT _AdditiveExpr {
    $$ = new FunctionOpLessThan(packageArgs($1, $3));
  }  ///Extra

  | _RelationalExpr TOKEN_LT _AdditiveExpr {
    $$ = new FunctionOpLessThan(packageArgs($1, $3));
  }

  | _RelationalExpr TOKEN_OPERATOR_GT _AdditiveExpr {
    $$ = new FunctionOpGreaterThan(packageArgs($1, $3));
  } ///Extra

  | _RelationalExpr TOKEN_GT _AdditiveExpr {
    $$ = new FunctionOpGreaterThan(packageArgs($1, $3));
  }

  | _RelationalExpr TOKEN_OPERATOR_LTE _AdditiveExpr {
    $$ = new FunctionOpLessThanEqual(packageArgs($1, $3));
  } ///Extra

  | _RelationalExpr TOKEN_LTE _AdditiveExpr {
    $$ = new FunctionOpLessThanEqual(packageArgs($1, $3));
  }

  | _RelationalExpr TOKEN_OPERATOR_GTE _AdditiveExpr {
    $$ = new FunctionOpGreaterThanEqual(packageArgs($1, $3));
  } ///Extra

  | _RelationalExpr TOKEN_GTE _AdditiveExpr {
    $$ = new FunctionOpGreaterThanEqual(packageArgs($1, $3));
  }

;

_AdditiveExpr:

  /*
          AdditiveExpr
                                ::=
                                    MultiplicativeExpr
                                    | AdditiveExpr '+' MultiplicativeExpr
                                    | AdditiveExpr '-' MultiplicativeExpr
        */



  _MultiplicativeExpr {
    $$ = $1;
  }

  | _AdditiveExpr TOKEN_PLUS _MultiplicativeExpr {
    $$ = new FunctionOpPlus(packageArgs($1, $3));
  }

  | _AdditiveExpr TOKEN_MINUS _MultiplicativeExpr {
    $$ = new FunctionOpMinus(packageArgs($1, $3));
  }
;

_MultiplicativeExpr:

  /*
    MultiplicativeExpr
                                ::=
                                    UnaryExpr
                                    | MultiplicativeExpr MultiplyOperator UnaryExpr
                                    | MultiplicativeExpr 'div' UnaryExpr
                                    | MultiplicativeExpr 'mod' UnaryExpr
  */


  _UnaryExpr {
    $$ = $1;
  }

  | _MultiplicativeExpr TOKEN_STAR _UnaryExpr {
    $$ = new FunctionOpTimes(packageArgs($1, $3));
  }

  | _MultiplicativeExpr TOKEN_OPERATOR_DIV _UnaryExpr {
    $$ = new FunctionOpDivide(packageArgs($1, $3));
  }

  | _MultiplicativeExpr TOKEN_OPERATOR_MOD _UnaryExpr {
    $$ = new FunctionOpMod(packageArgs($1, $3));
  }
;

_UnaryExpr:

  /*
          UnaryExpr
                                ::=
                                    UnionExpr
                                    | '-' UnaryExpr
  */


  _UnionExpr {
    $$ = $1;
  }

  | TOKEN_MINUS _UnaryExpr {
          vector<const DataItem*> args;
          args.push_back($2);
          $$ = new FunctionUnaryMinus(args);
  }
;


_UnionExpr:

  /*
          UnionExpr
                        ::=
                            PathExpr
                            | UnionExpr '|' PathExpr

  */


  _PathExpr {
    $$ = $1;
  }

  | _UnionExpr TOKEN_BAR _PathExpr {
    $$ = new FunctionUnion(packageArgs($1, $3));
  }
;

_PathExpr:

  /*
          PathExpr
                        ::=
                            LocationPath
                            | FilterExpr
                            | FilterExpr '/' RelativeLocationPath
                            | FilterExpr '//' RelativeLocationPath
  */


  _LocationPath {
    $$ = $1;
  }

  | _FilterExpr {
    $$ = $1;
  }

  | _FilterExpr TOKEN_SLASH _RelativeLocationPath {
    //Set _FilterExpr as the item which produces the initial nodelist
    //from which to navigate
    DataItemNav *nav = new DataItemNav($1);
    nav->addNavSteps($3);
    $$ = nav;
    delete $3;
  }

  | _FilterExpr TOKEN_SLASH_SLASH _RelativeLocationPath {

    //Set _FilterExpr as the item which produces the initial nodelist off which to navigate
    //Additionally add a new DOS axis step at the _beginning_ of _RelativeLocationPath
    DescendantOrSelfAxis *step = new DescendantOrSelfAxis();
    step->setTypeWildcard();
    step->setNameWildcard();
    step->setNamespaceWildcard();

    DataItemNav *nav = new DataItemNav($1);
    nav->addStep(step);
    nav->addNavSteps($3);

    $$ = nav;
    delete $3;
  }
;

_FilterExpr:

  /*
    FilterExpr
                        ::=
                            PrimaryExpr
                            | FilterExpr Predicate
  */



  _PrimaryExpr {
    $$ = $1;
  }


  | _FilterExpr _Predicate {   ////Bug fix * 2

    //Create new navigation for child axis predicates

    NavStep *step = new NavStep();
    step->addPredicate($2);
    step->setTypeWildcard();
    step->setNameWildcard();
    step->setNamespaceWildcard();


    DataItemNav *nav = new DataItemNav($1);
    nav->addStep(step);
    $$ = nav;
  }

;




_PrimaryExpr:

  /*
    PrimaryExpr
                          ::=
                             VariableReference
                             | '(' Expr ')'
                             | Literal
                             | Number
                             | FunctionCall

  */


  _VariableReference {
    $$ = $1;
  }

  | TOKEN_LEFT_BRACE _Expr TOKEN_RIGHT_BRACE {
    $$ = $2;
  }

  | _Literal {
    DataItemString *str_val  = new DataItemString($1);
    $$ = str_val;
  }

  | _Number {
    $$ = $1;
  }

  | _FunctionCall {
    $$ = $1;
  }

;


_VariableReference: /*~*/

  /*
    VariableReference
                                ::=
                                   '$' QName
  */


  TOKEN_DOLLAR _NCName {
    DataItemVariable *var = new DataItemVariable(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""), $2);
    $$ = var;
  }

  | TOKEN_DOLLAR _NCName TOKEN_COLON _NCName {  ///Is this extra? [36]
    DataItemVariable *var = new DataItemVariable($2, $4);
    $$ = var;
  }
;


_NCName:

  /*
          NCName
   */


  TOKEN_NCNAME {
    $$ = $1;
  }



;

_Literal:

  /*
            Literal
                                ::=
                                   '"' [^"]* '"'
                                   | "'" [^']* "'"
  */



  TOKEN_LITERAL {
    $$ = $1;
  }

;

_Number:

  /*
          Number
                                ::=
                                   Digits ('.' Digits?)?
  */

  TOKEN_NUMERAL {
    DataItemNumeric *num = new DataItemNumeric($1);
    $$ = num;
  }

;


_FunctionCall:

  /*
          FunctionCall
                          ::=
                              FunctionName '(' ( Argument ( ',' Argument )* )? ')'

  */

  _FunctionName TOKEN_LEFT_BRACE TOKEN_RIGHT_BRACE {
    $$ = ((XPathParserControl *)parm)->flook
      ->lookUpFunction($1, vector<const DataItem*>());
    delete[] $1;
  }

  | _FunctionName TOKEN_LEFT_BRACE _ArgumentList TOKEN_RIGHT_BRACE {
    $$ = ((XPathParserControl *)parm)->flook
      ->lookUpFunction($1, $3->getArgs());
    delete[] $1;
    delete $3;
  }

;


_FunctionName:

  /*
    FunctionName
                                ::=
                                   QName - NodeType
  */



  _NCName {
    $$ = $1;
  }
;

_ArgumentList:

  /*
    ArgumentList
       ::= Argument | ',' Argument | ',' Argument ArgumentList

  */

  _Expr {
//    DataItemFunction *dataFunc = new DataItemFunction("");
    ArgHolder *argHolder = new ArgHolder();
    argHolder->addArg($1);
    $$ = argHolder;
    //delete $1; //I make a dataitem out of dataitemimpl and hence I don't want to delete it
    }

  | _ArgumentList TOKEN_COMMA _Expr  {
    static_cast<ArgHolder*>($1)->addArg($3);
    //delete $3;  //likewise
    $$ = $1;
  }

;




_LocationPath: /*~*/

  /*
    LocationPath
                                   ::=
                                      ContextSpecifier RelativeLocationPath
                                      | ContextSpecifier AbsoluteLocationPath

  */


  _ContextSpecifier _RelativeLocationPath {  ///Extra [1]
    static_cast<DataItemNav*>($2)->setInitialContext(static_cast<DataItemNav*>($1)->getInitialContext());
    delete $1;
    $$ = $2;
  }

  | _ContextSpecifier _AbsoluteLocationPath {  //Extra
    static_cast<DataItemNav*>($2)->setInitialContext(static_cast<DataItemNav*>($1)->getInitialContext());
    delete $1;
    $$ = $2;
  }


  | _RelativeLocationPath {
    $$ = $1;
  }

  | _AbsoluteLocationPath {
    $$ = $1;
  }
;


_ContextSpecifier: /*~*/    ///Extra

  /*
          ContextSpecifier
                                  ::=
                                    ('$$' | '%' | '^')?
  */


   TOKEN_DOLLAR_DOLLAR {
    DataItemNav *nav = new DataItemNav();
    nav->setInitialContext(INSTRUCTION);
    $$ = nav;
  }

  | TOKEN_PERCENT {
    DataItemNav *nav = new DataItemNav();
    nav->setInitialContext(DATA);
    $$ = nav;
  }

  | TOKEN_HAT {
    DataItemNav *nav = new DataItemNav();
    //    nav->setGotoOldContextFirst(OLD);
    nav->setInitialContext(OLD);
    $$ = nav;
  }


;

_AbsoluteLocationPath:


  /*
           AbsoluteLocationPath
                                   ::=
                                      '/' RelativeLocationPath?
              | AbbreviatedAbsoluteLocationPath
  */


  TOKEN_SLASH {
    DataItemNav *nav = new DataItemNav();
    nav->setGotoRootFirst(true);
    $$ = nav;
  }

  | TOKEN_SLASH _RelativeLocationPath {
    static_cast<DataItemNav*>($2)->setGotoRootFirst(true);
    $$ = $2;
  }

  | _AbbreviatedAbsoluteLocationPath {
    $$ = $1;
  }


;


_AbbreviatedAbsoluteLocationPath:

  /*
    AbbreviatedAbsoluteLocationPath
      ::=
        '//' RelativeLocationPath


  */

  TOKEN_SLASH_SLASH _RelativeLocationPath {

    DescendantOrSelfAxis *step = new DescendantOrSelfAxis();
    step->setTypeWildcard();
    step->setNameWildcard();
    step->setNamespaceWildcard();

    DataItemNav *newNavigation = new DataItemNav();

    newNavigation->addStep(step);
    newNavigation->addNavSteps($2);
    newNavigation->setGotoRootFirst(true);

    delete $2;
    $$ = newNavigation;
  }

;


_RelativeLocationPath:

  /*
    RelativeLocationPath
                                   ::=
                                      Step
                                      | RelativeLocationPath '/' Step
                                      | AbbreviatedRelativeLocationPath
  */



  _Step {
    DataItemNav *nav = new DataItemNav();
    nav->addStep($1);
    $$ = nav;
  }

  | _RelativeLocationPath TOKEN_SLASH _Step {
    static_cast<DataItemNav*>($1)->addStep($3);
    $$ = $1;
  }

  | _AbbreviatedRelativeLocationPath {
    $$ = $1;

  }
;


_AbbreviatedRelativeLocationPath:


  /*
            AbbreviatedRelativeLocationPath
                                                 ::=
                                                     RelativeLocationPath '//' Step
  */

  _RelativeLocationPath TOKEN_SLASH_SLASH _Step {
    DescendantOrSelfAxis *step = new DescendantOrSelfAxis();
    step->setTypeWildcard();
    step->setNameWildcard();
    step->setNamespaceWildcard();
    static_cast<DataItemNav*>($1)->addStep(step);
    static_cast<DataItemNav*>($1)->addStep($3);
    $$ = $1;
  }

;

_Step: /*~*/

  /*
    Step
                                   ::=
                                     AxisSpecifier NodeTest Predicate*
                                     | AbbreviatedStep
  */

  /* Allow the _NodeTest specifier (suffix) to override the default nodetype for the axis */
  _AxisSpecifier _NodeTest {
          importStep($1, $2);

    if($2->getTypeSet()) {
      $1->setNodeType($2->getNodeType());
    }

    $$ = $1;
    delete $2;
  }

  | _AxisSpecifier _NodeTest _PredicateList {
      importStep($1, $2);

    if($2->getTypeSet()) {
      $1->setNodeType($2->getNodeType());
    }

    $1->addPredicates(*($3));
    $$ = $1;
    delete $3;
    delete $2;
  }

  | _NodeTest {  ///Extra [4]
          ChildAxis *step = new ChildAxis();
    importStep(step, $1);

    if($1->getTypeSet()) {
      step->setNodeType($1->getNodeType());
    }
    else if(!($1->getTypeSet() || step->getTypeWildcard())) {
      step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    }

    delete $1;
    $$ = step;
  }

  | _NodeTest _PredicateList {  ///Extra

          ChildAxis *step = new ChildAxis();
    importStep(step, $1);

    if($1->getTypeSet()) {
      step->setNodeType($1->getNodeType());
    }
    else if(!($1->getTypeSet() || step->getTypeWildcard())) {
      step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    }

    step->addPredicates(*($2));
    $$ = step;
    delete $2;
    delete $1;
  }

  | _AbbreviatedStep {
    $$ = $1;
  }

;


_AxisSpecifier:

  /*
         AxisSpecifier
                                   ::=
                                     AxisName '::'
                                     | AbbreviatedAxisSpecifier
  */


  _AxisName  {
    $$ = $1;
  }

  | _AbbreviatedAxisSpecifier {
    $$ = $1;
  }

;



_AbbreviatedAxisSpecifier: /* ! */ ////Incorrect? Spec says '@?' [13]

  /*
          AbbreviatedAxisSpecifier
                                                 ::=
                                                     ('!@' | '@' )?
  */

  TOKEN_BANG TOKEN_AT {
          InheritedAttributeAxis *at = new InheritedAttributeAxis;
    at->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE);
    $$ = at;
  }

  | TOKEN_AT {
          AttributeAxis *at = new AttributeAxis;
    at->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE);
    $$ = at;

  }

;


_AbbreviatedStep:

  /*

            AbbreviatedStep
                                                 ::=
                                                     '.'
                                                     | '..'
  */


  TOKEN_DOT {
    SelfAxis *step = new SelfAxis;
    step->setNamespaceWildcard();
    step->setNameWildcard();
    step->setTypeWildcard();
    $$ = step;
  }

  | TOKEN_DOT_DOT {
    ParentAxis *step = new ParentAxis;
    step->setNameWildcard();
    step->setNamespaceWildcard();
    step->setTypeWildcard();
    $$ = step;
  }

;

_PredicateList:


  /*
    PredicateList
                            ::=
                             '[' PredicateExpr ']' |

           PredicateList '[' PredicateExpr ']'

  */


  _Predicate {
    vector<PredicateStep*> *store = new vector<PredicateStep*>;
    store->push_back($1);
    $$ = store;

  }

  | _PredicateList _Predicate {
    $1->push_back($2);
    $$ = $1;
  }


;

_Predicate:

     /*
    Predicate
                          ::=
                             '[' PredicateExpr ']'
     */

     TOKEN_LEFT_SQUARE _Expr TOKEN_RIGHT_SQUARE {
          DataItem *itemCopy = $2;
    PredicateStepExpr *predStep = new PredicateStepExprXPath(itemCopy);
    $$ = predStep;
     }
;

_NodeTest:

  /*
         NodeTest
                     ::=
                         NameTest
                         | NodeType '(' ')'
                         | 'processing-instruction' '(' Literal ')'

  */

  _NameTest {
    $$ = $1;
  }

  | _NodeType  {
    $$ = $1;
  }

  | TOKEN_NODETYPE_PROCESSING_INSTRUCTION  _Literal TOKEN_RIGHT_BRACE {
    NavStep *step = new NavStep();
    step->setNodeName($2);
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE);
    step->setNamespaceWildcard();
    $$ = step;
  }

;


_NodeType:

  /*
          NodeType
                                ::=
                                   'comment'
                                   | 'text'
                                   | 'processing-instruction'
                                   | 'node'
  */


  TOKEN_NODETYPE_COMMENT {
    NavStep *step = new NavStep();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::COMMENT_NODE);
    step->setNameWildcard();
    step->setNamespaceWildcard();
    $$ = step;
  }

  | TOKEN_NODETYPE_TEXT {
    NavStep *step = new NavStep();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE);
    step->setNameWildcard();
    step->setNamespaceWildcard();
    $$ = step;
  }

  | TOKEN_NODETYPE_PROCESSING_INSTRUCTION  TOKEN_RIGHT_BRACE{
    NavStep *step = new NavStep();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::PROCESSING_INSTRUCTION_NODE);
    step->setNameWildcard();
    step->setNamespaceWildcard();
    $$ = step;
  }

  | TOKEN_NODETYPE_NODE {
    NavStep *step = new NavStep();
    step->setTypeWildcard();
    step->setNameWildcard();
    step->setNamespaceWildcard();
    $$ = step;
  }

;


_NameTest: /*~*/

  /*
            NameTest
                                ::=
                                   '*'
                                   | NCName ':' '*'
                                   | QName
  */

  TOKEN_STAR {
    NavStep *step = new NavStep();
    step->setNameWildcard();
    step->setNamespaceWildcard();
    $$ = step;
  }

  | _NCName TOKEN_COLON TOKEN_STAR {
    NavStep *step = new NavStep();
    step->setNameWildcard();
    step->setNodePrefix($1);
    $$ = step;
  }

  | _NCName TOKEN_COLON _NCName {
    NavStep *step = new NavStep();
    step->setNodeName($3);
    step->setNodePrefix($1);
    $$ = step;
  }

  | _NCName {     ///Extra? [37]
    NavStep *step = new NavStep();
    step->setNodeName($1);
    $$ = step;
  }

;




_AxisName: /*~*/

  /*

         AxisName
                      ::=
                          'ancestor'
                          | 'ancestor-or-self'
                          | 'attribute'
        | 'inherited-attribute'
                          | 'child'
                          | 'descendant'
                          | 'descendant-or-self'
                          | 'following'
                          | 'following-sibling'
                          | 'namespace'
                          | 'parent'
                          | 'preceding'
                          | 'preceding-sibling'
                          | 'self'

  */



  TOKEN_AXISNAME_ANCESTOR {
      AncestorAxis *step = new AncestorAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_ANCESTOR_OR_SELF {
      AncestorOrSelfAxis *step = new AncestorOrSelfAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_ATTRIBUTE {
      AttributeAxis *step = new AttributeAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_INHERITED_ATTRIBUTE {   ///Extra [6]
    InheritedAttributeAxis *step = new InheritedAttributeAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_CHILD {
    ChildAxis *step = new ChildAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_DESCENDANT {
    DescendantAxis *step = new DescendantAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_DESCENDANT_OR_SELF {
    DescendantOrSelfAxis *step = new DescendantOrSelfAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_FOLLOWING {
    FollowingAxis *step = new FollowingAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_FOLLOWING_SIBLING {
    FollowingSiblingAxis *step = new FollowingSiblingAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_NAMESPACE {
    NamespaceAxis *step = new NamespaceAxis();
    /*step->setNodeType(DOM_Node::XPATH_NAMESPACE_NODE);*/
    step->setNodeType(XPATH_NAMESPACE_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_PARENT {
    ParentAxis *step = new ParentAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_PRECEDING {
    PrecedingAxis *step = new PrecedingAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_PRECEDING_SIBLING {
    PrecedingSiblingAxis *step = new PrecedingSiblingAxis();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

  | TOKEN_AXISNAME_SELF {
    SelfAxis *step = new SelfAxis();
    step->setNamespaceWildcard();
    step->setNodeType(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE);
    $$ = step;
  }

;



%%

void yyerror(const char *s)
{
  throw XPathParseException(s);
}

inline vector<const DataItem*> packageArgs(DataItem *arg1, DataItem *arg2)
{
  vector<const DataItem*> args;
  args.push_back(arg1);
  args.push_back(arg2);

  return args;
}

//imports the data in step 2 into step1
void importStep(NavStep *step1, NavStep *step2)
{

  const XMLCh *prefix = step2->getNodePrefix();
  XMLCh *clonedPrefix = new XMLCh[XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(prefix)+1];
  XERCES_CPP_NAMESPACE_QUALIFIER XMLString::copyString(clonedPrefix,prefix);
  step1->setNodePrefix(clonedPrefix);

  const XMLCh *name = step2->getNodeName();
  XMLCh *clonedName = new XMLCh[XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(name)+1];
  XERCES_CPP_NAMESPACE_QUALIFIER XMLString::copyString(clonedName,name);
  step1->setNodeName(clonedName);

  if(step2->getNameWildcard()) {
    step1->setNameWildcard();
  }
  if(step2->getNamespaceWildcard()) {
    step1->setNamespaceWildcard();
  }
  if(step2->getTypeWildcard()) {
    step1->setTypeWildcard();
  }

}
