/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "TestAllFunctions.h"
#include "TestCaller.h"
#include "../functionAPI/FunctionConcat.hpp"
#include "../functionAPI/FunctionOr.hpp"
#include "../functionAPI/FunctionAnd.hpp"
#include "../functionAPI/FunctionOpPlus.hpp"
#include "../functionAPI/FunctionOpMinus.hpp"
#include "../functionAPI/FunctionOpTimes.hpp"
#include "../functionAPI/FunctionOpDivide.hpp"
#include "../functionAPI/FunctionOpMod.hpp"
#include "../functionAPI/FunctionOpEqualsEquals.hpp"
#include "../functionAPI/FunctionOpNotEquals.hpp"
#include "../functionAPI/FunctionOpLessThan.hpp"
#include "../functionAPI/FunctionOpGreaterThan.hpp"
#include "../functionAPI/FunctionOpLessThanEqual.hpp"
#include "../functionAPI/FunctionOpGreaterThanEqual.hpp"
#include "../functionAPI/FunctionTag.hpp"
#include "../functionAPI/FunctionBoolean.hpp"
#include "../functionAPI/FunctionCount.hpp"
#include "../functionAPI/FunctionStrlen.hpp"
#include "../functionAPI/FunctionSubstr.hpp"
#include "../functionAPI/FunctionNumber.hpp"
#include "../functionAPI/FunctionNot.hpp"
#include "../functionAPI/FunctionNamespace.hpp"
#include "../functionAPI/FunctionLocalname.hpp"
#include "../functionAPI/FunctionSumNodes.hpp"
#include "../functionAPI/FunctionUnaryMinus.hpp"
#include "../functionAPI/FunctionStartsWith.hpp"
#include "../functionAPI/FunctionId.hpp"
#include "../functionAPI/FunctionContains.hpp"
#include "../functionAPI/FunctionSubstringBefore.hpp"
#include "../functionAPI/FunctionSubstringAfter.hpp"
#include "../functionAPI/FunctionLang.hpp"
#include "../functionAPI/FunctionNormalizeSpace.hpp"
#include "../functionAPI/FunctionTranslate.hpp"
#include "../functionAPI/FunctionFloor.hpp"
#include "../functionAPI/FunctionCeiling.hpp"
#include "../functionAPI/FunctionRound.hpp"
#include "../functionAPI/FunctionUnion.hpp"

TestAllFunctions::TestAllFunctions(string name)
  : TestCase(name)
{
  // Nothing to do
}

TestAllFunctions::~TestAllFunctions()
{
  //Nothing to do
}


void TestAllFunctions::setUp()
{
  // Nothing to do
}

Test* TestAllFunctions::suite()
{
  TestSuite* testSuite = new TestSuite("TestAllFunctions");

  testSuite->addTest(new TestCaller<FunctionConcat> ("execute mid-range",
				    &FunctionConcat::testExecute));
  testSuite->addTest(new TestCaller<FunctionConcat> ("execute null-string",
                                    &FunctionConcat::testNullString));

  testSuite->addTest(new TestCaller<FunctionOr> ("execute mid-range",
		  		    &FunctionOr::testExecute));

  testSuite->addTest(new TestCaller<FunctionAnd> ("execute mid-range",
				    &FunctionAnd::testExecute));

  testSuite->addTest(new TestCaller<FunctionOpPlus>("check plus",
				    &FunctionOpPlus::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpMinus>("check minus",
				    &FunctionOpMinus::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpTimes>("check times",
				    &FunctionOpTimes::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpDivide>("check divide",
				    &FunctionOpDivide::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpMod>("check mod",
				    &FunctionOpMod::testExecute));

  testSuite->addTest(new TestCaller<FunctionOpEqualsEquals>("check equals",
				    &FunctionOpEqualsEquals::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpNotEquals>("check not-equals",
				    &FunctionOpNotEquals::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpLessThan>("check less than",
				    &FunctionOpLessThan::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpGreaterThan>("check greater",
				    &FunctionOpGreaterThan::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpLessThanEqual>("check less eq",
				    &FunctionOpLessThanEqual::testExecute));
  testSuite->addTest(new TestCaller<FunctionOpGreaterThanEqual>("check greq",
				    &FunctionOpGreaterThanEqual::testExecute));
  testSuite->addTest(new TestCaller<FunctionTag>("check tag",
				    &FunctionTag::testExecute));
  testSuite->addTest(new TestCaller<FunctionBoolean>("check boolean and exists",
				    &FunctionBoolean::testExecute));
  testSuite->addTest(new TestCaller<FunctionCount>("check count",
				    &FunctionCount::testExecute));
  testSuite->addTest(new TestCaller<FunctionStrlen>("check strlen",
				    &FunctionStrlen::testExecute));
  
  testSuite->addTest(new TestCaller<FunctionSubstr>("check substr",
				    &FunctionSubstr::testExecute));
  
  testSuite->addTest(new TestCaller<FunctionNumber>("check number",
				    &FunctionNumber::testExecute));

  testSuite->addTest(new TestCaller<FunctionNot>("check not",
				    &FunctionNot::testExecute));

  testSuite->addTest(new TestCaller<FunctionNamespace> ("test namespace", 
				    &FunctionNamespace::testExecute));

  testSuite->addTest(new TestCaller<FunctionLocalname> ("test localname", 
				    &FunctionLocalname::testExecute));

  //  testSuite->addTest(new TestCaller<TestAllFunctions> ("FunctionLookup", 
  //				    &TestAllFunctions::testFunctionLookup));

  testSuite->addTest(new TestCaller<FunctionSumNodes> ("sumNodes", 
				    &FunctionSumNodes::testExecute));

  testSuite->addTest(new TestCaller<FunctionUnaryMinus> ("unaryMinus", 
				    &FunctionUnaryMinus::testExecute));

  testSuite->addTest(new TestCaller<FunctionStartsWith> ("starts-with", 
				    &FunctionStartsWith::testExecute));
/*
  testSuite->addTest(new TestCaller<FunctionId> ("id", 
				    &FunctionId::testExecute));
*/
  testSuite->addTest(new TestCaller<FunctionContains> ("contains",
			  	    &FunctionContains::testExecute));

  testSuite->addTest(new TestCaller<FunctionSubstringBefore> ("substring-before",
			  	    &FunctionSubstringBefore::testExecute));

  testSuite->addTest(new TestCaller<FunctionSubstringAfter> ("substring-after",
			            &FunctionSubstringAfter::testExecute));

  testSuite->addTest(new TestCaller<FunctionFloor> ("floor",
			  	    &FunctionFloor::testExecute));

  testSuite->addTest(new TestCaller<FunctionCeiling> ("ceiling",
			  	    &FunctionCeiling::testExecute));

  testSuite->addTest(new TestCaller<FunctionRound> ("round",
			  	    &FunctionRound::testExecute));

/*
  testSuite->addTest(new TestCaller<FunctionLang> ("lang",
				    &FunctionLang::testExecute));
*/
  testSuite->addTest(new TestCaller<FunctionNormalizeSpace> ("normalize-space",
			            &FunctionNormalizeSpace::testExecute));

  testSuite->addTest(new TestCaller<FunctionTranslate> ("translate",
			  	    &FunctionTranslate::testExecute));

  testSuite->addTest(new TestCaller<FunctionUnion> ("union",
			  	    &FunctionUnion::testExecute));

  return testSuite;
}

/*void TestAllFunctions::testFunctionLookup()
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMParser xmlparser;
  initDOMParser(xmlparser);

  FunctionLookup funcLookup = initFunctionLookup();  

  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> script(initScriptTree());
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(initDataTree());
  
  DSLPathParserControl *pc = initDSLPathParser();
  
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *whyNode;
  DSLPathContext context(data->getDocumentElement(), data->getDocumentElement(),
			    script->getDocumentElement(), script->getDocumentElement(), 
			    NULL, new VarStoreImpl());
  
  parseExpression(pc, "1 + 2");
  DataItem output = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output.asNum(context) == 3);  
  delete pc->p_MainExpr;

  parseExpression(pc, "3 - 2");
  DataItem output2 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output2.asNum(context) == 1);  
  delete pc->p_MainExpr;

  parseExpression(pc, "9 * 2");
  DataItem output3 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output3.asNum(context) == 18);  
  delete pc->p_MainExpr;

  parseExpression(pc, "9 / 3");
  DataItem output4 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output4.asNum(context) == 3);  
  delete pc->p_MainExpr;

  parseExpression(pc, "1 or 0");
  DataItem output6 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output6.asBool(context) == true);  
  delete pc->p_MainExpr;

  parseExpression(pc, "1 and 0");
  DataItem output7 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output7.asBool(context) == false);  
  delete pc->p_MainExpr;

  parseExpression(pc, "1 == 0");
  DataItem output8 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output8.asBool(context) == false);  
  delete pc->p_MainExpr;

  parseExpression(pc, "1 != 0");
  DataItem output9 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output9.asBool(context) == true);  
  delete pc->p_MainExpr;

  parseExpression(pc, "3 lt 7");
  DataItem output10 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output10.asBool(context) == true);  
  delete pc->p_MainExpr;

  parseExpression(pc, "3 gt 7");
  DataItem output11 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output11.asBool(context) == false);  
  delete pc->p_MainExpr;

  parseExpression(pc, "4 lte 2");
  DataItem output12 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output12.asBool(context) == false);  
  delete pc->p_MainExpr;

  parseExpression(pc, "5 gte 5");
  DataItem output13 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output13.asBool(context) == true);  
  delete pc->p_MainExpr;

  parseExpression(pc, "7 mod 2");
  DataItem output14 = pc->p_MainExpr->generateResult(funcLookup, false);
  assert(output14.asInt(context) == 1);  
  delete pc->p_MainExpr;

  delete pc;
}*/





