/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "TestFunction.hpp"
#include "../simpleVariables/VariableStore.hpp"
#include <xercesc/parsers/XercesDOMParser.hpp>
#include "../DOMutils/XStr.hpp"

TestFunction::TestFunction()
{
  // Nothing to do
}

TestFunction::~TestFunction()
{
  // Nothing to do
}

void TestFunction::initDOM()
{
  try {
    XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
  }
  catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
    cerr << "Exception during Xerces-C initialisation.\n"
         << "Error message:" << endl
	 << v2Utils::DSLtranscode(toCatch.getMessage()) << endl;
    exit(1);
  }
}

void TestFunction::initDOMParser(XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser &xmlparser)
{
  xmlparser.setDoValidation(false);
  xmlparser.setDoNamespaces(true);
  xmlparser.setCreateEntityReferenceNodes(false);
  Context context;
}


XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *TestFunction::parseXML(XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser &xmlparser, char *file)
{
  try {
    xmlparser.parse(file);
  }
    catch (const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& eParse) {
      cerr << "Error during script parsing, with file " << file <<".\n"
	   << "XML parser exception message: "
	   << v2Utils::DSLtranscode(eParse.getMessage()) << endl;
      assert(false);
    }
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *script = xmlparser.getDocument();
  xmlparser.reset();
  return script;
}

FunctionLookup *TestFunction::initFunctionLookup()
{
    FunctionLookup fLook;
    return &fLook;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *TestFunction::initScriptTree()
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *script = factory->createDocument(X("http://www.xmlscript.org/2.0"), X("xs:__script__"), NULL);
  return script;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *TestFunction::initDataTree()
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *data = factory->createDocument(X("http://www.xmlscript.org/2.0"),  X("xs:dd"), NULL);
  return data;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *TestFunction::initOutputTree()
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *out = factory->createDocument(X("http://www.xmlscript.org/2.0"), X("xs:__output__"), NULL);
  return out;
}
